/*!
 * Mercury Editor is a CoffeeScript and jQuery based WYSIWYG editor.  Documentation and other useful information can be
 * found at https://github.com/jejacks0n/mercury
 *
 * Minimum jQuery requirements are 1.7
 *= require_self
 *
 * You can include the Rails jQuery ujs script here to get some nicer behaviors in modals, panels and lightviews when
 * using :remote => true within the contents rendered in them.
 * require jquery_ujs
 *
 * Add any requires for the support libraries that integrate nicely with Mercury Editor.
 * require mercury/support/history
 *
 * Require Mercury Editor itself.
 *= require mercury/mercury
 *
 * Require any localizations you wish to support
 * Example: es.locale, or fr.locale -- regional dialects are in each language file so never en_US for instance.
 * Make sure you enable the localization feature in the configuration.
 * require mercury/locales/swedish_chef.locale
 *
 * Add all requires for plugins that extend or change the behavior of Mercury Editor.
 * require mercury/plugins/save_as_xml/plugin.js
 *
 * Require any files you want to use that either extend, or change the default Mercury behavior.
 * require mercury_overrides
 */
window.Mercury = {

  // # Mercury Configuration
  config: {
    // ## Toolbars
    //
    // This is where you can customize the toolbars by adding or removing buttons, or changing them and their
    // behaviors.  Any top level object put here will create a new toolbar.  Buttons are simply nested inside the
    // toolbars, along with button groups.
    //
    // Some toolbars are custom (the snippets toolbar for instance), and to denote that use _custom: true.  You can then
    // build the toolbar yourself with it's own behavior.
    //
    // Buttons can be grouped, and a button group is simply a way to wrap buttons for styling -- they can also handle
    // enabling or disabling all the buttons within it by using a context.  The table button group is a good example of
    // this.
    //
    // It's important to note that each of the button names (keys), in each toolbar object must be unique, regardless of
    // if it's in a button group, or nested, etc.  This is because styling is applied to them by name, and because their
    // name is used in the event that's fired when you click on them.
    //
    // Button format: `[label, description, {type: action, type: action, etc}]`
    //
    // ### The available button types are:
    //
    // - toggle:    toggles on or off when clicked, otherwise behaves like a button
    // - modal:     opens a modal window, expects the action to be one of:
    //   1. a string url
    //   2. a function that returns a string url
    // - lightview: opens a lightview window (like modal, but different UI), expects the action to be one of:
    //   1. a string url
    //   2. a function that returns a string url
    // - panel:     opens a panel dialog, expects the action to be one of:
    //   1. a string url
    //   2. a function that returns a string url
    // - palette:   opens a palette window, expects the action to be one of:
    //   1. a string url
    //   2. a function that returns a string url
    // - select:    opens a pulldown style window, expects the action to be one of:
    //   1. a string url
    //   2. a function that returns a string url
    // - context:   calls a callback function, expects the action to be:
    //   1. a function that returns a boolean to highlight the button
    //   note: if a function isn't provided, the key will be passed to the contextHandler, in which case a default
    //         context will be used (for more info read the Contexts section below)
    // - mode:      toggle a given mode in the editor, expects the action to be:
    //   1. a string, denoting the name of the mode
    //   note: it's assumed that when a specific mode is turned on, all other modes will be turned off, which happens
    //         automatically, thus putting the editor into a specific "state"
    // - regions:   allows buttons to be enabled/disabled based on what region type has focus, expects:
    //   1. an array of region types (eg. ['full', 'markdown'])
    // - preload:   allows some dialog views to be loaded when the button is created instead of on first open, expects:
    //   1. a boolean true / false
    //   note: this is only used by panels, selects, and palettes
    //
    // Separators are any "button" that's not an array, and are expected to be a string.  You can use two different
    // separator styles: line ('-'), and spacer (' ').
    //
    // ### Adding Contexts
    //
    // Contexts are used callback functions used for highlighting and disabling/enabling buttons and buttongroups.  When
    // the cursor enters an element within an html region for instance we want to disable or highlight buttons based on
    // the properties of the given node.  You can see examples of contexts in, and add your own to:
    // `Mercury.Toolbar.Button.contexts` and `Mercury.Toolbar.ButtonGroup.contexts`
    toolbars: {
      primary: {
        save:                  ['Save', 'Save this page'],
        preview:               ['Preview', 'Preview this page', { toggle: true, mode: true }],
        sep1:                  ' ',
        undoredo:              {
          undo:                ['Undo', 'Undo your last action'],
          redo:                ['Redo', 'Redo your last action'],
          sep:                 ' '
          },
        insertLink:            ['Link', 'Insert Link', { modal: '/mercury/modals/link.html', regions: ['full', 'markdown'] }],
        insertMedia:           ['Media', 'Insert Media (images and videos)', { modal: '/mercury/modals/media.html', regions: ['full', 'markdown'] }],
        insertTable:           ['Table', 'Insert Table', { modal: '/mercury/modals/table.html', regions: ['full', 'markdown'] }],
        insertCharacter:       ['Character', 'Special Characters', { modal: '/mercury/modals/character.html', regions: ['full', 'markdown'] }],
        snippetPanel:          ['Snippet', 'Snippet Panel', { panel: '/mercury/panels/snippets.html' }],
        sep2:                  ' ',
        historyPanel:          ['History', 'Page Version History', { panel: '/mercury/panels/history.html' }],
        sep3:                  ' ',
        notesPanel:            ['Notes', 'Page Notes', { panel: '/mercury/panels/notes.html' }]
        },

      editable: {
        _regions:              ['full', 'markdown'],
        predefined:            {
          style:               ['Style', null, { select: '/mercury/selects/style.html', preload: true }],
          sep1:                ' ',
          formatblock:         ['Block Format', null, { select: '/mercury/selects/formatblock.html', preload: true }],
          sep2:                '-'
          },
        colors:                {
          backColor:           ['Background Color', null, { palette: '/mercury/palettes/backcolor.html', context: true, preload: true, regions: ['full'] }],
          sep1:                ' ',
          foreColor:           ['Text Color', null, { palette: '/mercury/palettes/forecolor.html', context: true, preload: true, regions: ['full'] }],
          sep2:                '-'
          },
        decoration:            {
          bold:                ['Bold', null, { context: true }],
          italic:              ['Italicize', null, { context: true }],
          overline:            ['Overline', null, { context: true, regions: ['full'] }],
          strikethrough:       ['Strikethrough', null, { context: true, regions: ['full'] }],
          underline:           ['Underline', null, { context: true, regions: ['full'] }],
          sep:                 '-'
          },
        script:                {
          subscript:           ['Subscript', null, { context: true }],
          superscript:         ['Superscript', null, { context: true }],
          sep: '-'
          },
        justify:               {
          justifyLeft:         ['Align Left', null, { context: true, regions: ['full'] }],
          justifyCenter:       ['Center', null, { context: true, regions: ['full'] }],
          justifyRight:        ['Align Right', null, { context: true, regions: ['full'] }],
          justifyFull:         ['Justify Full', null, { context: true, regions: ['full'] }],
          sep:                 '-'
          },
        list:                  {
          insertUnorderedList: ['Unordered List', null, { context: true }],
          insertOrderedList:   ['Numbered List', null, { context: true }],
          sep:                 '-'
          },
        indent:                {
          outdent:             ['Decrease Indentation'],
          indent:              ['Increase Indentation'],
          sep:                 '-'
          },
        table:                 {
          _context:            true,
          insertRowBefore:     ['Insert Table Row', 'Insert a table row before the cursor', { regions: ['full'] }],
          insertRowAfter:      ['Insert Table Row', 'Insert a table row after the cursor', { regions: ['full'] }],
          deleteRow:           ['Delete Table Row', 'Delete this table row', { regions: ['full'] }],
          insertColumnBefore:  ['Insert Table Column', 'Insert a table column before the cursor', { regions: ['full'] }],
          insertColumnAfter:   ['Insert Table Column', 'Insert a table column after the cursor', { regions: ['full'] }],
          deleteColumn:        ['Delete Table Column', 'Delete this table column', { regions: ['full'] }],
          sep1:                ' ',
          increaseColspan:     ['Increase Cell Columns', 'Increase the cells colspan'],
          decreaseColspan:     ['Decrease Cell Columns', 'Decrease the cells colspan and add a new cell'],
          increaseRowspan:     ['Increase Cell Rows', 'Increase the cells rowspan'],
          decreaseRowspan:     ['Decrease Cell Rows', 'Decrease the cells rowspan and add a new cell'],
          sep2:                '-'
          },
        rules:                 {
          horizontalRule:      ['Horizontal Rule', 'Insert a horizontal rule'],
          sep1:                '-'
          },
        formatting:            {
          removeFormatting:    ['Remove Formatting', 'Remove formatting for the selection', { regions: ['full'] }],
          sep2:                ' '
          },
        editors:               {
          htmlEditor:          ['Edit HTML', 'Edit the HTML content', { regions: ['full'] }]
          }
        },

      snippets: {
        _custom:               true,
        actions:               {
          editSnippet:         ['Edit Snippet Settings'],
          sep1:                ' ',
          removeSnippet:       ['Remove Snippet']
          }
        }
      },


    // ## Region Options
    //
    // You can customize some aspects of how regions are found, identified, and saved.
    //
    // attribute: Mercury identifies editable regions by a data-mercury attribute.  This attribute has to be added in
    // your HTML in advance, and is the only real code/naming exposed in the implementation of Mercury.  To allow this
    // to be as configurable as possible, you can set the name of this attribute.  If you change this, you should adjust
    // the injected styles as well.
    //
    // identifier: This is used as a unique identifier for any given region (and thus should be unique to the page).
    // By default this is the id attribute but can be changed to a data attribute should you want to use something
    // custom instead.
    //
    // dataAttributes: The dataAttributes is an array of data attributes that will be serialized and returned to the
    // server upon saving.  These attributes, when applied to a Mercury region element, will be automatically serialized
    // and submitted with the AJAX request sent when a page is saved.  These are expected to be HTML5 data attributes,
    // and 'data-' will automatically be prepended to each item in this directive. (ex. ['scope', 'version'])
    //
    // determineType: This function is called after checking the data-type attribute for the correct field type. Use
    // it if you want to dynamically set the type based on inspection of the region.
    regions: {
      attribute: 'data-mercury',
      identifier: 'id',
      dataAttributes: []
      // determineType: function(region){},
      },


    // ## Snippet Options / Preview
    //
    // When a user drags a snippet onto the page they'll be prompted to enter options for the given snippet.  The server
    // is expected to respond with a form.  Once the user submits this form, an Ajax request is sent to the server with
    // the options provided; this preview request is expected to respond with the rendered markup for the snippet.
    //
    // method: The HTTP method used when submitting both the options and the preview.  We use POST by default because a
    // snippet options form may contain large text inputs and we don't want that to be truncated when sent to the
    // server.
    //
    // optionsUrl: The url that the options form will be loaded from.
    //
    // previewUrl: The url that the options will be submitted to, and will return the rendered snippet markup.
    //
    // **Note:** `:name` will be replaced with the snippet name in the urls (eg. /mercury/snippets/example/options.html)
    snippets: {
      method: 'POST',
      optionsUrl: '/mercury/snippets/:name/options.html',
      previewUrl: '/mercury/snippets/:name/preview.html'
      },


    // ## Image Uploading
    //
    // If you drag images from your desktop into regions that support it, it will be uploaded to the server and inserted
    // into the region.  You can disable or enable this feature, the accepted mime-types, file size restrictions, and
    // other things related to uploading.
    //
    // **Note:** Image uploading is only supported in some region types, and some browsers.
    //
    // enabled: You can set this to true, or false if you want to disable the feature entirely.
    //
    // allowedMimeTypes: You can restrict the types of files that can be uploaded by providing a list of allowed mime
    // types.
    //
    // maxFileSize: You can restrict large files by setting the maxFileSize (in bytes).
    //
    // inputName: When uploading, a form is generated and submitted to the server via Ajax.  If your server would prefer
    // a different name for how the image comes through, you can change the inputName.
    //
    // url: The url that the image upload will be submitted to.
    //
    // handler: You can use false to let Mercury handle it for you, or you can provide a handler function that can
    // modify the response from the server.  This can be useful if your server doesn't respond the way Mercury expects.
    // The handler function should take the response from the server and return an object that matches:
    // `{image: {url: '[your provided url]'}`
    uploading: {
      enabled: true,
      allowedMimeTypes: ['image/jpeg', 'image/gif', 'image/png'],
      maxFileSize: 1235242880,
      inputName: 'image[image]',
      url: '/mercury/images',
      handler: false
      },


    // ## Localization / I18n
    //
    // Include the .locale files you want to support when loading Mercury.  The files are always named by the language,
    // and not the regional dialect (eg. en.locale.js) because the regional dialects are nested within the primary
    // locale files.
    //
    // The client locale will be used first, and if no proper locale file is found for their language then the fallback
    // preferredLocale configuration will be used.  If one isn't provided, and the client locale isn't included, the
    // strings will remain untranslated.
    //
    // enabled: Set to false to disable, true to enable.
    //
    // preferredLocale: If a client doesn't support the locales you've included, this is used as a fallback.
    localization: {
      enabled: false,
      preferredLocale: 'swedish_chef-BORK'
      },


    // ## Behaviors
    //
    // Behaviors are used to change the default behaviors of a given region type when a given button is clicked.  For
    // example, you may prefer to add HR tags using an HR wrapped within a div with a classname (for styling).  You
    // can add your own complex behaviors here and they'll be shared across all regions.
    //
    // If you want to add behaviors to specific region types, you can mix them into the actions property of any region
    // type.
    //
    //     Mercury.Regions.Full.actions.htmlEditor = function() {}
    //
    // You can see how the behavior matches up directly with the button names.  It's also important to note that the
    // callback functions are executed within the scope of the given region, so you have access to all it's methods.
    behaviors: {
      //foreColor: function(selection, options) { selection.wrap('<span style="color:' + options.value.toHex() + '">', true) },
      htmlEditor: function() { Mercury.modal('/mercury/modals/htmleditor.html', { title: 'HTML Editor', fullHeight: true, handler: 'htmlEditor' }); }
      },


    // ## Global Behaviors
    //
    // Global behaviors are much like behaviors, but are more "global".  Things like save, exit, etc. can be included
    // here.  They'll only be called once, and execute within the scope of whatever editor is instantiated (eg.
    // PageEditor).
    //
    // An example of changing how saving works:
    //
    //     save: function() {
    //       var data = top.JSON.stringify(this.serialize(), null, '  ');
    //       var content = '<textarea style="width:500px;height:200px" wrap="off">' + data + '</textarea>';
    //       Mercury.modal(null, {title: 'Saving', closeButton: true, content: content})
    //     }
    //
    // This is a nice way to add functionality, when the behaviors aren't region specific.  These can be triggered by a
    // button, or manually with `Mercury.trigger('action', {action: 'barrelRoll'})`
    globalBehaviors: {
      exit: function() { window.location.href = this.iframeSrc() },
      barrelRoll: function() { $('body').css({webkitTransform: 'rotate(360deg)'}) }
      },


    // ## Ajax and CSRF Headers
    //
    // Some server frameworks require that you provide a specific header for Ajax requests.  The values for these CSRF
    // tokens are typically stored in the rendered DOM.  By default, Mercury will look for the Rails specific meta tag,
    // and provide the X-CSRF-Token header on Ajax requests, but you can modify this configuration if the system you're
    // using doesn't follow the same standard.
    csrfSelector: 'meta[name="csrf-token"]',
    csrfHeader: 'X-CSRF-Token',


    // ## Editor URLs
    //
    // When loading a given page, you may want to tweak this regex.  It's to allow the url to differ from the page
    // you're editing, and the url at which you access it.
    editorUrlRegEx: /([http|https]:\/\/.[^\/]*)\/editor\/?(.*)/i,


    // ## Hijacking Links & Forms
    //
    // Mercury will hijack links and forms that don't have a target set, or the target is set to _self and will set it
    // to _parent.  This is because the target must be set properly for Mercury to not get in the way of some
    // functionality, like proper page loads on form submissions etc.  Mercury doesn't do this to links or forms that
    // are within editable regions because it doesn't want to impact the html that's saved.  With that being explained,
    // you can add classes to links or forms that you don't want this behavior added to.  Let's say you have links that
    // open a lightbox style window, and you don't want the targets of these to be set to _parent.  You can add classes
    // to this array, and they will be ignored when the hijacking is applied.
    nonHijackableClasses: [],


    // ## Pasting & Sanitizing
    //
    // When pasting content into Mercury it may sometimes contain HTML tags and attributes.  This markup is used to
    // style the content and makes the pasted content look (and behave) the same as the original content.  This can be a
    // desired feature or an annoyance, so you can enable various sanitizing methods to clean the content when it's
    // pasted.
    //
    // sanitize: Can be any of the following:
    // - false: no sanitizing is done, the content is pasted the exact same as it was copied by the user
    // - 'whitelist': content is cleaned using the settings specified in the tag white list (described below)
    // - 'text': all html is stripped before pasting, leaving only the raw text
    //
    // whitelist: The white list allows you to specify tags and attributes that are allowed when pasting content.  Each
    // item in this object should contain the allowed tag, and an array of attributes that are allowed on that tag.  If
    // the allowed attributes array is empty, all attributes will be removed.  If a tag is not present in this list, it
    // will be removed, but without removing any of the text or tags inside it.
    //
    // **Note:** Content is *always* sanitized if looks like it's from MS Word or similar editors regardless of this
    // configuration.
    pasting: {
      sanitize: 'whitelist',
      whitelist: {
        h1:     [],
        h2:     [],
        h3:     [],
        h4:     [],
        h5:     [],
        h6:     [],
        table:  [],
        thead:  [],
        tbody:  [],
        tfoot:  [],
        tr:     [],
        th:     ['colspan', 'rowspan'],
        td:     ['colspan', 'rowspan'],
        div:    ['class'],
        span:   ['class'],
        ul:     [],
        ol:     [],
        li:     [],
        b:      [],
        strong: [],
        i:      [],
        em:     [],
        u:      [],
        strike: [],
        br:     [],
        p:      [],
        hr:     [],
        a:      ['href', 'target', 'title', 'name'],
        img:    ['src', 'title', 'alt']
        }
      },


    // ## Injected Styles
    //
    // Mercury tries to stay as much out of your code as possible, but because regions appear within your document we
    // need to include a few styles to indicate regions, as well as the different states of them (eg. focused).  These
    // styles are injected into your document, and as simple as they might be, you may want to change them.
    injectedStyles: '' +
      '[data-mercury]       { min-height: 10px; outline: 1px dotted #09F } ' +
      '[data-mercury]:focus { outline: none; -webkit-box-shadow: 0 0 10px #09F, 0 0 1px #045; box-shadow: 0 0 10px #09F, 0 0 1px #045 }' +
      '[data-mercury].focus { outline: none; -webkit-box-shadow: 0 0 10px #09F, 0 0 1px #045; box-shadow: 0 0 10px #09F, 0 0 1px #045 }' +
      '[data-mercury]:after { content: "."; display: block; visibility: hidden; clear: both; height: 0; overflow: hidden; }' +
      '[data-mercury] table { border: 1px dotted red; min-width: 6px; }' +
      '[data-mercury] th    { border: 1px dotted red; min-width: 6px; }' +
      '[data-mercury] td    { border: 1px dotted red; min-width: 6px; }' +
      '[data-mercury] .mercury-textarea       { border: 0; box-sizing: border-box; -moz-box-sizing: border-box; -webkit-box-sizing: border-box; resize: none; }' +
      '[data-mercury] .mercury-textarea:focus { outline: none; -webkit-box-shadow: none; -moz-box-shadow: none; box-shadow: none; }'
  },

  // ## Silent Mode
  //
  // Turning silent mode on will disable asking about unsaved changes before leaving the page.
  silent: false,

  // ## Debug Mode
  //
  // Turning debug mode on will log events and other various things (using console.debug if available).
  debug: false

};
/*!
 * jQuery UI 1.8.13
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI
 */
(function(e,t){function n(t,n){var i=t.nodeName.toLowerCase();return"area"===i?(n=t.parentNode,i=n.name,!t.href||!i||n.nodeName.toLowerCase()!=="map"?!1:(t=e("img[usemap=#"+i+"]")[0],!!t&&r(t))):(/input|select|textarea|button|object/.test(i)?!t.disabled:"a"==i?t.href||n:n)&&r(t)}function r(t){return!e(t).parents().andSelf().filter(function(){return e.curCSS(this,"visibility")==="hidden"||e.expr.filters.hidden(this)}).length}e.ui=e.ui||{},e.ui.version||(e.extend(e.ui,{version:"1.8.13",keyCode:{ALT:18,BACKSPACE:8,CAPS_LOCK:20,COMMA:188,COMMAND:91,COMMAND_LEFT:91,COMMAND_RIGHT:93,CONTROL:17,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,INSERT:45,LEFT:37,MENU:93,NUMPAD_ADD:107,NUMPAD_DECIMAL:110,NUMPAD_DIVIDE:111,NUMPAD_ENTER:108,NUMPAD_MULTIPLY:106,NUMPAD_SUBTRACT:109,PAGE_DOWN:34,PAGE_UP:33,PERIOD:190,RIGHT:39,SHIFT:16,SPACE:32,TAB:9,UP:38,WINDOWS:91}}),e.fn.extend({_focus:e.fn.focus,focus:function(t,n){return typeof t=="number"?this.each(function(){var r=this;setTimeout(function(){e(r).focus(),n&&n.call(r)},t)}):this._focus.apply(this,arguments)},scrollParent:function(){var t;return t=e.browser.msie&&/(static|relative)/.test(this.css("position"))||/absolute/.test(this.css("position"))?this.parents().filter(function(){return/(relative|absolute|fixed)/.test(e.curCSS(this,"position",1))&&/(auto|scroll)/.test(e.curCSS(this,"overflow",1)+e.curCSS(this,"overflow-y",1)+e.curCSS(this,"overflow-x",1))}).eq(0):this.parents().filter(function(){return/(auto|scroll)/.test(e.curCSS(this,"overflow",1)+e.curCSS(this,"overflow-y",1)+e.curCSS(this,"overflow-x",1))}).eq(0),/fixed/.test(this.css("position"))||!t.length?e(document):t},zIndex:function(n){if(n!==t)return this.css("zIndex",n);if(this.length){n=e(this[0]);for(var r;n.length&&n[0]!==document;){r=n.css("position");if(r==="absolute"||r==="relative"||r==="fixed"){r=parseInt(n.css("zIndex"),10);if(!isNaN(r)&&r!==0)return r}n=n.parent()}}return 0},disableSelection:function(){return this.bind((e.support.selectstart?"selectstart":"mousedown")+".ui-disableSelection",function(e){e.preventDefault()})},enableSelection:function(){return this.unbind(".ui-disableSelection")}}),e.each(["Width","Height"],function(n,r){function i(t,n,r,i){return e.each(s,function(){n-=parseFloat(e.curCSS(t,"padding"+this,!0))||0,r&&(n-=parseFloat(e.curCSS(t,"border"+this+"Width",!0))||0),i&&(n-=parseFloat(e.curCSS(t,"margin"+this,!0))||0)}),n}var s=r==="Width"?["Left","Right"]:["Top","Bottom"],o=r.toLowerCase(),u={innerWidth:e.fn.innerWidth,innerHeight:e.fn.innerHeight,outerWidth:e.fn.outerWidth,outerHeight:e.fn.outerHeight};e.fn["inner"+r]=function(n){return n===t?u["inner"+r].call(this):this.each(function(){e(this).css(o,i(this,n)+"px")})},e.fn["outer"+r]=function(t,n){return typeof t!="number"?u["outer"+r].call(this,t):this.each(function(){e(this).css(o,i(this,t,!0,n)+"px")})}}),e.extend(e.expr[":"],{data:function(t,n,r){return!!e.data(t,r[3])},focusable:function(t){return n(t,!isNaN(e.attr(t,"tabindex")))},tabbable:function(t){var r=e.attr(t,"tabindex"),i=isNaN(r);return(i||r>=0)&&n(t,!i)}}),e(function(){var t=document.body,n=t.appendChild(n=document.createElement("div"));e.extend(n.style,{minHeight:"100px",height:"auto",padding:0,borderWidth:0}),e.support.minHeight=n.offsetHeight===100,e.support.selectstart="onselectstart"in n,t.removeChild(n).style.display="none"}),e.extend(e.ui,{plugin:{add:function(t,n,r){t=e.ui[t].prototype;for(var i in r)t.plugins[i]=t.plugins[i]||[],t.plugins[i].push([n,r[i]])},call:function(e,t,n){if((t=e.plugins[t])&&e.element[0].parentNode)for(var r=0;r<t.length;r++)e.options[t[r][0]]&&t[r][1].apply(e.element,n)}},contains:function(e,t){return document.compareDocumentPosition?e.compareDocumentPosition(t)&16:e!==t&&e.contains(t)},hasScroll:function(t,n){if(e(t).css("overflow")==="hidden")return!1;n=n&&n==="left"?"scrollLeft":"scrollTop";var r=!1;return t[n]>0?!0:(t[n]=1,r=t[n]>0,t[n]=0,r)},isOverAxis:function(e,t,n){return e>t&&e<t+n},isOver:function(t,n,r,i,s,o){return e.ui.isOverAxis(t,r,s)&&e.ui.isOverAxis(n,i,o)}}))})(jQuery),function(e,t){if(e.cleanData){var n=e.cleanData;e.cleanData=function(t){for(var r=0,i;(i=t[r])!=null;r++)e(i).triggerHandler("remove");n(t)}}else{var r=e.fn.remove;e.fn.remove=function(t,n){return this.each(function(){return n||(!t||e.filter(t,[this]).length)&&e("*",this).add([this]).each(function(){e(this).triggerHandler("remove")}),r.call(e(this),t,n)})}}e.widget=function(t,n,r){var i=t.split(".")[0],s;t=t.split(".")[1],s=i+"-"+t,r||(r=n,n=e.Widget),e.expr[":"][s]=function(n){return!!e.data(n,t)},e[i]=e[i]||{},e[i][t]=function(e,t){arguments.length&&this._createWidget(e,t)},n=new n,n.options=e.extend(!0,{},n.options),e[i][t].prototype=e.extend(!0,n,{namespace:i,widgetName:t,widgetEventPrefix:e[i][t].prototype.widgetEventPrefix||t,widgetBaseClass:s},r),e.widget.bridge(t,e[i][t])},e.widget.bridge=function(n,r){e.fn[n]=function(i){var s=typeof i=="string",o=Array.prototype.slice.call(arguments,1),u=this;return i=!s&&o.length?e.extend.apply(null,[!0,i].concat(o)):i,s&&i.charAt(0)==="_"?u:(s?this.each(function(){var r=e.data(this,n),s=r&&e.isFunction(r[i])?r[i].apply(r,o):r;if(s!==r&&s!==t)return u=s,!1}):this.each(function(){var t=e.data(this,n);t?t.option(i||{})._init():e.data(this,n,new r(i,this))}),u)}},e.Widget=function(e,t){arguments.length&&this._createWidget(e,t)},e.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",options:{disabled:!1},_createWidget:function(t,n){e.data(n,this.widgetName,this),this.element=e(n),this.options=e.extend(!0,{},this.options,this._getCreateOptions(),t);var r=this;this.element.bind("remove."+this.widgetName,function(){r.destroy()}),this._create(),this._trigger("create"),this._init()},_getCreateOptions:function(){return e.metadata&&e.metadata.get(this.element[0])[this.widgetName]},_create:function(){},_init:function(){},destroy:function(){this.element.unbind("."+this.widgetName).removeData(this.widgetName),this.widget().unbind("."+this.widgetName).removeAttr("aria-disabled").removeClass(this.widgetBaseClass+"-disabled ui-state-disabled")},widget:function(){return this.element},option:function(n,r){var i=n;if(arguments.length===0)return e.extend({},this.options);if(typeof n=="string"){if(r===t)return this.options[n];i={},i[n]=r}return this._setOptions(i),this},_setOptions:function(t){var n=this;return e.each(t,function(e,t){n._setOption(e,t)}),this},_setOption:function(e,t){return this.options[e]=t,e==="disabled"&&this.widget()[t?"addClass":"removeClass"](this.widgetBaseClass+"-disabled ui-state-disabled").attr("aria-disabled",t),this},enable:function(){return this._setOption("disabled",!1)},disable:function(){return this._setOption("disabled",!0)},_trigger:function(t,n,r){var i=this.options[t];n=e.Event(n),n.type=(t===this.widgetEventPrefix?t:this.widgetEventPrefix+t).toLowerCase(),r=r||{};if(n.originalEvent){t=e.event.props.length;for(var s;t;)s=e.event.props[--t],n[s]=n.originalEvent[s]}return this.element.trigger(n,r),!(e.isFunction(i)&&i.call(this.element[0],n,r)===!1||n.isDefaultPrevented())}}}(jQuery),function(e,t){var n=!1;e(document).mousedown(function(e){n=!1}),e.widget("ui.mouse",{options:{cancel:":input,option",distance:1,delay:0},_mouseInit:function(){var t=this;this.element.bind("mousedown."+this.widgetName,function(e){return t._mouseDown(e)}).bind("click."+this.widgetName,function(n){if(!0===e.data(n.target,t.widgetName+".preventClickEvent"))return e.removeData(n.target,t.widgetName+".preventClickEvent"),n.stopImmediatePropagation(),!1}),this.started=!1},_mouseDestroy:function(){this.element.unbind("."+this.widgetName)},_mouseDown:function(t){e(this.document||document).mousedown(function(e){n=!1});if(n)return;this._mouseStarted&&this._mouseUp(t),this._mouseDownEvent=t;var r=this,i=t.which==1,s=typeof this.options.cancel=="string"?e(t.target).parents().add(t.target).filter(this.options.cancel).length:!1;if(!i||s||!this._mouseCapture(t))return!0;this.mouseDelayMet=!this.options.delay,this.mouseDelayMet||(this._mouseDelayTimer=setTimeout(function(){r.mouseDelayMet=!0},this.options.delay));if(this._mouseDistanceMet(t)&&this._mouseDelayMet(t)){this._mouseStarted=this._mouseStart(t)!==!1;if(!this._mouseStarted)return t.preventDefault(),!0}return!0===e.data(t.target,this.widgetName+".preventClickEvent")&&e.removeData(t.target,this.widgetName+".preventClickEvent"),this._mouseMoveDelegate=function(e){return r._mouseMove(e)},this._mouseUpDelegate=function(e){return r._mouseUp(e)},e(this.document||document).bind("mousemove."+this.widgetName,this._mouseMoveDelegate).bind("mouseup."+this.widgetName,this._mouseUpDelegate),t.preventDefault(),n=!0,!0},_mouseMove:function(t){return!e.browser.msie||(this.document||document).documentMode>=9||!!t.button?this._mouseStarted?(this._mouseDrag(t),t.preventDefault()):(this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=this._mouseStart(this._mouseDownEvent,t)!==!1,this._mouseStarted?this._mouseDrag(t):this._mouseUp(t)),!this._mouseStarted):this._mouseUp(t)},_mouseUp:function(t){return e(this.document||document).unbind("mousemove."+this.widgetName,this._mouseMoveDelegate).unbind("mouseup."+this.widgetName,this._mouseUpDelegate),this._mouseStarted&&(this._mouseStarted=!1,t.target==this._mouseDownEvent.target&&e.data(t.target,this.widgetName+".preventClickEvent",!0),this._mouseStop(t)),!1},_mouseDistanceMet:function(e){return Math.max(Math.abs(this._mouseDownEvent.pageX-e.pageX),Math.abs(this._mouseDownEvent.pageY-e.pageY))>=this.options.distance},_mouseDelayMet:function(e){return this.mouseDelayMet},_mouseStart:function(e){},_mouseDrag:function(e){},_mouseStop:function(e){},_mouseCapture:function(e){return!0}})}(jQuery),function(e){e.widget("ui.draggable",e.ui.mouse,{widgetEventPrefix:"drag",options:{addClasses:!0,appendTo:"parent",axis:!1,connectToSortable:!1,containment:!1,cursor:"auto",cursorAt:!1,grid:!1,handle:!1,helper:"original",iframeFix:!1,opacity:!1,refreshPositions:!1,revert:!1,revertDuration:500,scope:"default",scroll:!0,scrollSensitivity:20,scrollSpeed:20,snap:!1,snapMode:"both",snapTolerance:20,stack:!1,zIndex:!1},_create:function(){this.options.helper=="original"&&!/^(?:r|a|f)/.test(this.element.css("position"))&&(this.element[0].style.position="relative"),this.options.addClasses&&this.element.addClass("ui-draggable"),this.options.disabled&&this.element.addClass("ui-draggable-disabled"),this._mouseInit()},destroy:function(){if(this.element.data("draggable"))return this.element.removeData("draggable").unbind(".draggable").removeClass("ui-draggable ui-draggable-dragging ui-draggable-disabled"),this._mouseDestroy(),this},_mouseCapture:function(t){var n=this.options;return this.helper||n.disabled||e(t.target).is(".ui-resizable-handle")?!1:(this.handle=this._getHandle(t),this.handle?(e(n.iframeFix===!0?"iframe":n.iframeFix).each(function(){e('<div class="ui-draggable-iframeFix" style="background: #fff;"></div>').css({width:this.offsetWidth+"px",height:this.offsetHeight+"px",position:"absolute",opacity:"0.001",zIndex:1e3}).css(e(this).offset()).appendTo("body")}),!0):!1)},_mouseStart:function(t){var n=this.options;return this.helper=this._createHelper(t),this._cacheHelperProportions(),e.ui.ddmanager&&(e.ui.ddmanager.current=this),this._cacheMargins(),this.cssPosition=this.helper.css("position"),this.scrollParent=this.helper.scrollParent(),this.offset=this.positionAbs=this.element.offset(),this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left},e.extend(this.offset,{click:{left:t.pageX-this.offset.left,top:t.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()}),this.originalPosition=this.position=this._generatePosition(t),this.originalPageX=t.pageX,this.originalPageY=t.pageY,n.cursorAt&&this._adjustOffsetFromHelper(n.cursorAt),n.containment&&this._setContainment(),this._trigger("start",t)===!1?(this._clear(),!1):(this._cacheHelperProportions(),e.ui.ddmanager&&!n.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t),this.helper.addClass("ui-draggable-dragging"),this._mouseDrag(t,!0),!0)},_mouseDrag:function(t,n){this.position=this._generatePosition(t),this.positionAbs=this._convertPositionTo("absolute");if(!n){n=this._uiHash();if(this._trigger("drag",t,n)===!1)return this._mouseUp({}),!1;this.position=n.position}if(!this.options.axis||this.options.axis!="y")this.helper[0].style.left=this.position.left+"px";if(!this.options.axis||this.options.axis!="x")this.helper[0].style.top=this.position.top+"px";return e.ui.ddmanager&&e.ui.ddmanager.drag(this,t),!1},_mouseStop:function(t){var n=!1;e.ui.ddmanager&&!this.options.dropBehaviour&&(n=e.ui.ddmanager.drop(this,t)),this.dropped&&(n=this.dropped,this.dropped=!1);if((!this.element[0]||!this.element[0].parentNode)&&this.options.helper=="original")return!1;if(this.options.revert=="invalid"&&!n||this.options.revert=="valid"&&n||this.options.revert===!0||e.isFunction(this.options.revert)&&this.options.revert.call(this.element,n)){var r=this;e(this.helper).animate(this.originalPosition,parseInt(this.options.revertDuration,10),function(){r._trigger("stop",t)!==!1&&r._clear()})}else this._trigger("stop",t)!==!1&&this._clear();return!1},_mouseUp:function(t){return this.options.iframeFix===!0&&e("div.ui-draggable-iframeFix").each(function(){this.parentNode.removeChild(this)}),e.ui.mouse.prototype._mouseUp.call(this,t)},cancel:function(){return this.helper.is(".ui-draggable-dragging")?this._mouseUp({}):this._clear(),this},_getHandle:function(t){var n=!this.options.handle||!e(this.options.handle,this.element).length?!0:!1;return e(this.options.handle,this.element).find("*").andSelf().each(function(){this==t.target&&(n=!0)}),n},_createHelper:function(t){var n=this.options;return t=e.isFunction(n.helper)?e(n.helper.apply(this.element[0],[t])):n.helper=="clone"?this.element.clone().removeAttr("id"):this.element,t.parents("body").length||t.appendTo(n.appendTo=="parent"?this.element[0].parentNode:n.appendTo),t[0]!=this.element[0]&&!/(fixed|absolute)/.test(t.css("position"))&&t.css("position","absolute"),t},_adjustOffsetFromHelper:function(t){typeof t=="string"&&(t=t.split(" ")),e.isArray(t)&&(t={left:+t[0],top:+t[1]||0}),"left"in t&&(this.offset.click.left=t.left+this.margins.left),"right"in t&&(this.offset.click.left=this.helperProportions.width-t.right+this.margins.left),"top"in t&&(this.offset.click.top=t.top+this.margins.top),"bottom"in t&&(this.offset.click.top=this.helperProportions.height-t.bottom+this.margins.top)},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var t=this.offsetParent.offset();this.cssPosition=="absolute"&&this.scrollParent[0]!=document&&e.ui.contains(this.scrollParent[0],this.offsetParent[0])&&(t.left+=this.scrollParent.scrollLeft(),t.top+=this.scrollParent.scrollTop());if(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&this.offsetParent[0].tagName.toLowerCase()=="html"&&e.browser.msie)t={top:0,left:0};return{top:t.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:t.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if(this.cssPosition=="relative"){var e=this.element.position();return{top:e.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:e.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.element.css("marginLeft"),10)||0,top:parseInt(this.element.css("marginTop"),10)||0,right:parseInt(this.element.css("marginRight"),10)||0,bottom:parseInt(this.element.css("marginBottom"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var t=this.options;t.containment=="parent"&&(t.containment=this.helper[0].parentNode);if(t.containment=="document"||t.containment=="window")this.containment=[(t.containment=="document"?0:e(window).scrollLeft())-this.offset.relative.left-this.offset.parent.left,(t.containment=="document"?0:e(window).scrollTop())-this.offset.relative.top-this.offset.parent.top,(t.containment=="document"?0:e(window).scrollLeft())+e(t.containment=="document"?document:window).width()-this.helperProportions.width-this.margins.left,(t.containment=="document"?0:e(window).scrollTop())+(e(t.containment=="document"?document:window).height()||document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top];if(!/^(document|window|parent)$/.test(t.containment)&&t.containment.constructor!=Array){t=e(t.containment);var n=t[0];if(n){t.offset();var r=e(n).css("overflow")!="hidden";this.containment=[(parseInt(e(n).css("borderLeftWidth"),10)||0)+(parseInt(e(n).css("paddingLeft"),10)||0),(parseInt(e(n).css("borderTopWidth"),10)||0)+(parseInt(e(n).css("paddingTop"),10)||0),(r?Math.max(n.scrollWidth,n.offsetWidth):n.offsetWidth)-(parseInt(e(n).css("borderLeftWidth"),10)||0)-(parseInt(e(n).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left-this.margins.right,(r?Math.max(n.scrollHeight,n.offsetHeight):n.offsetHeight)-(parseInt(e(n).css("borderTopWidth"),10)||0)-(parseInt(e(n).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top-this.margins.bottom],this.relative_container=t}}else t.containment.constructor==Array&&(this.containment=t.containment)},_convertPositionTo:function(t,n){n||(n=this.position),t=t=="absolute"?1:-1;var r=this.cssPosition!="absolute"||this.scrollParent[0]!=document&&!!e.ui.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,i=/(html|body)/i.test(r[0].tagName);return{top:n.top+this.offset.relative.top*t+this.offset.parent.top*t-(e.browser.safari&&e.browser.version<526&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollTop():i?0:r.scrollTop())*t),left:n.left+this.offset.relative.left*t+this.offset.parent.left*t-(e.browser.safari&&e.browser.version<526&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():i?0:r.scrollLeft())*t)}},_generatePosition:function(t){var n=this.options,r=this.cssPosition!="absolute"||this.scrollParent[0]!=document&&!!e.ui.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,i=/(html|body)/i.test(r[0].tagName),s=t.pageX,o=t.pageY;if(this.originalPosition){var u;this.containment&&(this.relative_container?(u=this.relative_container.offset(),u=[this.containment[0]+u.left,this.containment[1]+u.top,this.containment[2]+u.left,this.containment[3]+u.top]):u=this.containment,t.pageX-this.offset.click.left<u[0]&&(s=u[0]+this.offset.click.left),t.pageY-this.offset.click.top<u[1]&&(o=u[1]+this.offset.click.top),t.pageX-this.offset.click.left>u[2]&&(s=u[2]+this.offset.click.left),t.pageY-this.offset.click.top>u[3]&&(o=u[3]+this.offset.click.top)),n.grid&&(o=this.originalPageY+Math.round((o-this.originalPageY)/n.grid[1])*n.grid[1],o=u?o-this.offset.click.top<u[1]||o-this.offset.click.top>u[3]?o-this.offset.click.top<u[1]?o+n.grid[1]:o-n.grid[1]:o:o,s=this.originalPageX+Math.round((s-this.originalPageX)/n.grid[0])*n.grid[0],s=u?s-this.offset.click.left<u[0]||s-this.offset.click.left>u[2]?s-this.offset.click.left<u[0]?s+n.grid[0]:s-n.grid[0]:s:s)}return{top:o-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+(e.browser.safari&&e.browser.version<526&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollTop():i?0:r.scrollTop()),left:s-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+(e.browser.safari&&e.browser.version<526&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():i?0:r.scrollLeft())}},_clear:function(){this.helper.removeClass("ui-draggable-dragging"),this.helper[0]!=this.element[0]&&!this.cancelHelperRemoval&&this.helper.remove(),this.helper=null,this.cancelHelperRemoval=!1},_trigger:function(t,n,r){return r=r||this._uiHash(),e.ui.plugin.call(this,t,[n,r]),t=="drag"&&(this.positionAbs=this._convertPositionTo("absolute")),e.Widget.prototype._trigger.call(this,t,n,r)},plugins:{},_uiHash:function(){return{helper:this.helper,position:this.position,originalPosition:this.originalPosition,offset:this.positionAbs}}}),e.extend(e.ui.draggable,{version:"1.8.13"}),e.ui.plugin.add("draggable","connectToSortable",{start:function(t,n){var r=e(this).data("draggable"),i=r.options,s=e.extend({},n,{item:r.element});r.sortables=[],e(i.connectToSortable).each(function(){var n=e.data(this,"sortable");n&&!n.options.disabled&&(r.sortables.push({instance:n,shouldRevert:n.options.revert}),n.refreshPositions(),n._trigger("activate",t,s))})},stop:function(t,n){var r=e(this).data("draggable"),i=e.extend({},n,{item:r.element});e.each(r.sortables,function(){this.instance.isOver?(this.instance.isOver=0,r.cancelHelperRemoval=!0,this.instance.cancelHelperRemoval=!1,this.shouldRevert&&(this.instance.options.revert=!0),this.instance._mouseStop(t),this.instance.options.helper=this.instance.options._helper,r.options.helper=="original"&&this.instance.currentItem.css({top:"auto",left:"auto"})):(this.instance.cancelHelperRemoval=!1,this.instance._trigger("deactivate",t,i))})},drag:function(t,n){var r=e(this).data("draggable"),i=this;e.each(r.sortables,function(){this.instance.positionAbs=r.positionAbs,this.instance.helperProportions=r.helperProportions,this.instance.offset.click=r.offset.click,this.instance._intersectsWith(this.instance.containerCache)?(this.instance.isOver||(this.instance.isOver=1,this.instance.currentItem=e(i).clone().removeAttr("id").appendTo(this.instance.element).data("sortable-item",!0),this.instance.options._helper=this.instance.options.helper,this.instance.options.helper=function(){return n.helper[0]},t.target=this.instance.currentItem[0],this.instance._mouseCapture(t,!0),this.instance._mouseStart(t,!0,!0),this.instance.offset.click.top=r.offset.click.top,this.instance.offset.click.left=r.offset.click.left,this.instance.offset.parent.left-=r.offset.parent.left-this.instance.offset.parent.left,this.instance.offset.parent.top-=r.offset.parent.top-this.instance.offset.parent.top,r._trigger("toSortable",t),r.dropped=this.instance.element,r.currentItem=r.element,this.instance.fromOutside=r),this.instance.currentItem&&this.instance._mouseDrag(t)):this.instance.isOver&&(this.instance.isOver=0,this.instance.cancelHelperRemoval=!0,this.instance.options.revert=!1,this.instance._trigger("out",t,this.instance._uiHash(this.instance)),this.instance._mouseStop(t,!0),this.instance.options.helper=this.instance.options._helper,this.instance.currentItem.remove(),this.instance.placeholder&&this.instance.placeholder.remove(),r._trigger("fromSortable",t),r.dropped=!1)})}}),e.ui.plugin.add("draggable","cursor",{start:function(){var t=e("body"),n=e(this).data("draggable").options;t.css("cursor")&&(n._cursor=t.css("cursor")),t.css("cursor",n.cursor)},stop:function(){var t=e(this).data("draggable").options;t._cursor&&e("body").css("cursor",t._cursor)}}),e.ui.plugin.add("draggable","opacity",{start:function(t,n){t=e(n.helper),n=e(this).data("draggable").options,t.css("opacity")&&(n._opacity=t.css("opacity")),t.css("opacity",n.opacity)},stop:function(t,n){t=e(this).data("draggable").options,t._opacity&&e(n.helper).css("opacity",t._opacity)}}),e.ui.plugin.add("draggable","scroll",{start:function(){var t=e(this).data("draggable");t.scrollParent[0]!=document&&t.scrollParent[0].tagName!="HTML"&&(t.overflowOffset=t.scrollParent.offset())},drag:function(t){var n=e(this).data("draggable"),r=n.options,i=!1;if(n.scrollParent[0]!=document&&n.scrollParent[0].tagName!="HTML"){if(!r.axis||r.axis!="x")n.overflowOffset.top+n.scrollParent[0].offsetHeight-t.pageY<r.scrollSensitivity?n.scrollParent[0].scrollTop=i=n.scrollParent[0].scrollTop+r.scrollSpeed:t.pageY-n.overflowOffset.top<r.scrollSensitivity&&(n.scrollParent[0].scrollTop=i=n.scrollParent[0].scrollTop-r.scrollSpeed);if(!r.axis||r.axis!="y")n.overflowOffset.left+n.scrollParent[0].offsetWidth-t.pageX<r.scrollSensitivity?n.scrollParent[0].scrollLeft=i=n.scrollParent[0].scrollLeft+r.scrollSpeed:t.pageX-n.overflowOffset.left<r.scrollSensitivity&&(n.scrollParent[0].scrollLeft=i=n.scrollParent[0].scrollLeft-r.scrollSpeed)}else{if(!r.axis||r.axis!="x")t.pageY-e(document).scrollTop()<r.scrollSensitivity?i=e(document).scrollTop(e(document).scrollTop()-r.scrollSpeed):e(window).height()-(t.pageY-e(document).scrollTop())<r.scrollSensitivity&&(i=e(document).scrollTop(e(document).scrollTop()+r.scrollSpeed));if(!r.axis||r.axis!="y")t.pageX-e(document).scrollLeft()<r.scrollSensitivity?i=e(document).scrollLeft(e(document).scrollLeft()-r.scrollSpeed):e(window).width()-(t.pageX-e(document).scrollLeft())<r.scrollSensitivity&&(i=e(document).scrollLeft(e(document).scrollLeft()+r.scrollSpeed))}i!==!1&&e.ui.ddmanager&&!r.dropBehaviour&&e.ui.ddmanager.prepareOffsets(n,t)}}),e.ui.plugin.add("draggable","snap",{start:function(){var t=e(this).data("draggable"),n=t.options;t.snapElements=[],e(n.snap.constructor!=String?n.snap.items||":data(draggable)":n.snap).each(function(){var n=e(this),r=n.offset();this!=t.element[0]&&t.snapElements.push({item:this,width:n.outerWidth(),height:n.outerHeight(),top:r.top,left:r.left})})},drag:function(t,n){for(var r=e(this).data("draggable"),i=r.options,s=i.snapTolerance,o=n.offset.left,u=o+r.helperProportions.width,a=n.offset.top,f=a+r.helperProportions.height,l=r.snapElements.length-1;l>=0;l--){var c=r.snapElements[l].left,h=c+r.snapElements[l].width,p=r.snapElements[l].top,v=p+r.snapElements[l].height;if(c-s<o&&o<h+s&&p-s<a&&a<v+s||c-s<o&&o<h+s&&p-s<f&&f<v+s||c-s<u&&u<h+s&&p-s<a&&a<v+s||c-s<u&&u<h+s&&p-s<f&&f<v+s){if(i.snapMode!="inner"){var m=Math.abs(p-f)<=s,g=Math.abs(v-a)<=s,y=Math.abs(c-u)<=s,b=Math.abs(h-o)<=s;m&&(n.position.top=r._convertPositionTo("relative",{top:p-r.helperProportions.height,left:0}).top-r.margins.top),g&&(n.position.top=r._convertPositionTo("relative",{top:v,left:0}).top-r.margins.top),y&&(n.position.left=r._convertPositionTo("relative",{top:0,left:c-r.helperProportions.width}).left-r.margins.left),b&&(n.position.left=r._convertPositionTo("relative",{top:0,left:h}).left-r.margins.left)}var w=m||g||y||b;i.snapMode!="outer"&&(m=Math.abs(p-a)<=s,g=Math.abs(v-f)<=s,y=Math.abs(c-o)<=s,b=Math.abs(h-u)<=s,m&&(n.position.top=r._convertPositionTo("relative",{top:p,left:0}).top-r.margins.top),g&&(n.position.top=r._convertPositionTo("relative",{top:v-r.helperProportions.height,left:0}).top-r.margins.top),y&&(n.position.left=r._convertPositionTo("relative",{top:0,left:c}).left-r.margins.left),b&&(n.position.left=r._convertPositionTo("relative",{top:0,left:h-r.helperProportions.width}).left-r.margins.left)),!r.snapElements[l].snapping&&(m||g||y||b||w)&&r.options.snap.snap&&r.options.snap.snap.call(r.element,t,e.extend(r._uiHash(),{snapItem:r.snapElements[l].item})),r.snapElements[l].snapping=m||g||y||b||w}else r.snapElements[l].snapping&&r.options.snap.release&&r.options.snap.release.call(r.element,t,e.extend(r._uiHash(),{snapItem:r.snapElements[l].item})),r.snapElements[l].snapping=!1}}}),e.ui.plugin.add("draggable","stack",{start:function(){var t=e(this).data("draggable").options;t=e.makeArray(e(t.stack)).sort(function(t,n){return(parseInt(e(t).css("zIndex"),10)||0)-(parseInt(e(n).css("zIndex"),10)||0)});if(t.length){var n=parseInt(t[0].style.zIndex)||0;e(t).each(function(e){this.style.zIndex=n+e}),this[0].style.zIndex=n+t.length}}}),e.ui.plugin.add("draggable","zIndex",{start:function(t,n){t=e(n.helper),n=e(this).data("draggable").options,t.css("zIndex")&&(n._zIndex=t.css("zIndex")),t.css("zIndex",n.zIndex)},stop:function(t,n){t=e(this).data("draggable").options,t._zIndex&&e(n.helper).css("zIndex",t._zIndex)}})}(jQuery),function(e,t){e.widget("ui.sortable",e.ui.mouse,{widgetEventPrefix:"sort",options:{appendTo:"parent",axis:!1,connectWith:!1,containment:!1,cursor:"auto",cursorAt:!1,dropOnEmpty:!0,forcePlaceholderSize:!1,forceHelperSize:!1,grid:!1,handle:!1,helper:"original",items:"> *",opacity:!1,placeholder:!1,revert:!1,scroll:!0,scrollSensitivity:20,scrollSpeed:20,scope:"default",tolerance:"intersect",zIndex:1e3},_create:function(){var e=this.options;this.document=e.document,this.containerCache={},this.element.addClass("ui-sortable"),this.refresh(),this.floating=this.items.length?e.axis==="x"||/left|right/.test(this.items[0].item.css("float"))||/inline|table-cell/.test(this.items[0].item.css("display")):!1,this.offset=this.element.offset(),this._mouseInit()},destroy:function(){this.element.removeClass("ui-sortable ui-sortable-disabled").removeData("sortable").unbind(".sortable"),this._mouseDestroy();for(var e=this.items.length-1;e>=0;e--)this.items[e].item.removeData("sortable-item");return this},_setOption:function(t,n){t==="disabled"?(this.options[t]=n,this.widget()[n?"addClass":"removeClass"]("ui-sortable-disabled")):e.Widget.prototype._setOption.apply(this,arguments)},_mouseCapture:function(t,n){if(this.reverting)return!1;if(this.options.disabled||this.options.type=="static")return!1;this._refreshItems(t);var r=null,i=this,s=e(t.target).parents().each(function(){if(e.data(this,"sortable-item")==i)return r=e(this),!1});e.data(t.target,"sortable-item")==i&&(r=e(t.target));if(!r)return!1;if(this.options.handle&&!n){var o=!1;e(this.options.handle,r).find("*").andSelf().each(function(){this==t.target&&(o=!0)});if(!o)return!1}return this.currentItem=r,this._removeCurrentsFromItems(),!0},_mouseStart:function(t,n,r){var i=this.options,s=this;this.currentContainer=this,this.refreshPositions(),this.helper=this._createHelper(t),this._cacheHelperProportions(),this._cacheMargins(),this.scrollParent=this.helper.scrollParent(),this.offset=this.currentItem.offset(),this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left},this.helper.css("position","absolute"),this.cssPosition=this.helper.css("position"),e.extend(this.offset,{click:{left:t.pageX-this.offset.left,top:t.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()}),this.originalPosition=this._generatePosition(t),this.originalPageX=t.pageX,this.originalPageY=t.pageY,i.cursorAt&&this._adjustOffsetFromHelper(i.cursorAt),this.domPosition={prev:this.currentItem.prev()[0],parent:this.currentItem.parent()[0]},this.helper[0]!=this.currentItem[0]&&this.currentItem.hide(),this._createPlaceholder(),i.containment&&this._setContainment(),i.cursor&&(e("body").css("cursor")&&(this._storedCursor=e("body").css("cursor")),e("body").css("cursor",i.cursor)),i.opacity&&(this.helper.css("opacity")&&(this._storedOpacity=this.helper.css("opacity")),this.helper.css("opacity",i.opacity)),i.zIndex&&(this.helper.css("zIndex")&&(this._storedZIndex=this.helper.css("zIndex")),this.helper.css("zIndex",i.zIndex)),this.scrollParent[0]!=(this.document||document)&&this.scrollParent[0].tagName!="HTML"&&(this.overflowOffset=this.scrollParent.offset()),this._trigger("start",t,this._uiHash()),this._preserveHelperProportions||this._cacheHelperProportions();if(!r)for(var o=this.containers.length-1;o>=0;o--)this.containers[o]._trigger("activate",t,s._uiHash(this));return e.ui.ddmanager&&(e.ui.ddmanager.current=this),e.ui.ddmanager&&!i.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t),this.dragging=!0,this.helper.addClass("ui-sortable-helper"),this._mouseDrag(t),!0},_mouseDrag:function(t){this.position=this._generatePosition(t),this.positionAbs=this._convertPositionTo("absolute"),this.lastPositionAbs||(this.lastPositionAbs=this.positionAbs);if(this.options.scroll){var n=this.options,r=!1;this.scrollParent[0]!=(this.document||document)&&this.scrollParent[0].tagName!="HTML"?(this.overflowOffset.top+this.scrollParent[0].offsetHeight-t.pageY<n.scrollSensitivity?this.scrollParent[0].scrollTop=r=this.scrollParent[0].scrollTop+n.scrollSpeed:t.pageY-this.overflowOffset.top<n.scrollSensitivity&&(this.scrollParent[0].scrollTop=r=this.scrollParent[0].scrollTop-n.scrollSpeed),this.overflowOffset.left+this.scrollParent[0].offsetWidth-t.pageX<n.scrollSensitivity?this.scrollParent[0].scrollLeft=r=this.scrollParent[0].scrollLeft+n.scrollSpeed:t.pageX-this.overflowOffset.left<n.scrollSensitivity&&(this.scrollParent[0].scrollLeft=r=this.scrollParent[0].scrollLeft-n.scrollSpeed)):(t.pageY-e(this.document||document).scrollTop()<n.scrollSensitivity?r=e(this.document||document).scrollTop(e(this.document||document).scrollTop()-n.scrollSpeed):e(window).height()-(t.pageY-e(this.document||document).scrollTop())<n.scrollSensitivity&&(r=e(this.document||document).scrollTop(e(this.document||document).scrollTop
()+n.scrollSpeed)),t.pageX-e(this.document||document).scrollLeft()<n.scrollSensitivity?r=e(this.document||document).scrollLeft(e(this.document||document).scrollLeft()-n.scrollSpeed):e(window).width()-(t.pageX-e(this.document||document).scrollLeft())<n.scrollSensitivity&&(r=e(this.document||document).scrollLeft(e(this.document||document).scrollLeft()+n.scrollSpeed))),r!==!1&&e.ui.ddmanager&&!n.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t)}this.positionAbs=this._convertPositionTo("absolute");if(!this.options.axis||this.options.axis!="y")this.helper[0].style.left=this.position.left+"px";if(!this.options.axis||this.options.axis!="x")this.helper[0].style.top=this.position.top+"px";for(var i=this.items.length-1;i>=0;i--){var s=this.items[i],o=s.item[0],u=this._intersectsWithPointer(s);if(!u)continue;if(o!=this.currentItem[0]&&this.placeholder[u==1?"next":"prev"]()[0]!=o&&!e.ui.contains(this.placeholder[0],o)&&(this.options.type=="semi-dynamic"?!e.ui.contains(this.element[0],o):!0)){this.direction=u==1?"down":"up";if(this.options.tolerance!="pointer"&&!this._intersectsWithSides(s))break;this._rearrange(t,s),this._trigger("change",t,this._uiHash());break}}return this._contactContainers(t),e.ui.ddmanager&&e.ui.ddmanager.drag(this,t),this._trigger("sort",t,this._uiHash()),this.lastPositionAbs=this.positionAbs,!1},_mouseStop:function(t,n){if(!t)return;e.ui.ddmanager&&!this.options.dropBehaviour&&e.ui.ddmanager.drop(this,t);if(this.options.revert){var r=this,i=r.placeholder.offset();r.reverting=!0,e(this.helper).animate({left:i.left-this.offset.parent.left-r.margins.left+(this.offsetParent[0]==(this.document||document).body?0:this.offsetParent[0].scrollLeft),top:i.top-this.offset.parent.top-r.margins.top+(this.offsetParent[0]==(this.document||document).body?0:this.offsetParent[0].scrollTop)},parseInt(this.options.revert,10)||500,function(){r._clear(t)})}else this._clear(t,n);return!1},cancel:function(){var t=this;if(this.dragging){this._mouseUp({target:null}),this.options.helper=="original"?this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper"):this.currentItem.show();for(var n=this.containers.length-1;n>=0;n--)this.containers[n]._trigger("deactivate",null,t._uiHash(this)),this.containers[n].containerCache.over&&(this.containers[n]._trigger("out",null,t._uiHash(this)),this.containers[n].containerCache.over=0)}return this.placeholder&&(this.placeholder[0].parentNode&&this.placeholder[0].parentNode.removeChild(this.placeholder[0]),this.options.helper!="original"&&this.helper&&this.helper[0].parentNode&&this.helper.remove(),e.extend(this,{helper:null,dragging:!1,reverting:!1,_noFinalSort:null}),this.domPosition.prev?e(this.domPosition.prev).after(this.currentItem):e(this.domPosition.parent).prepend(this.currentItem)),this},serialize:function(t){var n=this._getItemsAsjQuery(t&&t.connected),r=[];return t=t||{},e(n).each(function(){var n=(e(t.item||this).attr(t.attribute||"id")||"").match(t.expression||/(.+)[-=_](.+)/);n&&r.push((t.key||n[1]+"[]")+"="+(t.key&&t.expression?n[1]:n[2]))}),!r.length&&t.key&&r.push(t.key+"="),r.join("&")},toArray:function(t){var n=this._getItemsAsjQuery(t&&t.connected),r=[];return t=t||{},n.each(function(){r.push(e(t.item||this).attr(t.attribute||"id")||"")}),r},_intersectsWith:function(e){var t=this.positionAbs.left,n=t+this.helperProportions.width,r=this.positionAbs.top,i=r+this.helperProportions.height,s=e.left,o=s+e.width,u=e.top,a=u+e.height,f=this.offset.click.top,l=this.offset.click.left,c=r+f>u&&r+f<a&&t+l>s&&t+l<o;return this.options.tolerance=="pointer"||this.options.forcePointerForContainers||this.options.tolerance!="pointer"&&this.helperProportions[this.floating?"width":"height"]>e[this.floating?"width":"height"]?c:s<t+this.helperProportions.width/2&&n-this.helperProportions.width/2<o&&u<r+this.helperProportions.height/2&&i-this.helperProportions.height/2<a},_intersectsWithPointer:function(t){var n=e.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,t.top,t.height),r=e.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,t.left,t.width),i=n&&r,s=this._getDragVerticalDirection(),o=this._getDragHorizontalDirection();return i?this.floating?o&&o=="right"||s=="down"?2:1:s&&(s=="down"?2:1):!1},_intersectsWithSides:function(t){var n=e.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,t.top+t.height/2,t.height),r=e.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,t.left+t.width/2,t.width),i=this._getDragVerticalDirection(),s=this._getDragHorizontalDirection();return this.floating&&s?s=="right"&&r||s=="left"&&!r:i&&(i=="down"&&n||i=="up"&&!n)},_getDragVerticalDirection:function(){var e=this.positionAbs.top-this.lastPositionAbs.top;return e!=0&&(e>0?"down":"up")},_getDragHorizontalDirection:function(){var e=this.positionAbs.left-this.lastPositionAbs.left;return e!=0&&(e>0?"right":"left")},refresh:function(e){return this._refreshItems(e),this.refreshPositions(),this},_connectWith:function(){var e=this.options;return e.connectWith.constructor==String?[e.connectWith]:e.connectWith},_getItemsAsjQuery:function(t){var n=this,r=[],i=[],s=this._connectWith();if(s&&t)for(var o=s.length-1;o>=0;o--){var u=e(s[o]);for(var a=u.length-1;a>=0;a--){var f=e.data(u[a],"sortable");f&&f!=this&&!f.options.disabled&&i.push([e.isFunction(f.options.items)?f.options.items.call(f.element):e(f.options.items,f.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),f])}}i.push([e.isFunction(this.options.items)?this.options.items.call(this.element,null,{options:this.options,item:this.currentItem}):e(this.options.items,this.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),this]);for(var o=i.length-1;o>=0;o--)i[o][0].each(function(){r.push(this)});return e(r)},_removeCurrentsFromItems:function(){var e=this.currentItem.find(":data(sortable-item)");for(var t=0;t<this.items.length;t++)for(var n=0;n<e.length;n++)e[n]==this.items[t].item[0]&&this.items.splice(t,1)},_refreshItems:function(t){this.items=[],this.containers=[this];var n=this.items,r=this,i=[[e.isFunction(this.options.items)?this.options.items.call(this.element[0],t,{item:this.currentItem}):e(this.options.items,this.element),this]],s=this._connectWith();if(s)for(var o=s.length-1;o>=0;o--){var u=e(s[o]);for(var a=u.length-1;a>=0;a--){var f=e.data(u[a],"sortable");f&&f!=this&&!f.options.disabled&&(i.push([e.isFunction(f.options.items)?f.options.items.call(f.element[0],t,{item:this.currentItem}):e(f.options.items,f.element),f]),this.containers.push(f))}}for(var o=i.length-1;o>=0;o--){var l=i[o][1],c=i[o][0];for(var a=0,h=c.length;a<h;a++){var p=e(c[a]);p.data("sortable-item",l),n.push({item:p,instance:l,width:0,height:0,left:0,top:0})}}},refreshPositions:function(t){this.offsetParent&&this.helper&&(this.offset.parent=this._getParentOffset());for(var n=this.items.length-1;n>=0;n--){var r=this.items[n];if(r.instance!=this.currentContainer&&this.currentContainer&&r.item[0]!=this.currentItem[0])continue;var i=this.options.toleranceElement?e(this.options.toleranceElement,r.item):r.item;t||(r.width=i.outerWidth(),r.height=i.outerHeight());var s=i.offset();r.left=s.left,r.top=s.top}if(this.options.custom&&this.options.custom.refreshContainers)this.options.custom.refreshContainers.call(this);else for(var n=this.containers.length-1;n>=0;n--){var s=this.containers[n].element.offset();this.containers[n].containerCache.left=s.left,this.containers[n].containerCache.top=s.top,this.containers[n].containerCache.width=this.containers[n].element.outerWidth(),this.containers[n].containerCache.height=this.containers[n].element.outerHeight()}return this},_createPlaceholder:function(t){var n=t||this,r=n.options;if(!r.placeholder||r.placeholder.constructor==String){var i=r.placeholder;r.placeholder={element:function(){var t=e((this.document||document).createElement(n.currentItem[0].nodeName)).addClass(i||n.currentItem[0].className+" ui-sortable-placeholder").removeClass("ui-sortable-helper")[0];return i||(t.style.visibility="hidden"),t},update:function(e,t){if(i&&!r.forcePlaceholderSize)return;t.height()||t.height(n.currentItem.innerHeight()-parseInt(n.currentItem.css("paddingTop")||0,10)-parseInt(n.currentItem.css("paddingBottom")||0,10)),t.width()||t.width(n.currentItem.innerWidth()-parseInt(n.currentItem.css("paddingLeft")||0,10)-parseInt(n.currentItem.css("paddingRight")||0,10))}}}n.placeholder=e(r.placeholder.element.call(n.element,n.currentItem)),n.currentItem.after(n.placeholder),r.placeholder.update(n,n.placeholder)},_contactContainers:function(t){var n=null,r=null;for(var i=this.containers.length-1;i>=0;i--){if(e.ui.contains(this.currentItem[0],this.containers[i].element[0]))continue;if(this._intersectsWith(this.containers[i].containerCache)){if(n&&e.ui.contains(this.containers[i].element[0],n.element[0]))continue;n=this.containers[i],r=i}else this.containers[i].containerCache.over&&(this.containers[i]._trigger("out",t,this._uiHash(this)),this.containers[i].containerCache.over=0)}if(!n)return;if(this.containers.length===1)this.containers[r]._trigger("over",t,this._uiHash(this)),this.containers[r].containerCache.over=1;else if(this.currentContainer!=this.containers[r]){var s=1e4,o=null,u=this.positionAbs[this.containers[r].floating?"left":"top"];for(var a=this.items.length-1;a>=0;a--){if(!e.ui.contains(this.containers[r].element[0],this.items[a].item[0]))continue;var f=this.items[a][this.containers[r].floating?"left":"top"];Math.abs(f-u)<s&&(s=Math.abs(f-u),o=this.items[a])}if(!o&&!this.options.dropOnEmpty)return;this.currentContainer=this.containers[r],o?this._rearrange(t,o,null,!0):this._rearrange(t,null,this.containers[r].element,!0),this._trigger("change",t,this._uiHash()),this.containers[r]._trigger("change",t,this._uiHash(this)),this.options.placeholder.update(this.currentContainer,this.placeholder),this.containers[r]._trigger("over",t,this._uiHash(this)),this.containers[r].containerCache.over=1}},_createHelper:function(t){var n=this.options,r=e.isFunction(n.helper)?e(n.helper.apply(this.element[0],[t,this.currentItem])):n.helper=="clone"?this.currentItem.clone():this.currentItem;return r.parents("body").length||e(n.appendTo!="parent"?n.appendTo:this.currentItem[0].parentNode)[0].appendChild(r[0]),r[0]==this.currentItem[0]&&(this._storedCSS={width:this.currentItem[0].style.width,height:this.currentItem[0].style.height,position:this.currentItem.css("position"),top:this.currentItem.css("top"),left:this.currentItem.css("left")}),(r[0].style.width==""||n.forceHelperSize)&&r.width(this.currentItem.width()),(r[0].style.height==""||n.forceHelperSize)&&r.height(this.currentItem.height()),r},_adjustOffsetFromHelper:function(t){typeof t=="string"&&(t=t.split(" ")),e.isArray(t)&&(t={left:+t[0],top:+t[1]||0}),"left"in t&&(this.offset.click.left=t.left+this.margins.left),"right"in t&&(this.offset.click.left=this.helperProportions.width-t.right+this.margins.left),"top"in t&&(this.offset.click.top=t.top+this.margins.top),"bottom"in t&&(this.offset.click.top=this.helperProportions.height-t.bottom+this.margins.top)},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var t=this.offsetParent.offset();this.cssPosition=="absolute"&&this.scrollParent[0]!=(this.document||document)&&e.ui.contains(this.scrollParent[0],this.offsetParent[0])&&(t.left+=this.scrollParent.scrollLeft(),t.top+=this.scrollParent.scrollTop());if(this.offsetParent[0]==(this.document||document).body||this.offsetParent[0].tagName&&this.offsetParent[0].tagName.toLowerCase()=="html"&&e.browser.msie)t={top:0,left:0};return{top:t.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:t.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if(this.cssPosition=="relative"){var e=this.currentItem.position();return{top:e.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:e.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.currentItem.css("marginLeft"),10)||0,top:parseInt(this.currentItem.css("marginTop"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var t=this.options;t.containment=="parent"&&(t.containment=this.helper[0].parentNode);if(t.containment=="document"||t.containment=="window")this.containment=[0-this.offset.relative.left-this.offset.parent.left,0-this.offset.relative.top-this.offset.parent.top,e(t.containment=="document"?this.document||document:window).width()-this.helperProportions.width-this.margins.left,(e(t.containment=="document"?this.document||document:window).height()||(this.document||document).body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top];if(!/^(document|window|parent)$/.test(t.containment)){var n=e(t.containment)[0],r=e(t.containment).offset(),i=e(n).css("overflow")!="hidden";this.containment=[r.left+(parseInt(e(n).css("borderLeftWidth"),10)||0)+(parseInt(e(n).css("paddingLeft"),10)||0)-this.margins.left,r.top+(parseInt(e(n).css("borderTopWidth"),10)||0)+(parseInt(e(n).css("paddingTop"),10)||0)-this.margins.top,r.left+(i?Math.max(n.scrollWidth,n.offsetWidth):n.offsetWidth)-(parseInt(e(n).css("borderLeftWidth"),10)||0)-(parseInt(e(n).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left,r.top+(i?Math.max(n.scrollHeight,n.offsetHeight):n.offsetHeight)-(parseInt(e(n).css("borderTopWidth"),10)||0)-(parseInt(e(n).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top]}},_convertPositionTo:function(t,n){n||(n=this.position);var r=t=="absolute"?1:-1,i=this.options,s=this.cssPosition!="absolute"||this.scrollParent[0]!=(this.document||document)&&!!e.ui.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,o=/(html|body)/i.test(s[0].tagName);return{top:n.top+this.offset.relative.top*r+this.offset.parent.top*r-(e.browser.safari&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollTop():o?0:s.scrollTop())*r),left:n.left+this.offset.relative.left*r+this.offset.parent.left*r-(e.browser.safari&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():o?0:s.scrollLeft())*r)}},_generatePosition:function(t){var n=this.options,r=this.cssPosition!="absolute"||this.scrollParent[0]!=(this.document||document)&&!!e.ui.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,i=/(html|body)/i.test(r[0].tagName);this.cssPosition=="relative"&&(this.scrollParent[0]==(this.document||document)||this.scrollParent[0]==this.offsetParent[0])&&(this.offset.relative=this._getRelativeOffset());var s=t.pageX,o=t.pageY;if(this.originalPosition){this.containment&&(t.pageX-this.offset.click.left<this.containment[0]&&(s=this.containment[0]+this.offset.click.left),t.pageY-this.offset.click.top<this.containment[1]&&(o=this.containment[1]+this.offset.click.top),t.pageX-this.offset.click.left>this.containment[2]&&(s=this.containment[2]+this.offset.click.left),t.pageY-this.offset.click.top>this.containment[3]&&(o=this.containment[3]+this.offset.click.top));if(n.grid){var u=this.originalPageY+Math.round((o-this.originalPageY)/n.grid[1])*n.grid[1];o=this.containment?u-this.offset.click.top<this.containment[1]||u-this.offset.click.top>this.containment[3]?u-this.offset.click.top<this.containment[1]?u+n.grid[1]:u-n.grid[1]:u:u;var a=this.originalPageX+Math.round((s-this.originalPageX)/n.grid[0])*n.grid[0];s=this.containment?a-this.offset.click.left<this.containment[0]||a-this.offset.click.left>this.containment[2]?a-this.offset.click.left<this.containment[0]?a+n.grid[0]:a-n.grid[0]:a:a}}return{top:o-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+(e.browser.safari&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollTop():i?0:r.scrollTop()),left:s-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+(e.browser.safari&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():i?0:r.scrollLeft())}},_rearrange:function(e,t,n,r){n?n[0].appendChild(this.placeholder[0]):t.item[0].parentNode.insertBefore(this.placeholder[0],this.direction=="down"?t.item[0]:t.item[0].nextSibling),this.counter=this.counter?++this.counter:1;var i=this,s=this.counter;window.setTimeout(function(){s==i.counter&&i.refreshPositions(!r)},0)},_clear:function(t,n){this.reverting=!1;var r=[],i=this;!this._noFinalSort&&this.currentItem[0].parentNode&&this.placeholder.before(this.currentItem),this._noFinalSort=null;if(this.helper[0]==this.currentItem[0]){for(var s in this._storedCSS)if(this._storedCSS[s]=="auto"||this._storedCSS[s]=="static")this._storedCSS[s]="";this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper")}else this.currentItem.show();this.fromOutside&&!n&&r.push(function(e){this._trigger("receive",e,this._uiHash(this.fromOutside))}),(this.fromOutside||this.domPosition.prev!=this.currentItem.prev().not(".ui-sortable-helper")[0]||this.domPosition.parent!=this.currentItem.parent()[0])&&!n&&r.push(function(e){this._trigger("update",e,this._uiHash())});if(!e.ui.contains(this.element[0],this.currentItem[0])){n||r.push(function(e){this._trigger("remove",e,this._uiHash())});for(var s=this.containers.length-1;s>=0;s--)e.ui.contains(this.containers[s].element[0],this.currentItem[0])&&!n&&(r.push(function(e){return function(t){e._trigger("receive",t,this._uiHash(this))}}.call(this,this.containers[s])),r.push(function(e){return function(t){e._trigger("update",t,this._uiHash(this))}}.call(this,this.containers[s])))}for(var s=this.containers.length-1;s>=0;s--)n||r.push(function(e){return function(t){e._trigger("deactivate",t,this._uiHash(this))}}.call(this,this.containers[s])),this.containers[s].containerCache.over&&(r.push(function(e){return function(t){e._trigger("out",t,this._uiHash(this))}}.call(this,this.containers[s])),this.containers[s].containerCache.over=0);this._storedCursor&&e("body").css("cursor",this._storedCursor),this._storedOpacity&&this.helper.css("opacity",this._storedOpacity),this._storedZIndex&&this.helper.css("zIndex",this._storedZIndex=="auto"?"":this._storedZIndex),this.dragging=!1;if(this.cancelHelperRemoval){if(!n){this._trigger("beforeStop",t,this._uiHash());for(var s=0;s<r.length;s++)r[s].call(this,t);this._trigger("stop",t,this._uiHash())}return!1}n||this._trigger("beforeStop",t,this._uiHash()),this.placeholder[0].parentNode.removeChild(this.placeholder[0]),this.helper[0]!=this.currentItem[0]&&this.helper.remove(),this.helper=null;if(!n){for(var s=0;s<r.length;s++)r[s].call(this,t);this._trigger("stop",t,this._uiHash())}return this.fromOutside=!1,!0},_trigger:function(){e.Widget.prototype._trigger.apply(this,arguments)===!1&&this.cancel()},_uiHash:function(t){var n=t||this;return{helper:n.helper,placeholder:n.placeholder||e([]),position:n.position,originalPosition:n.originalPosition,offset:n.positionAbs,item:n.currentItem,sender:t?t.element:null}}}),e.extend(e.ui.sortable,{version:"1.8.13"})}(jQuery),!function(e){e.serializeObject=function(t){var n={},r=n,i=t;return e.each(i,function(){var e=this.name.replace(/\[([^\]]+)?\]/g,",$1").split(","),t=e.length-1,i=0;for(;i<t;i++)r.push?((!r[r.length-1]||r[r.length-1].constructor!==Object||r[r.length-1][e[i+1]]!==undefined)&&r.push({}),r=r[r.length-1]):r=r[e[i]]=r[e[i]]||(e[i+1]==""?[]:{});r.push?r.push(this.value):r[e[t]]=this.value,r=n}),n},e.deserializeObject=function n(e,t,r){var i,s,o,u;t=t||[];if(Object.prototype.toString.call(e)==="[object Object]")for(i in e){o=r?[r,"[",i,"]"].join(""):i;if(e.hasOwnProperty(i)){u=Object.prototype.toString.call(e[i]);if(u==="[object Array]")for(s=0,jsonLen=e[i].length;s<jsonLen;s++)n(e[i][s],t,o+"[]");else u==="[object Object]"?n(e[i],t,o):t.push({name:o,value:e[i]})}}else t.push({name:r,value:e});return t};var t=function(){var t=!!e.fn.prop;return function(e,n){t?e.prop("checked",n):e.attr("checked",n?"checked":null)}}();e.applySerializedArray=function(n,r){var i=e(n).find("input,select,textarea"),s;t(i.filter(":checked"),!1);for(var o=r.length;o--;)s=i.filter("[name='"+r[o].name+"']"),s.filter(":checkbox").length?s.val()==r[o].value&&t(s.filter(":checkbox"),!0):s.filter(":radio").length?t(s.filter("[value='"+r[o].value+"']"),!0):s.val(r[o].value)},e.applySerializedObject=function(t,n){e.applySerializedArray(t,e.deserializeObject(n))},e.fn.serializeObject=e.fn.serializeObject||function(){return e.serializeObject(this.serializeArray())},e.fn.applySerializedObject=function(t){return e.applySerializedObject(this,t),this},e.fn.applySerializedArray=function(t){return e.applySerializedArray(this,t),this}}(jQuery),jQuery.extend(jQuery.easing,{easeInSine:function(e,t,n,r,i){return-r*Math.cos(t/i*(Math.PI/2))+r+n},easeOutSine:function(e,t,n,r,i){return r*Math.sin(t/i*(Math.PI/2))+n},easeInOutSine:function(e,t,n,r,i){return-r/2*(Math.cos(Math.PI*t/i)-1)+n}}),function(e){e.toJSON=function(t){if(typeof JSON=="object"&&JSON.stringify)return JSON.stringify(t);var n=typeof t;if(t===null)return"null";if(n=="undefined")return undefined;if(n=="number"||n=="boolean")return t+"";if(n=="string")return e.quoteString(t);if(n=="object"){if(typeof t.toJSON=="function")return e.toJSON(t.toJSON());if(t.constructor===Date){var r=t.getUTCFullYear(),i=t.getUTCMonth()+1;i<10&&(i="0"+i);var s=t.getUTCDate();s<10&&(s="0"+s);var o=t.getUTCHours();o<10&&(o="0"+o);var u=t.getUTCMinutes();u<10&&(u="0"+u);var a=t.getUTCSeconds();a<10&&(a="0"+a);var f=t.getUTCMilliseconds();return f<100&&(f="0"+f),f<10&&(f="0"+f),'"'+r+"-"+i+"-"+s+"T"+o+":"+u+":"+a+"."+f+'Z"'}if(t.constructor===Array){var l=[];for(var c=0;c<t.length;c++)l.push(e.toJSON(t[c])||"null");return"["+l.join(",")+"]"}var h=[];for(var p in t){var d;n=typeof p;if(n=="number")d='"'+p+'"';else{if(n!="string")continue;d=e.quoteString(p)}if(typeof t[p]=="function")continue;var v=e.toJSON(t[p]);h.push(d+":"+v)}return"{"+h.join(", ")+"}"}},e.quoteString=function(e){return e.match(t)?'"'+e.replace(t,function(e){var t=n[e];return typeof t=="string"?t:(t=e.charCodeAt(),"\\u00"+Math.floor(t/16).toString(16)+(t%16).toString(16))})+'"':'"'+e+'"'};var t=/["\\\x00-\x1f\x7f-\x9f]/g,n={"\b":"\\b","	":"\\t","\n":"\\n","\f":"\\f","\r":"\\r",'"':'\\"',"\\":"\\\\"}}(jQuery),function(e){e.fn.localize=function(t){this.find("*").contents().each(function(){var n=!1,r="";return typeof this.data=="string"&&(r=e.trim(this.data),r&&(n=t.sub[r]||t.top[r])&&(this.data=n)),this.nodeName=="IMG"&&this.attributes.src&&(r=this.attributes.src.nodeValue,r&&(n=t.sub[r]||t.top[r])&&e(this).attr("src",n)),this.nodeName=="A"&&this.attributes.href&&(r=e.trim(this.attributes.href.nodeValue),r&&(n=t.sub[r]||t.top[r])&&e(this).attr("href",n)),this.nodeName=="INPUT"&&this.attributes.type&&this.attributes.value&&["submit","reset","button"].indexOf(this.attributes.type.nodeValue.toLowerCase())>-1&&(r=e.trim(this.attributes.value.nodeValue),r&&(n=t.sub[r]||t.top[r])&&e(this).attr("value",n)),this})}}(jQuery),function(e){function t(e,t,n,r){if(!e.tag.isInline&&n.length>0){n.push("\n");for(var i=0;i<r;i++)n.push("	")}}function n(r,i){var s=[],o=r.attributes.length==0,c=0,p=null,d=r.tag.render&&(i.allowedTags.length==0||e.inArray(r.tag.name,i.allowedTags)>-1)&&(i.removeTags.length==0||e.inArray(r.tag.name,i.removeTags)==-1);!r.isRoot&&d&&(s.push("<"),s.push(r.tag.name),e.each(r.attributes,function(){if(e.inArray(this.name,i.removeAttrs)==-1){var t=(new RegExp(/^(['"]?)(.*?)['"]?$/)).exec(this.value),n=t[2],o=t[1]||"'";this.name=="class"&&(n=e.grep(n.split(" "),function(t){return e.grep(i.allowedClasses,function(n){return n[0]==t&&(n.length==1||e.inArray(r.tag.name,n[1])>-1)}).length>0}).join(" "),o="'"),n!=null&&(n.length>0||e.inArray(this.name,r.tag.requiredAttributes)>-1)&&(s.push(" "),s.push(this.name),s.push("="),s.push(o),s.push(n),s.push(o))}}));if(r.tag.isSelfClosing)d&&s.push(" />"),o=!1;else if(r.tag.isNonClosing)o=!1;else{!r.isRoot&&d&&s.push(">"),c=i.formatIndent++;if(r.tag.toProtect)p=e.htmlClean.trim(r.children.join("")).replace(/<br>/ig,"\n"),s.push(p),o=p.length==0,i.formatIndent--;else{p=[];for(var v=0;v<r.children.length;v++){var m=r.children[v],g=e.htmlClean.trim(h(f(m)?m:m.childrenToString()));l(m)&&v>0&&g.length>0&&(u(m)||a(r.children[v-1]))&&p.push(" ");if(f(m))g.length>0&&p.push(g);else if(v!=r.children.length-1||m.tag.name!="br")i.format&&t(m,i,p,c),p=p.concat(n(m,i))}i.formatIndent--,p.length>0&&(i.format&&p[0]!="\n"&&t(r,i,s,c),s=s.concat(p),o=!1)}!r.isRoot&&d&&(i.format&&t(r,i,s,c-1),s.push("</"),s.push(r.tag.name),s.push(">"))}return!r.tag.allowEmpty&&o?[]:s}function r(t,n,i){return i=i||1,e.inArray(t[t.length-i].tag.name,n)>-1?!0:t.length-(i+1)>0&&r(t,n,i+1)?(t.pop(),!0):!1}function i(e){return e?(this.tag=e,this.isRoot=!1):(this.tag=new o("root"),this.isRoot=!0),this.attributes=[],this.children=[],this.hasAttribute=function(e){for(var t=0;t<this.attributes.length;t++)if(this.attributes[t].name==e)return!0;return!1},this.childrenToString=function(){return this.children.join("")},this}function s(e,t){return this.name=e,this.value=t,this}function o(t,n,r,i){return this.name=t.toLowerCase(),this.isSelfClosing=e.inArray(this.name,y)>-1,this.isNonClosing=e.inArray(this.name,b)>-1,this.isClosing=n!=undefined&&n.length>0,this.isInline=e.inArray(this.name,p)>-1,this.disallowNest=e.inArray(this.name,d)>-1,this.requiredParent=m[e.inArray(this.name,m)+1],this.allowEmpty=e.inArray(this.name,v)>-1,this.toProtect=e.inArray(this.name,g)>-1,this.rawAttributes=r,this.allowedAttributes=w[e.inArray(this.name,w)+1],this.requiredAttributes=E[e.inArray(this.name,E)+1],this.render=i&&e.inArray(this.name,i.notRenderedTags)==-1,this}function u(t){while(c(t)&&t.children.length>0)t=t.children[0];return f(t)&&t.length>0&&e.htmlClean.isWhitespace(t.charAt(0))}function a(t){while(c(t)&&t.children.length>0)t=t.children[t.children.length-1];return f(t)&&t.length>0&&e.htmlClean.isWhitespace(t.charAt(t.length-1))}function f(e){return e.constructor==String}function l(e){return f(e)||e.tag.isInline}function c(e){return e.constructor==i}function h(e){return e.replace(/&nbsp;|\n/g," ").replace(/\s\s+/g," ")}e.fn.htmlClean=function(t){return this.each(function(){this.value?this.value=e.htmlClean(this.value,t):this.innerHTML=e.htmlClean(this.innerHTML,t)})},e.htmlClean=function(t,u){u=e.extend({},e.htmlClean.defaults,u);var a=/<(\/)?(\w+:)?([\w]+)([^>]*)>/gi,l=/(\w+)=(".*?"|'.*?'|[^\s>]*)/gi,c,h=new i,p=[h],d=h;u.bodyOnly&&(c=/<body[^>]*>((\n|.)*)<\/body>/i.exec(t))&&(t=c[1]),t=t.concat("<xxx>");var v;while(c=a.exec(t)){var m=new o(c[3],c[1],c[4],u),g=t.substring(v,c.index);if(g.length>0){var y=d.children[d.children.length-1];d.children.length>0&&f(y=d.children[d.children.length-1])?d.children[d.children.length-1]=y.concat(g):d.children.push(g)}v=a.lastIndex;if(m.isClosing)r(p,[m.name])&&(p.pop(),d=p[p.length-1]);else{var b=new i(m),w;while(w=l.exec(m.rawAttributes)){if(w[1].toLowerCase()=="style"&&u.replaceStyles){var E=!m.isInline;for(var S=0;S<u.replaceStyles.length;S++)u.replaceStyles[S][0].test(w[2])&&(E||(m.render=!1,E=!0),d.children.push(b),p.push(b),d=b,m=new o(u.replaceStyles[S][1],"","",u),b=new i(m))}m.allowedAttributes!=null&&(m.allowedAttributes.length==0||e.inArray(w[1],m.allowedAttributes)>-1)&&b.attributes.push(new s(w[1],w[2]))}e.each(m.requiredAttributes,function(){var e=this.toString();b.hasAttribute(e)||b.attributes.push(new s(e,""))});for(var x=0;x<u.replace.length;x++)for(var T=0;T<u.replace[x][0].length;T++){var N=typeof u.replace[x][0][T]=="string";if(N&&u.replace[x][0][T]==m.name||!N&&u.replace[x][0][T].test(c)){m.render=!1,d.children.push(b),p.push(b),d=b,m=new o(u.replace[x][1],c[1],c[4],u),b=new i(m),b.attributes=d.attributes,x=u.replace.length;break}}var C=!0;d.isRoot||(d.tag.isInline&&!m.isInline?C=!1:d.tag.disallowNest&&m.disallowNest&&!m.requiredParent?C=!1:m.requiredParent&&(C=r(p,m.requiredParent))&&(d=p[p.length-1]));if(C){d.children.push(b);if(m.toProtect){var k=null;while(k=a.exec(t)){var L=new o(k[3],k[1],k[4],u);if(L.isClosing&&L.name==m.name){b.children.push(RegExp.leftContext.substring(v)),v=a.lastIndex;break}}}else!m.isSelfClosing&&!m.isNonClosing&&(p.push(b),d=b)}}}return n(h,u).join("")},e.htmlClean.defaults={bodyOnly:!0,allowedTags:[],removeTags:["basefont","center","dir","font","frame","frameset","iframe","isindex","menu","noframes","s","strike","u"],removeAttrs:[],allowedClasses:[],notRenderedTags:[],format:!1,formatIndent:0,replace:[[["b","big"],"strong"],[["i"],"em"]],replaceStyles:[[/font-weight:\s*bold/i,"strong"],[/font-style:\s*italic/i,"em"],[/vertical-align:\s*super/i,"sup"],[/vertical-align:\s*sub/i,"sub"]]},e.htmlClean.trim=function(t){return e.htmlClean.trimStart(e.htmlClean.trimEnd(t))},e.htmlClean.trimStart=function(t){return t.substring(e.htmlClean.trimStartIndex(t))},e.htmlClean.trimStartIndex=function(t){for(var n=0;n<t.length-1&&e.htmlClean.isWhitespace(t.charAt(n));n++);return n},e.htmlClean.trimEnd=function(t){return t.substring(0,e.htmlClean.trimEndIndex(t))},e.htmlClean.trimEndIndex=function(t){for(var n=t.length-1;n>=0&&e.htmlClean.isWhitespace(t.charAt(n));n--);return n+1},e.htmlClean.isWhitespace=function(t){return e.inArray(t,S)!=-1};var p=["a","abbr","acronym","address","b","big","br","button","caption","cite","code","del","em","font","hr","i","input","img","ins","label","legend","map","q","samp","select","small","span","strong","sub","sup","tt","var"],d=["h1","h2","h3","h4","h5","h6","p","th","td"],v=["th","td"],m=[null,"li",["ul","ol"],"dt",["dl"],"dd",["dl"],"td",["tr"],"th",["tr"],"tr",["table","thead","tbody","tfoot"],"thead",["table"],"tbody",["table"],"tfoot",["table"]],g=["script","style","pre","code"],y=["br","hr","img","link","meta"],b=["!doctype","?xml"],w=[["class"],"?xml",[],"!doctype",[],"a",["accesskey","class","href","name","title","rel","rev","type","tabindex"],"abbr",["class","title"],"acronym",["class","title"],"blockquote",["cite","class"],"button",["class","disabled","name","type","value"],"del",["cite","class","datetime"],"form",["accept","action","class","enctype","method","name"],"input",["accept","accesskey","alt","checked","class","disabled","ismap","maxlength","name","size","readonly","src","tabindex","type","usemap","value"],"img",["alt","class","height","src","width"],"ins",["cite","class","datetime"],"label",["accesskey","class","for"],"legend",["accesskey","class"],"link",["href","rel","type"],"meta",["content","http-equiv","name","scheme"],"map",["name"],"optgroup",["class","disabled","label"],"option",["class","disabled","label","selected","value"],"q",["class","cite"],"td",["colspan","rowspan"],"th",["colspan","rowspan"],"script",["src","type"],"select",["class","disabled","multiple","name","size","tabindex"],"style",["type"],"table",["class","summary"],"textarea",["accesskey","class","cols","disabled","name","readonly","rows","tabindex"]],E=[[],"img",["alt"]],S=[" "," ","	","\n","\r","\f"]}(jQuery);var LiquidMetal=function(){function s(e,t){var n=e.charAt(t);return"A"<=n&&n<="Z"}function o(e,t){var n=e.charAt(t-1);return n==" "||n=="	"}function u(e,t,n,r){n=Math.max(n||0,0),r=Math.min(r||e.length,e.length);for(var i=n;i<r;i++)e[i]=t;return e}var e=0,t=1,n=.8,r=.9,i=.85;return{score:function(t,r){if(r.length==0)return n;if(r.length>t.length)return e;var i=this.buildScoreArray(t,r),s=0;for(var o=0;o<i.length;o++)s+=i[o];return s/i.length},buildScoreArray:function(a,f){var l=new Array(a.length),c=a.toLowerCase(),h=f.toLowerCase().split(""),p=-1,d=!1;for(var v=0;v<h.length;v++){var m=h[v],g=c.indexOf(m,p+1);if(g<0)return u(l,e);g==0&&(d=!0),o(a,g)?(l[g-1]=1,u(l,i,p+1,g-1)):s(a,g)?u(l,i,p+1,g):u(l,e,p+1,g),l[g]=t,p=g}var y=d?r:n;return u(l,y,p+1),l}}}(),Showdown={};Showdown.converter=function(){var e,t,n,r=0;this.makeHtml=function(r){return e=new Array,t=new Array,n=new Array,r=r.replace(/~/g,"~T"),r=r.replace(/\$/g,"~D"),r=r.replace(/\r\n/g,"\n"),r=r.replace(/\r/g,"\n"),r="\n\n"+r+"\n\n",r=M(r),r=r.replace(/^[ \t]+$/mg,""),r=o(r),r=s(r),r=a(r),r=A(r),r=r.replace(/~D/g,"$$"),r=r.replace(/~T/g,"~"),r=r.replace(/https?\:\/\/[^"\s\<\>]*[^.,;'">\:\s\<\>\)\]\!]/g,function(e,t){var n=r.slice(0,t),i=r.slice(t);if(n.match(/<[^>]+$/)&&i.match(/^[^>]*>/))return e;var s=e.replace(/^http:\/\/github.com\//,"https://github.com/");return"<a href='"+s+"'>"+e+"</a>"}),r=r.replace(/[a-z0-9_\-+=.]+@[a-z0-9\-]+(\.[a-z0-9-]+)+/ig,function(e){return"<a href='mailto:"+e+"'>"+e+"</a>"}),r=r.replace(/[a-f0-9]{40}/ig,function(e,t){if(typeof GitHub=="undefined"||typeof GitHub.nameWithOwner=="undefined")return e;var n=r.slice(0,t),i=r.slice(t);return n.match(/@$/)||n.match(/<[^>]+$/)&&i.match(/^[^>]*>/)?e:"<a href='http://github.com/"+GitHub.nameWithOwner+"/commit/"+e+"'>"+e.substring(0
,7)+"</a>"}),r=r.replace(/([a-z0-9_\-+=.]+)@([a-f0-9]{40})/ig,function(e,t,n,s){if(typeof GitHub=="undefined"||typeof GitHub.nameWithOwner=="undefined")return e;GitHub.repoName=GitHub.repoName||i();var o=r.slice(0,s),u=r.slice(s);return o.match(/\/$/)||o.match(/<[^>]+$/)&&u.match(/^[^>]*>/)?e:"<a href='http://github.com/"+t+"/"+GitHub.repoName+"/commit/"+n+"'>"+t+"@"+n.substring(0,7)+"</a>"}),r=r.replace(/([a-z0-9_\-+=.]+\/[a-z0-9_\-+=.]+)@([a-f0-9]{40})/ig,function(e,t,n){return"<a href='http://github.com/"+t+"/commit/"+n+"'>"+t+"@"+n.substring(0,7)+"</a>"}),r=r.replace(/#([0-9]+)/ig,function(e,t,n){if(typeof GitHub=="undefined"||typeof GitHub.nameWithOwner=="undefined")return e;var i=r.slice(0,n),s=r.slice(n);return i==""||i.match(/[a-z0-9_\-+=.]$/)||i.match(/<[^>]+$/)&&s.match(/^[^>]*>/)?e:"<a href='http://github.com/"+GitHub.nameWithOwner+"/issues/#issue/"+t+"'>"+e+"</a>"}),r=r.replace(/([a-z0-9_\-+=.]+)#([0-9]+)/ig,function(e,t,n,s){if(typeof GitHub=="undefined"||typeof GitHub.nameWithOwner=="undefined")return e;GitHub.repoName=GitHub.repoName||i();var o=r.slice(0,s),u=r.slice(s);return o.match(/\/$/)||o.match(/<[^>]+$/)&&u.match(/^[^>]*>/)?e:"<a href='http://github.com/"+t+"/"+GitHub.repoName+"/issues/#issue/"+n+"'>"+e+"</a>"}),r=r.replace(/([a-z0-9_\-+=.]+\/[a-z0-9_\-+=.]+)#([0-9]+)/ig,function(e,t,n){return"<a href='http://github.com/"+t+"/issues/#issue/"+n+"'>"+e+"</a>"}),r};var i=function(){return GitHub.nameWithOwner.match(/^.+\/(.+)$/)[1]},s=function(n){var n=n.replace(/^[ ]{0,3}\[(.+)\]:[ \t]*\n?[ \t]*<?(\S+?)>?[ \t]*\n?[ \t]*(?:(\n*)["(](.+?)[")][ \t]*)?(?:\n+|\Z)/gm,function(n,r,i,s,o){return r=r.toLowerCase(),e[r]=N(i),s?s+o:(o&&(t[r]=o.replace(/"/g,"&quot;")),"")});return n},o=function(e){e=e.replace(/\n/g,"\n\n");var t="p|div|h[1-6]|blockquote|pre|table|dl|ol|ul|script|noscript|form|fieldset|iframe|math|ins|del",n="p|div|h[1-6]|blockquote|pre|table|dl|ol|ul|script|noscript|form|fieldset|iframe|math";return e=e.replace(/^(<(p|div|h[1-6]|blockquote|pre|table|dl|ol|ul|script|noscript|form|fieldset|iframe|math|ins|del)\b[^\r]*?\n<\/\2>[ \t]*(?=\n+))/gm,u),e=e.replace(/^(<(p|div|h[1-6]|blockquote|pre|table|dl|ol|ul|script|noscript|form|fieldset|iframe|math)\b[^\r]*?.*<\/\2>[ \t]*(?=\n+)\n)/gm,u),e=e.replace(/(\n[ ]{0,3}(<(hr)\b([^<>])*?\/?>)[ \t]*(?=\n{2,}))/g,u),e=e.replace(/(\n\n[ ]{0,3}<!(--[^\r]*?--\s*)+>[ \t]*(?=\n{2,}))/g,u),e=e.replace(/(?:\n\n)([ ]{0,3}(?:<([?%])[^\r]*?\2>)[ \t]*(?=\n{2,}))/g,u),e=e.replace(/\n\n/g,"\n"),e},u=function(e,t){var r=t;return r=r.replace(/\n\n/g,"\n"),r=r.replace(/^\n/,""),r=r.replace(/\n+$/g,""),r="\n\n~K"+(n.push(r)-1)+"K\n\n",r},a=function(e){e=v(e);var t=b("<hr />");return e=e.replace(/^[ ]{0,2}([ ]?\*[ ]?){3,}[ \t]*$/gm,t),e=e.replace(/^[ ]{0,2}([ ]?\-[ ]?){3,}[ \t]*$/gm,t),e=e.replace(/^[ ]{0,2}([ ]?\_[ ]?){3,}[ \t]*$/gm,t),e=g(e),e=y(e),e=x(e),e=o(e),e=T(e),e},f=function(e){return e=w(e),e=l(e),e=C(e),e=p(e),e=c(e),e=k(e),e=N(e),e=S(e),e=e.replace(/  +\n/g," <br />\n"),e},l=function(e){var t=/(<[a-z\/!$]("[^"]*"|'[^']*'|[^'">])*>|<!(--.*?--\s*)+>)/gi;return e=e.replace(t,function(e){var t=e.replace(/(.)<\/?code>(?=.)/g,"$1`");return t=_(t,"\\`*_"),t}),e},c=function(e){return e=e.replace(/(\[((?:\[[^\]]*\]|[^\[\]])*)\][ ]?(?:\n[ ]*)?\[(.*?)\])()()()()/g,h),e=e.replace(/(\[((?:\[[^\]]*\]|[^\[\]])*)\]\([ \t]*()<?(.*?)>?[ \t]*((['"])(.*?)\6[ \t]*)?\))/g,h),e=e.replace(/(\[([^\[\]]+)\])()()()()()/g,h),e},h=function(n,r,i,s,o,u,a,f){f==undefined&&(f="");var l=r,c=i,h=s.toLowerCase(),p=o,d=f;if(p==""){h==""&&(h=c.toLowerCase().replace(/ ?\n/g," ")),p="#"+h;if(e[h]!=undefined)p=e[h],t[h]!=undefined&&(d=t[h]);else{if(!(l.search(/\(\s*\)$/m)>-1))return l;p=""}}p=_(p,"*_");var v='<a href="'+p+'"';return d!=""&&(d=d.replace(/"/g,"&quot;"),d=_(d,"*_"),v+=' title="'+d+'"'),v+=">"+c+"</a>",v},p=function(e){return e=e.replace(/(!\[(.*?)\][ ]?(?:\n[ ]*)?\[(.*?)\])()()()()/g,d),e=e.replace(/(!\[(.*?)\]\s?\([ \t]*()<?(\S+?)>?[ \t]*((['"])(.*?)\6[ \t]*)?\))/g,d),e},d=function(n,r,i,s,o,u,a,f){var l=r,c=i,h=s.toLowerCase(),p=o,d=f;d||(d="");if(p==""){h==""&&(h=c.toLowerCase().replace(/ ?\n/g," ")),p="#"+h;if(e[h]==undefined)return l;p=e[h],t[h]!=undefined&&(d=t[h])}c=c.replace(/"/g,"&quot;"),p=_(p,"*_");var v='<img src="'+p+'" alt="'+c+'"';return d=d.replace(/"/g,"&quot;"),d=_(d,"*_"),v+=' title="'+d+'"',v+=" />",v},v=function(e){return e=e.replace(/^(.+)[ \t]*\n=+[ \t]*\n+/gm,function(e,t){return b("<h1>"+f(t)+"</h1>")}),e=e.replace(/^(.+)[ \t]*\n-+[ \t]*\n+/gm,function(e,t){return b("<h2>"+f(t)+"</h2>")}),e=e.replace(/^(\#{1,6})[ \t]*(.+?)[ \t]*\#*\n+/gm,function(e,t,n){var r=t.length;return b("<h"+r+">"+f(n)+"</h"+r+">")}),e},m,g=function(e){e+="~0";var t=/^(([ ]{0,3}([*+-]|\d+[.])[ \t]+)[^\r]+?(~0|\n{2,}(?=\S)(?![ \t]*(?:[*+-]|\d+[.])[ \t]+)))/gm;return r?e=e.replace(t,function(e,t,n){var r=t,i=n.search(/[*+-]/g)>-1?"ul":"ol";r=r.replace(/\n{2,}/g,"\n\n\n");var s=m(r);return s=s.replace(/\s+$/,""),s="<"+i+">"+s+"</"+i+">\n",s}):(t=/(\n\n|^\n?)(([ ]{0,3}([*+-]|\d+[.])[ \t]+)[^\r]+?(~0|\n{2,}(?=\S)(?![ \t]*(?:[*+-]|\d+[.])[ \t]+)))/g,e=e.replace(t,function(e,t,n,r){var i=t,s=n,o=r.search(/[*+-]/g)>-1?"ul":"ol",s=s.replace(/\n{2,}/g,"\n\n\n"),u=m(s);return u=i+"<"+o+">\n"+u+"</"+o+">\n",u})),e=e.replace(/~0/,""),e};m=function(e){return r++,e=e.replace(/\n{2,}$/,"\n"),e+="~0",e=e.replace(/(\n)?(^[ \t]*)([*+-]|\d+[.])[ \t]+([^\r]+?(\n{1,2}))(?=\n*(~0|\2([*+-]|\d+[.])[ \t]+))/gm,function(e,t,n,r,i){var s=i,o=t,u=n;return o||s.search(/\n{2,}/)>-1?s=a(O(s)):(s=g(O(s)),s=s.replace(/\n$/,""),s=f(s)),"<li>"+s+"</li>\n"}),e=e.replace(/~0/g,""),r--,e};var y=function(e){return e+="~0",e=e.replace(/(?:\n\n|^)((?:(?:[ ]{4}|\t).*\n+)+)(\n*[ ]{0,3}[^ \t\n]|(?=~0))/g,function(e,t,n){var r=t,i=n;return r=E(O(r)),r=M(r),r=r.replace(/^\n+/g,""),r=r.replace(/\n+$/g,""),r="<pre><code>"+r+"\n</code></pre>",b(r)+i}),e=e.replace(/~0/,""),e},b=function(e){return e=e.replace(/(^\n+|\n+$)/g,""),"\n\n~K"+(n.push(e)-1)+"K\n\n"},w=function(e){return e=e.replace(/(^|[^\\])(`+)([^\r]*?[^`])\2(?!`)/gm,function(e,t,n,r,i){var s=r;return s=s.replace(/^([ \t]*)/g,""),s=s.replace(/[ \t]*$/g,""),s=E(s),t+"<code>"+s+"</code>"}),e},E=function(e){return e=e.replace(/&/g,"&amp;"),e=e.replace(/</g,"&lt;"),e=e.replace(/>/g,"&gt;"),e=_(e,"*_{}[]\\",!1),e},S=function(e){return e=e.replace(/(\*\*|__)(?=\S)([^\r]*?\S[*_]*)\1/g,"<strong>$2</strong>"),e=e.replace(/(\w)_(\w)/g,"$1~E95E$2"),e=e.replace(/(\*|_)(?=\S)([^\r]*?\S)\1/g,"<em>$2</em>"),e},x=function(e){return e=e.replace(/((^[ \t]*>[ \t]?.+\n(.+\n)*\n*)+)/gm,function(e,t){var n=t;return n=n.replace(/^[ \t]*>[ \t]?/gm,"~0"),n=n.replace(/~0/g,""),n=n.replace(/^[ \t]+$/gm,""),n=a(n),n=n.replace(/(^|\n)/g,"$1  "),n=n.replace(/(\s*<pre>[^\r]+?<\/pre>)/gm,function(e,t){var n=t;return n=n.replace(/^  /mg,"~0"),n=n.replace(/~0/g,""),n}),b("<blockquote>\n"+n+"\n</blockquote>")}),e},T=function(e){e=e.replace(/^\n+/g,""),e=e.replace(/\n+$/g,"");var t=e.split(/\n{2,}/g),r=new Array,i=t.length;for(var s=0;s<i;s++){var o=t[s];o.search(/~K(\d+)K/g)>=0?r.push(o):o.search(/\S/)>=0&&(o=f(o),o=o.replace(/\n/g,"<br />"),o=o.replace(/^([ \t]*)/g,"<p>"),o+="</p>",r.push(o))}i=r.length;for(var s=0;s<i;s++)while(r[s].search(/~K(\d+)K/)>=0){var u=n[RegExp.$1];u=u.replace(/\$/g,"$$$$"),r[s]=r[s].replace(/~K\d+K/,u)}return r.join("\n\n")},N=function(e){return e=e.replace(/&(?!#?[xX]?(?:[0-9a-fA-F]+|\w+);)/g,"&amp;"),e=e.replace(/<(?![a-z\/?\$!])/gi,"&lt;"),e},C=function(e){return e=e.replace(/\\(\\)/g,D),e=e.replace(/\\([`*_{}\[\]()>#+-.!])/g,D),e},k=function(e){return e=e.replace(/<((https?|ftp|dict):[^'">\s]+)>/gi,'<a href="$1">$1</a>'),e=e.replace(/<(?:mailto:)?([-.\w]+\@[-a-z0-9]+(\.[-a-z0-9]+)*\.[a-z]+)>/gi,function(e,t){return L(A(t))}),e},L=function(e){function t(e){var t="0123456789ABCDEF",n=e.charCodeAt(0);return t.charAt(n>>4)+t.charAt(n&15)}var n=[function(e){return"&#"+e.charCodeAt(0)+";"},function(e){return"&#x"+t(e)+";"},function(e){return e}];return e="mailto:"+e,e=e.replace(/./g,function(e){if(e=="@")e=n[Math.floor(Math.random()*2)](e);else if(e!=":"){var t=Math.random();e=t>.9?n[2](e):t>.45?n[1](e):n[0](e)}return e}),e='<a href="'+e+'">'+e+"</a>",e=e.replace(/">.+:/g,'">'),e},A=function(e){return e=e.replace(/~E(\d+)E/g,function(e,t){var n=parseInt(t);return String.fromCharCode(n)}),e},O=function(e){return e=e.replace(/^(\t|[ ]{1,4})/gm,"~0"),e=e.replace(/~0/g,""),e},M=function(e){return e=e.replace(/\t(?=\t)/g,"    "),e=e.replace(/\t/g,"~A~B"),e=e.replace(/~B(.+?)~A/g,function(e,t,n){var r=t,i=4-r.length%4;for(var s=0;s<i;s++)r+=" ";return r}),e=e.replace(/~A/g,"    "),e=e.replace(/~B/g,""),e},_=function(e,t,n){var r="(["+t.replace(/([\[\]\\])/g,"\\$1")+"])";n&&(r="\\\\"+r);var i=new RegExp(r,"g");return e=e.replace(i,D),e},D=function(e,t){var n=t.charCodeAt(0);return"~E"+n+"E"}},function(){var e=[].slice;this.Mercury||(this.Mercury={}),jQuery.extend(this.Mercury,{version:"0.9.0",Regions:Mercury.Regions||{},modalHandlers:Mercury.modalHandlers||{},lightviewHandlers:Mercury.lightviewHandlers||{},dialogHandlers:Mercury.dialogHandlers||{},preloadedViews:Mercury.preloadedViews||{},ajaxHeaders:function(){var e;return e={},e[Mercury.config.csrfHeader]=Mercury.csrfToken,e},on:function(e,t){return jQuery(window).on("mercury:"+e,t)},off:function(e,t){return jQuery(window).off("mercury:"+e,t)},one:function(e,t){return jQuery(window).one("mercury:"+e,t)},trigger:function(e,t){return Mercury.log(e,t),jQuery(window).trigger("mercury:"+e,t)},notify:function(){var t;return t=1<=arguments.length?e.call(arguments,0):[],window.alert(Mercury.I18n.apply(this,t))},warn:function(e,t){t==null&&(t=0);if(console)try{return console.warn(e)}catch(n){if(t>=1)try{return console.debug(e)}catch(r){}}else if(t>=1)return Mercury.notify(e)},deprecated:function(e){return console&&console.trace&&(e=""+e+" -- "+console.trace()),Mercury.warn("deprecated: "+e,1)},log:function(){if(Mercury.debug&&console){if(arguments[0]==="hide:toolbar"||arguments[0]==="show:toolbar")return;try{return console.debug(arguments)}catch(e){}}},locale:function(){var e,t,n;return Mercury.determinedLocale?Mercury.determinedLocale:(Mercury.config.localization.enabled&&(e=[],navigator.language&&(e=navigator.language.toString().split("-")).length&&(n=Mercury.I18n[e[0]]||{},t=e.length>1?n["_"+e[1].toUpperCase()+"_"]:void 0),Mercury.I18n[e[0]]||(e=Mercury.config.localization.preferredLocale.split("-"),n=Mercury.I18n[e[0]]||{},t=e.length>1?n["_"+e[1].toUpperCase()+"_"]:void 0)),Mercury.determinedLocale={top:n||{},sub:t||{}})},I18n:function(){var t,n,r,i;return r=arguments[0],t=2<=arguments.length?e.call(arguments,1):[],n=Mercury.locale(),i=(n.sub[r]||n.top[r]||r||"").toString(),t.length?i.printf.apply(i,t):i}})}.call(this),function(){String.prototype.titleize=function(){return this[0].toUpperCase()+this.slice(1)},String.prototype.toHex=function(){return this[0]==="#"?this:this.replace(/rgb(a)?\(([0-9|%]+)[\s|,]?\s?([0-9|%]+)[\s|,]?\s?([0-9|%]+)[\s|,]?\s?([0-9|.|%]+\s?)?\)/gi,function(e,t,n,r,i,s){return"#"+parseInt(n).toHex()+parseInt(r).toHex()+parseInt(i).toHex()})},String.prototype.regExpEscape=function(){var e,t;return t=["/",".","*","+","?","|","(",")","[","]","{","}","\\"],e=new RegExp("(\\"+t.join("|\\")+")","g"),this.replace(e,"\\$1")},String.prototype.printf=function(){var e,t,n,r,i,s,o,u,a,f;n=this.split("%"),u=n[0],o=/^([sdf])([\s\S%]*)$/,i=0;for(r=a=0,f=n.length;a<f;r=++a){t=n[r],s=o.exec(t);if(r===0||!s||arguments[r]===null){r>1&&(i+=2,u+="%"+t);continue}e=arguments[r-1-i];switch(s[1]){case"s":u+=e;break;case"d":case"i":u+=parseInt(e.toString(),10);break;case"f":u+=parseFloat(e)}u+=s[2]}return u},Number.prototype.toHex=function(){var e;return e=this.toString(16).toUpperCase(),e[1]?e:"0"+e},Number.prototype.toBytes=function(){var e,t;e=parseInt(this),t=0;while(1023<e)e/=1024,t+=1;return t?""+e.toFixed(2)+[""," kb"," Mb"," Gb"," Tb"," Pb"," Eb"][t]:""+e+" bytes"}}.call(this),function(){this.Mercury.PageEditor=function(){function e(e,t){var n;this.saveUrl=e!=null?e:null,this.options=t!=null?t:{};if(window.mercuryInstance)throw Mercury.I18n("Mercury.PageEditor can only be instantiated once.");this.options.visible!==!1&&this.options.visible!=="false"&&(this.options.visible=!0),this.visible=this.options.visible,this.options.saveDataType!==!1&&!this.options.saveDataType&&(this.options.saveDataType="json"),window.mercuryInstance=this,this.regions=[],this.initializeInterface();if(n=jQuery(Mercury.config.csrfSelector).attr("content"))Mercury.csrfToken=n}return e.prototype.initializeInterface=function(){var e,t,n=this;return this.focusableElement=jQuery("<input>",{"class":"mercury-focusable",type:"text"}).appendTo((e=this.options.appendTo)!=null?e:"body"),this.iframe=jQuery("<iframe>",{id:"mercury_iframe","class":"mercury-iframe",frameborder:"0",src:"about:blank"}),this.iframe.appendTo((t=jQuery(this.options.appendTo).get(0))!=null?t:"body"),this.toolbar=new Mercury.Toolbar(jQuery.extend(!0,{},this.options,this.options.toolbarOptions)),this.statusbar=new Mercury.Statusbar(jQuery.extend(!0,{},this.options,this.options.statusbarOptions)),this.resize(),this.iframe.one("load",function(){return n.bindEvents()}),this.iframe.on("load",function(){return n.initializeFrame()}),this.loadIframeSrc(null)},e.prototype.initializeFrame=function(){var e;try{if(this.iframe.data("loaded"))return;return this.iframe.data("loaded",!0),this.document=jQuery(this.iframe.get(0).contentWindow.document),jQuery('<style mercury-styles="true">').html(Mercury.config.injectedStyles).appendTo(this.document.find("head")),e=this.iframe.get(0).contentWindow,jQuery.globalEval=function(t){if(t&&/\S/.test(t))return(e.execScript||function(t){return e.eval.call(e,t)})(t)},e.Mercury=Mercury,window.History&&History.Adapter&&(e.History=History),this.bindDocumentEvents(),this.resize(),this.initializeRegions(),this.finalizeInterface(),Mercury.trigger("ready"),e.jQuery&&e.jQuery(e).trigger("mercury:ready"),e.Event&&e.Event.fire&&e.Event.fire(e,"mercury:ready"),e.onMercuryReady&&e.onMercuryReady(),this.iframe.css({visibility:"visible"})}catch(t){return Mercury.notify("Mercury.PageEditor failed to load: %s\n\nPlease try refreshing.",t)}},e.prototype.initializeRegions=function(){var e,t,n,r,i,s,o,u;this.regions=[],s=jQuery("["+Mercury.config.regions.attribute+"]",this.document);for(t=0,r=s.length;t<r;t++)e=s[t],this.buildRegion(jQuery(e));if(!this.visible)return;o=this.regions,u=[];for(n=0,i=o.length;n<i;n++){e=o[n];if(e.focus){e.focus();break}u.push(void 0)}return u},e.prototype.buildRegion=function(e){var t,n;if(e.data("region"))e=e.data("region");else{t=(e.attr(Mercury.config.regions.attribute)||(typeof (n=Mercury.config.regions).determineType=="function"?n.determineType(e):void 0)||"unknown").titleize();if(t==="Unknown"||!Mercury.Regions[t])throw Mercury.I18n('Region type is malformed, no data-type provided, or "%s" is unknown for the "%s" region.',t,e.attr("id")||"unknown");if(!Mercury.Regions[t].supported)return Mercury.notify("Mercury.Regions.%s is unsupported in this client. Supported browsers are %s.",t,Mercury.Regions[t].supportedText),!1;e=new Mercury.Regions[t](e,this.iframe.get(0).contentWindow),this.previewing&&e.togglePreview()}return this.regions.push(e)},e.prototype.finalizeInterface=function(){var e;this.santizerElement=jQuery("<div>",{id:"mercury_sanitizer",contenteditable:"true",style:"position:fixed;width:100px;height:100px;min-width:0;top:0;left:-100px;opacity:0;overflow:hidden"}),this.santizerElement.appendTo((e=this.options.appendTo)!=null?e:this.document.find("body")),this.snippetToolbar&&this.snippetToolbar.release(),this.snippetToolbar=new Mercury.SnippetToolbar(this.document),this.hijackLinksAndForms();if(!this.visible)return Mercury.trigger("mode",{mode:"preview"})},e.prototype.bindDocumentEvents=function(){var e=this;return this.document.on("mousedown",function(e){Mercury.trigger("hide:dialogs");if(Mercury.region&&jQuery(e.target).closest("["+Mercury.config.regions.attribute+"]").get(0)!==Mercury.region.element.get(0))return Mercury.trigger("unfocus:regions")}),jQuery(this.document).bind("keydown",function(e){if(!e.ctrlKey&&!e.metaKey)return;if(e.keyCode===83)return Mercury.trigger("action",{action:"save"}),e.preventDefault()})},e.prototype.bindEvents=function(){var e=this;return Mercury.on("initialize:frame",function(){return setTimeout(e.initializeFrame,100)}),Mercury.on("focus:frame",function(){return e.iframe.focus()}),Mercury.on("focus:window",function(){return setTimeout(function(){return e.focusableElement.focus()},10)}),Mercury.on("toggle:interface",function(){return e.toggleInterface()}),Mercury.on("reinitialize",function(){return e.initializeRegions()}),Mercury.on("mode",function(t,n){if(n.mode==="preview")return e.previewing=!e.previewing}),Mercury.on("action",function(t,n){var r;r=Mercury.config.globalBehaviors[n.action]||e[n.action];if(typeof r!="function")return;return t.preventDefault(),r.call(e,n)}),jQuery(window).on("resize",function(){return e.resize()}),jQuery(window).bind("keydown",function(e){if(!e.ctrlKey&&!e.metaKey)return;if(e.keyCode===83)return Mercury.trigger("action",{action:"save"}),e.preventDefault()}),window.onbeforeunload=this.beforeUnload},e.prototype.toggleInterface=function(){var e=this;return this.visible?(this.visible=!1,this.toolbar.hide(),this.statusbar.hide(),this.previewing||Mercury.trigger("mode",{mode:"preview"}),this.previewing=!0,this.resize()):(this.visible=!0,this.iframe.animate({top:this.toolbar.height(!0)},200,"easeInOutSine",function(){return e.resize()}),this.toolbar.show(),this.statusbar.show(),Mercury.trigger("mode",{mode:"preview"}),this.previewing=!1)},e.prototype.resize=function(){var e,t,n;return n=jQuery(window).width(),e=this.statusbar.top(),t=this.toolbar.top()+this.toolbar.height(),Mercury.displayRect={top:t,left:0,width:n,height:e-t,fullHeight:e},this.iframe.css({top:t,left:0,height:e-t}),Mercury.trigger("resize")},e.prototype.iframeSrc=function(e,t){var n,r;return e==null&&(e=null),t==null&&(t=!1),e=(e!=null?e:window.location.href).replace((r=(n=Mercury.config).editorUrlRegEx)!=null?r:n.editorUrlRegEx=/([http|https]:\/\/.[^\/]*)\/editor\/?(.*)/i,"$1/$2"),e=e.replace(/[\?|\&]mercury_frame=true/gi,""),e=e.replace(/\&_=\d+/gi,""),t?""+e+(e.indexOf("?")>-1?"&":"?")+"mercury_frame=true&_="+(new Date).getTime():e},e.prototype.loadIframeSrc=function(e){return this.document&&this.document.off(),this.iframe.data("loaded",!1),this.iframe.get(0).contentWindow.document.location.href=this.iframeSrc(e,!0)},e.prototype.hijackLinksAndForms=function(){var e,t,n,r,i,s,o,u,a,f;u=jQuery("a, form",this.document),f=[];for(r=0,s=u.length;r<s;r++){t=u[r],n=!1,a=Mercury.config.nonHijackableClasses||[];for(i=0,o=a.length;i<o;i++){e=a[i];if(jQuery(t).hasClass(e)){n=!0;continue}}!n&&(t.target===""||t.target==="_self")&&!jQuery(t).closest("["+Mercury.config.regions.attribute+"]").length?f.push(jQuery(t).attr("target","_parent")):f.push(void 0)}return f},e.prototype.beforeUnload=function(){return Mercury.changes&&!Mercury.silent?Mercury.I18n("You have unsaved changes.  Are you sure you want to leave without saving them first?"):null},e.prototype.getRegionByName=function(e){var t,n,r,i;i=this.regions;for(n=0,r=i.length;n<r;n++){t=i[n];if(t.name===e)return t}return null},e.prototype.save=function(e){var t,n,r,i,s,o,u=this;return i=(s=(o=this.saveUrl)!=null?o:Mercury.saveUrl)!=null?s:this.iframeSrc(),t=this.serialize(),t={content:t},this.options.saveMethod==="POST"?n="POST":(n="PUT",t._method=n),Mercury.log("saving",t),r={headers:Mercury.ajaxHeaders(),type:n,dataType:this.options.saveDataType,data:t,success:function(t){Mercury.changes=!1,Mercury.trigger("saved",t);if(typeof e=="function")return e()},error:function(e){return Mercury.trigger("save_failed",e),Mercury.notify("Mercury was unable to save to the url: %s",i)}},this.options.saveStyle!=="form"&&(r.data=jQuery.toJSON(t),r.contentType="application/json"),jQuery.ajax(i,r)},e.prototype.serialize=function(){var e,t,n,r,i;t={},i=this.regions;for(n=0,r=i.length;n<r;n++)e=i[n],t[e.name]=e.serialize();return t},e}()}.call(this),function(){this.Mercury.HistoryBuffer=function(){function e(e){this.maxLength=e!=null?e:200,this.index=0,this.stack=[],this.markerRegExp=/<em class="mercury-marker"><\/em>/g}return e.prototype.push=function(e){if(jQuery.type(e)==="string"){if(this.stack[this.index]&&this.stack[this.index].replace(this.markerRegExp,"")===e.replace(this.markerRegExp,""))return}else if(jQuery.type(e)==="object"&&e.html&&this.stack[this.index]&&this.stack[this.index].html===e.html)return;return this.stack=this.stack.slice(0,this.index+1),this.stack.push(e),this.stack.length>this.maxLength&&this.stack.shift(),this.index=this.stack.length-1},e.prototype.undo=function(){return this.index<1?null:(this.index-=1,this.stack[this.index])},e.prototype.redo=function(){return this.index>=this.stack.length-1?null:(this.index+=1,this.stack[this.index])},e}()}.call(this),function(){this.Mercury.tableEditor=function(e,t,n){return Mercury.tableEditor.load(e,t,n),Mercury.tableEditor},jQuery.extend(Mercury.tableEditor,{load:function(e,t,n){return this.table=e,this.cell=t,this.cellContent=n!=null?n:"",this.row=this.cell.parent("tr"),this.columnCount=this.getColumnCount(),this.rowCount=this.getRowCount()},addColumnBefore:function(){return this.addColumn("before")},addColumnAfter:function(){return this.addColumn("after")},addColumn:function(e){var t,n,r,i,s,o,u,a,f,l,c,h;e==null&&(e="after"),a=this.cellSignatureFor(this.cell),c=this.table.find("tr"),h=[];for(t=f=0,l=c.length;f<l;t=++f)o=c[t],u=1,r=e==="after"?{right:a.right}:{left:a.left},(i=this.findCellByOptionsFor(o,r))?(s=jQuery("<"+i.cell.get(0).tagName+">").html(this.cellContent),this.setRowspanFor(s,i.height),e==="before"?i.cell.before(s):i.cell.after(s),h.push(t+=i.height-1)):(n=this.findCellByIntersectionFor(o,a))?h.push(this.setColspanFor(n.cell,n.width+1)):h.push(void 0);return h},removeColumn:function(){var e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v;u=this.cellSignatureFor(this.cell);if(u.width>1)return;s=[],e=[],d=this.table.find("tr");for(n=a=0,c=d.length;a<c;n=++a)o=d[n],(i=this.findCellByOptionsFor(o,{left:u.left,width:u.width}))?(s.push(i.cell),n+=i.height-1):(r=this.findCellByIntersectionFor(o,u))&&e.push(r.cell);for(f=0,h=s.length;f<h;f++)t=s[f],jQuery(t).remove();v=[];for(l=0,p=e.length;l<p;l++)t=e[l],v.push(this.setColspanFor(t,this.colspanFor(t)-1));return v},addRowBefore:function(){return this.addRow("before")},addRowAfter:function(){return this.addRow("after")},addRow:function(e){var t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g;e==null&&(e="after"),s=jQuery("<tr>"),(a=this.rowspanFor(this.cell))>1&&e==="after"&&(this.row=jQuery(this.row.nextAll("tr")[a-2])),n=0,v=this.row.find("th, td");for(f=0,h=v.length;f<h;f++){t=v[f],r=this.colspanFor(t),i=jQuery("<"+t.tagName+">").html(this.cellContent),this.setColspanFor(i,r),n+=r;if((a=this.rowspanFor(t))>1&&e==="after"){this.setRowspanFor(t,a+1);continue}s.append(i)}if(n<this.columnCount){u=0,m=this.row.prevAll("tr");for(l=0,p=m.length;l<p;l++){o=m[l],u+=1,g=jQuery(o).find("td[rowspan], th[rowspan]");for(c=0,d=g.length;c<d;c++)t=g[c],a=this.rowspanFor(t),a-1>=u&&e==="before"?this.setRowspanFor(t,a+1):a-1>=u&&e==="after"&&(a-1===u?(i=jQuery("<"+t.tagName+">").html(this.cellContent),this.setColspanFor(i,this.colspanFor(t)),s.append(i)):this.setRowspanFor(t,a+1))}}return e==="before"?this.row.before(s):this.row.after(s)},removeRow:function(){var e,t,n,r,i,s,o,u,a,f,l,c,h,p,d,v,m,g,y,b,w,E,S,x;a=!0,s=0,i=0,b=this.row.find("td, th");for(l=0,d=b.length;l<d;l++){t=b[l],u=this.rowspanFor(t),s&&u!==s&&(a=!1);if(u<i||!i)i=u;s=u}if(!a&&this.rowspanFor(this.cell)>i)return;if(i>1)for(n=c=0,w=i-2;0<=w?c<=w:c>=w;n=0<=w?++c:--c)jQuery(this.row.nextAll("tr")[n]).remove();E=this.row.find("td[rowspan], th[rowspan]");for(h=0,v=E.length;h<v;h++){t=E[h],f=this.cellSignatureFor(t);if(f.height===i)continue;if(r=this.findCellByOptionsFor(this.row.nextAll("tr")[i-1],{left:f.left,forceAdjacent:!0}))this.setRowspanFor(t,this.rowspanFor(t)-this.rowspanFor(this.cell)),r.direction==="before"?r.cell.before(jQuery(t).clone()):r.cell.after(jQuery(t).clone())}if(this.columnsFor(this.row.find("td, th"))<this.columnCount){o=0,S=this.row.prevAll("tr");for(p=0,m=S.length;p<m;p++){e=S[p],o+=1,x=jQuery(e).find("td[rowspan], th[rowspan]");for(y=0,g=x.length;y<g;y++)t=x[y],u=this.rowspanFor(t),u>o&&this.setRowspanFor(t,u-this.rowspanFor(this.cell))}}return this.row.remove()},increaseColspan:function(){var e;e=this.cell.next("td, th");if(!e.length)return;if(this.rowspanFor(e)!==this.rowspanFor(this.cell))return;if(this.cellIndexFor(e)>this.cellIndexFor(this.cell)+this.colspanFor(this.cell))return;return this.setColspanFor(this.cell,this.colspanFor(this.cell)+this.colspanFor(e)),e.remove()},decreaseColspan:function(){var e;if(this.colspanFor(this.cell)===1)return;return this.setColspanFor(this.cell,this.colspanFor(this.cell)-1),e=jQuery("<"+this.cell.get(0).tagName+">").html(this.cellContent),this.setRowspanFor(e,this.rowspanFor(this.cell)),this.cell.after(e)},increaseRowspan:function(){var e,t,n;n=this.cellSignatureFor(this.cell),t=this.row.nextAll("tr")[n.height-1];if(t&&(e=this.findCellByOptionsFor(t,{left:n.left,width:n.width})))return this.setRowspanFor(this.cell,n.height+e.height),e.cell.remove()},decreaseRowspan:function(){var e,t,n,r;r=this.cellSignatureFor(this.cell);if(r.height===1)return;n=this.row.nextAll("tr")[r.height-2];if(e=this.findCellByOptionsFor(n,{left:r.left,forceAdjacent:!0}))return t=jQuery("<"+this.cell.get(0).tagName+">").html(this.cellContent),this.setColspanFor(t,this.colspanFor(this.cell)),this.setRowspanFor(this.cell,r.height-1),e.direction==="before"?e.cell.before(t):e.cell.after(t)},getColumnCount:function(){return this.columnsFor(this.table.find("thead tr:first-child, tbody tr:first-child, tfoot tr:first-child").first().find("td, th"))},getRowCount:function(){return this.table.find("tr").length},cellIndexFor:function(e){var t,n,r,i,s,o,u,a,f,l,c,h;e=jQuery(e),s=e.parent("tr"),r=this.columnsFor(s.find("td, th")),i=this.columnsFor(e.prevAll("td, th"));if(r<this.columnCount){o=0,c=s.prevAll("tr");for(u=0,f=c.length;u<f;u++){n=c[u],o+=1,h=jQuery(n).find("td[rowspan], th[rowspan]");for(a=0,l=h.length;a<l;a++)t=h[a],this.rowspanFor(t)>o&&this.cellIndexFor(t)<=i&&(i+=this.colspanFor(t))}}return i},cellSignatureFor:function(e){var t;return t={cell:jQuery(e)},t.left=this.cellIndexFor(e),t.width=this.colspanFor(e),t.height=this.rowspanFor(e),t.right=t.left+t.width,t},findCellByOptionsFor:function(e,t){var n,r,i,s,o,u;u=jQuery(e).find("td, th");for(s=0,o=u.length;s<o;s++){n=u[s],i=this.cellSignatureFor(n);if(typeof t.right!="undefined"&&i.right===t.right)return i;if(typeof t.left!="undefined")if(t.width){if(i.left===t.left&&i.width===t.width)return i}else if(!t.forceAdjacent){if(i.left===t.left)return i}else if(t.forceAdjacent&&i.left>t.left)return r=jQuery(n).prev("td, th"),r.length?(i=this.cellSignatureFor(r),i.direction="after"):i.direction="before",i}return t.forceAdjacent?(i.direction="after",i):null},findCellByIntersectionFor:function(e,t){var n,r,i,s,o;o=jQuery(e).find("td, th");for(i=0,s=o.length;i<s;i++){n=o[i],r=this.cellSignatureFor(n);if(r.right-t.left>=0&&r.right>t.left)return r}return null},columnsFor:function(e){var t,n,r,i;n=0;for(r=0,i=e.length;r<i;r++)t=e[r],n+=this.colspanFor(t);return n},colspanFor:function(e){return parseInt(jQuery(e).attr("colspan"))||1},rowspanFor:function(e){return parseInt(jQuery(e).attr("rowspan"))||1},setColspanFor:function(e,t){return jQuery(e).attr("colspan",t>1?t:null)},setRowspanFor:function(e,t){return jQuery(e).attr("rowspan",t>1?t:null)}})}.call(this),function(){this.Mercury.Dialog=function(){function e(e,t,n){this.url=e,this.name=t,this.options=n!=null?n:{},this.button=this.options["for"],this.build(),this.bindEvents(),this.preload()}return e.prototype.build=function(){var e;return this.element=jQuery("<div>",{"class":"mercury-dialog mercury-"+this.name+"-dialog loading",style:"display:none"}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body")},e.prototype.bindEvents=function(){return this.element.on("mousedown",function(e){return e.stopPropagation()})},e.prototype.preload=function(){if(this.options.preload)return this.load()},e.prototype.toggle=function(){return this.visible?this.hide():this.show()},e.prototype.resize=function(){return this.show()},e.prototype.show=function(){return Mercury.trigger("hide:dialogs",this),this.visible=!0,this.loaded?(this.element.css({width:"auto",height:"auto"}),this.position(this.visible)):this.position(),this.appear()},e.prototype.position=function(e){},e.prototype.appear=function(){var e=this;return this.element.css({display:"block",opacity:0}),this.element.animate({opacity:.95},200,"easeInOutSine",function(){if(!e.loaded)return e.load(function(){return e.resize()})})},e.prototype.hide=function(){return this.element.hide(),this.visible=!1},e.prototype.load=function(e){var t=this;if(!this.url)return;if(!Mercury.preloadedViews[this.url])return jQuery.ajax(this.url,{success:function(n){t.loadContent(n),Mercury.dialogHandlers[t.name]&&Mercury.dialogHandlers[t.name].call(t);if(e)return e()},error:function(){return t.hide(),t.button&&t.button.removeClass("pressed"),Mercury.notify('Mercury was unable to load %s for the "%s" dialog.',t.url,t.name)}});this.loadContent(Mercury.preloadedViews[this.url]),Mercury.dialogHandlers[this.name]&&Mercury.dialogHandlers[this.name].call(this);if(e)return e()},e.prototype.loadContent=function(e){this.loaded=!0,this.element.removeClass("loading"),this.element.html(e);if(Mercury.config.localization.enabled)return this.element.localize(Mercury.locale())},e}()}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Palette=function(e){function n(e,t,r){this.url=e,this.name=t,this.options=r!=null?r:{},n.__super__.constructor.apply(this,arguments)}return t(n,e),n.prototype.build=function(){var e;return this.element=jQuery("<div>",{"class":"mercury-palette mercury-"+this.name+"-palette loading",style:"display:none"}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body")},n.prototype.bindEvents=function(){var e=this;return Mercury.on("hide:dialogs",function(t,n){if(n!==e)return e.hide()}),n.__super__.bindEvents.apply(this,arguments)},n.prototype.position=function(e){var t,n;return this.element.css({top:0,left:0,display:"block",visibility:"hidden"}),t=this.button.position(),n=this.element.width(),t.left+n>jQuery(window).width()&&(t.left=t.left-n+this.button.width()),this.element.css({top:t.top+this.button.height(),left:t.left,display:e?"block":"none",visibility:"visible"})},n}(Mercury.Dialog)}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Select=function(e){function n(e,t,r){this.url=e,this.name=t,this.options=r!=null?r:{},n.__super__.constructor.apply(this,arguments)}return t(n,e),n.prototype.build=function(){var e;return this.element=jQuery("<div>",{"class":"mercury-select mercury-"+this.name+"-select loading",style:"display:none"}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body")},n.prototype.bindEvents=function(){var e=this;return Mercury.on("hide:dialogs",function(t,n){if(n!==e)return e.hide()}),this.element.on("mousedown",function(e){return e.preventDefault()}),n.__super__.bindEvents.apply(this,arguments)},n.prototype.position=function(e){var t,n,r,i,s,o,u;return this.element.css({top:0,left:0,display:"block",visibility:"hidden"}),o=this.button.position(),r=this.element.width(),n=this.element.height(),t=jQuery(document).height(),u=o.top+this.button.height()/2-n/2,u<o.top-100&&(u=o.top-100),u<20&&(u=20),i=this.loaded?"auto":n,u+n>=t-20&&(i=t-u-20),s=o.left,s+r>jQuery(window).width()&&(s=s-r+this.button.width()),this.element.css({top:u,left:s,height:i,display:e?"block":"none",visibility:"visible"})},n}(Mercury.Dialog)}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Panel=function(e){function n(e,t,r){this.url=e,this.name=t,this.options=r!=null?r:{},n.__super__.constructor.apply(this,arguments)}return t(n,e),n.prototype.build=function(){var e;return this.element=jQuery("<div>",{"class":"mercury-panel loading",style:"display:none;"}),this.titleElement=jQuery("<h1><span>"+Mercury.I18n(this.options.title)+"</span></h1>").appendTo(this.element),this.paneElement=
jQuery("<div>",{"class":"mercury-panel-pane"}).appendTo(this.element),this.options.closeButton&&jQuery("<a/>",{"class":"mercury-panel-close"}).appendTo(this.titleElement).css({opacity:0}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body")},n.prototype.bindEvents=function(){var e=this;return Mercury.on("resize",function(){return e.position(e.visible)}),Mercury.on("hide:panels",function(t,n){if(n===e)return;return e.button.removeClass("pressed"),e.hide()}),this.titleElement.find(".mercury-panel-close").on("click",function(e){return e.preventDefault(),Mercury.trigger("hide:panels")}),this.element.on("mousedown",function(e){return e.stopPropagation()}),this.element.on("ajax:beforeSend",function(t,n,r){return r.success=function(t){return e.loadContent(t),e.resize()}}),n.__super__.bindEvents.apply(this,arguments)},n.prototype.show=function(){return Mercury.trigger("hide:panels",this),n.__super__.show.apply(this,arguments)},n.prototype.resize=function(){var e,t,n,r=this;this.titleElement.find(".mercury-panel-close").css({opacity:0}),this.paneElement.css({display:"none"}),n=this.element.width(),this.paneElement.css({visibility:"hidden",width:"auto",display:"block"}),t=this.element.width(),this.paneElement.css({visibility:"visible",display:"none"}),e=this.element.offset(),this.element.animate({left:e.left-(t-n),width:t},200,"easeInOutSine",function(){return r.titleElement.find(".mercury-panel-close").animate({opacity:1},100),r.paneElement.css({display:"block",width:t}),jQuery(r.paneElement.find(".focusable").get(0)).focus(),r.makeDraggable()});if(!this.visible)return this.hide()},n.prototype.position=function(e){var t,n,r,i,s;this.element.css({display:"block",visibility:"hidden"}),i=this.element.offset(),t=this.element.width(),n=Mercury.displayRect.height-16,s=n-this.titleElement.outerHeight(),this.paneElement.css({height:s,overflowY:s<30?"hidden":"auto"}),this.moved||(r=Mercury.displayRect.width-t-20),r<=8&&(r=8);if(this.pinned||t+i.left>Mercury.displayRect.width-20)r=Mercury.displayRect.width-t-20;this.element.css({top:Mercury.displayRect.top+8,left:r,height:n,display:e?"block":"none",visibility:"visible"}),this.makeDraggable();if(!e)return this.element.hide()},n.prototype.loadContent=function(e){this.loaded=!0,this.element.removeClass("loading"),this.paneElement.css({visibility:"hidden"}),this.paneElement.html(e);if(Mercury.config.localization.enabled)return this.paneElement.localize(Mercury.locale())},n.prototype.makeDraggable=function(){var e,t=this;return e=this.element.width(),this.element.draggable({handle:"h1 span",axis:"x",opacity:.7,scroll:!1,addClasses:!1,iframeFix:!0,containment:[8,0,Mercury.displayRect.width-e-20,0],stop:function(){var n;return n=t.element.offset().left,t.moved=!0,t.pinned=n>Mercury.displayRect.width-e-30?!0:!1,!0}})},n}(Mercury.Dialog)}.call(this),function(){var e=function(e,t){return function(){return e.apply(t,arguments)}};this.Mercury.modal=function(e,t){var n;return t==null&&(t={}),n=new Mercury.Modal(e,t),n.show(),n},this.Mercury.Modal=function(){function t(t,n){this.url=t,this.options=n!=null?n:{},this.hide=e(this.hide,this)}return t.prototype.show=function(e,t){var n,r=this;e==null&&(e=null),t==null&&(t=null),this.url=e||this.url,this.options=t||this.options,(n=this.options).minWidth||(n.minWidth=400),this.options.ujsHandling!==!1&&(this.options.ujsHandling=!0),Mercury.trigger("focus:window"),this.initializeModal(),this.visible?this.update():this.appear();if(this.options.content)return setTimeout(function(){return r.loadContent(r.options.content)},500)},t.prototype.initializeModal=function(){if(this.initialized)return;return this.build(),this.bindEvents(),this.initialized=!0},t.prototype.build=function(){var e,t;return this.element=jQuery("<div>",{"class":"mercury-modal loading"}),this.element.html('<h1 class="mercury-modal-title"><span></span><a>&times;</a></h1>'),this.element.append('<div class="mercury-modal-content-container"><div class="mercury-modal-content"></div></div>'),this.overlay=jQuery("<div>",{"class":"mercury-modal-overlay"}),this.titleElement=this.element.find(".mercury-modal-title"),this.contentContainerElement=this.element.find(".mercury-modal-content-container"),this.contentElement=this.element.find(".mercury-modal-content"),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body"),this.overlay.appendTo((t=jQuery(this.options.appendTo).get(0))!=null?t:"body")},t.prototype.bindEvents=function(){var e=this;return Mercury.on("refresh",function(){return e.resize(!0)}),Mercury.on("resize",function(){return e.position()}),this.overlay.on("click",function(){if(e.options.allowHideUsingOverlay)return e.hide()}),this.titleElement.find("a").on("click",function(){return e.hide()}),this.options.ujsHandling&&this.element.on("ajax:beforeSend",function(t,n,r){return r.success=function(t){return e.loadContent(t)}}),jQuery(document).on("keydown",function(t){if(t.keyCode===27&&e.visible)return e.hide()})},t.prototype.appear=function(){var e=this;return this.showing=!0,this.position(),this.overlay.show(),this.overlay.animate({opacity:1},200,"easeInOutSine",function(){return e.element.css({top:-e.element.height()}),e.setTitle(),e.element.show(),e.element.animate({top:0},200,"easeInOutSine",function(){return e.visible=!0,e.showing=!1,e.load()})})},t.prototype.resize=function(e){var t,n,r,i,s=this;r=e?"visible":"hidden",n=this.titleElement.outerHeight(),i=this.contentElement.outerWidth(),this.contentPane&&this.contentPane.css({height:"auto"}),this.contentElement.css({height:"auto",visibility:r,display:"block"}),t=this.contentElement.outerHeight()+n,i<this.options.minWidth&&(i=this.options.minWidth);if(t>Mercury.displayRect.fullHeight||this.options.fullHeight)t=Mercury.displayRect.fullHeight;return this.element.stop().animate({left:(Mercury.displayRect.width-i)/2,width:i,height:t},200,"easeInOutSine",function(){var e;return s.contentElement.css({visibility:"visible",display:"block"}),s.contentPane.length?(s.contentElement.css({height:t-n,overflow:"visible"}),e=s.contentControl.length?s.contentControl.outerHeight()+10:0,s.contentPane.css({height:t-n-e-20}),s.contentPane.find(".mercury-display-pane").css({width:i-20})):s.contentElement.css({height:t-n,overflow:"auto"})})},t.prototype.position=function(){var e,t,n,r,i;r=Mercury.displayRect.width,this.contentPane&&this.contentPane.css({height:"auto"}),this.contentElement.css({height:"auto"}),this.element.css({width:"auto",height:"auto",display:"block",visibility:"hidden"}),i=this.element.width(),t=this.element.height(),i<this.options.minWidth&&(i=this.options.minWidth);if(t>Mercury.displayRect.fullHeight||this.options.fullHeight)t=Mercury.displayRect.fullHeight;return n=this.titleElement.outerHeight(),this.contentPane&&this.contentPane.length?(this.contentElement.css({height:t-n,overflow:"visible"}),e=this.contentControl.length?this.contentControl.outerHeight()+10:0,this.contentPane.css({height:t-n-e-20}),this.contentPane.find(".mercury-display-pane").css({width:i-20})):this.contentElement.css({height:t-n,overflow:"auto"}),this.element.css({left:(r-i)/2,width:i,height:t,display:this.visible?"block":"none",visibility:"visible"})},t.prototype.update=function(){return this.reset(),this.resize(),this.load()},t.prototype.load=function(){var e=this;this.setTitle();if(!this.url)return;return this.element.addClass("loading"),Mercury.preloadedViews[this.url]?setTimeout(function(){return e.loadContent(Mercury.preloadedViews[e.url])},10):jQuery.ajax(this.url,{headers:Mercury.ajaxHeaders(),type:this.options.loadType||"GET",data:this.options.loadData,success:function(t){return e.loadContent(t)},error:function(){return e.hide(),Mercury.notify("Mercury was unable to load %s for the modal.",e.url)}})},t.prototype.loadContent=function(e,t){return t==null&&(t=null),this.initializeModal(),this.options=t||this.options,this.setTitle(),this.loaded=!0,this.element.removeClass("loading"),this.contentElement.html(e),this.contentElement.css({display:"none",visibility:"hidden"}),this.contentPane=this.element.find(".mercury-display-pane-container"),this.contentControl=this.element.find(".mercury-display-controls"),this.options.afterLoad&&this.options.afterLoad.call(this),this.options.handler&&(Mercury.modalHandlers[this.options.handler]?typeof Mercury.modalHandlers[this.options.handler]=="function"?Mercury.modalHandlers[this.options.handler].call(this):(jQuery.extend(this,Mercury.modalHandlers[this.options.handler]),this.initialize()):Mercury.lightviewHandlers[this.options.handler]&&(typeof Mercury.lightviewHandlers[this.options.handler]=="function"?Mercury.lightviewHandlers[this.options.handler].call(this):(jQuery.extend(this,Mercury.lightviewHandlers[this.options.handler]),this.initialize()))),Mercury.config.localization.enabled&&this.element.localize(Mercury.locale()),this.element.find(".modal-close").on("click",this.hide),this.resize()},t.prototype.setTitle=function(){var e;return this.titleElement.find("span").html(Mercury.I18n(this.options.title)),e=this.titleElement.find("a"),this.options.closeButton===!1?e.hide():e.show()},t.prototype.serializeForm=function(){return this.element.find("form").serializeObject()||{}},t.prototype.reset=function(){return this.titleElement.find("span").html(""),this.contentElement.html("")},t.prototype.hide=function(){if(this.showing)return;return this.options={},Mercury.trigger("focus:frame"),this.element.hide(),this.overlay.hide(),this.reset(),this.visible=!1},t}()}.call(this),function(){var e=function(e,t){return function(){return e.apply(t,arguments)}};this.Mercury.lightview=function(e,t){var n;return t==null&&(t={}),(n=Mercury.lightview).instance||(n.instance=new Mercury.Lightview(e,t)),Mercury.lightview.instance.show(e,t),Mercury.lightview.instance},this.Mercury.Lightview=function(){function t(t,n){this.url=t,this.options=n!=null?n:{},this.hide=e(this.hide,this)}return t.prototype.show=function(e,t){var n=this;this.url=e||this.url,this.options=t||this.options,this.options.ujsHandling!==!1&&(this.options.ujsHandling=!0),Mercury.trigger("focus:window"),this.initializeLightview(),this.visible?this.update():this.appear();if(this.options.content)return setTimeout(function(){return n.loadContent(n.options.content)},500)},t.prototype.initializeLightview=function(){if(this.initialized)return;return this.build(),this.bindEvents(),this.initialized=!0},t.prototype.build=function(){var e,t;return this.element=jQuery("<div>",{"class":"mercury-lightview loading"}),this.element.html('<h1 class="mercury-lightview-title"><span></span></h1>'),this.element.append('<div class="mercury-lightview-content"></div>'),this.overlay=jQuery("<div>",{"class":"mercury-lightview-overlay"}),this.titleElement=this.element.find(".mercury-lightview-title"),this.options.closeButton&&this.titleElement.append('<a class="mercury-lightview-close"></a>'),this.contentElement=this.element.find(".mercury-lightview-content"),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body"),this.overlay.appendTo((t=jQuery(this.options.appendTo).get(0))!=null?t:"body")},t.prototype.bindEvents=function(){var e=this;return Mercury.on("refresh",function(){return e.resize(!0)}),Mercury.on("resize",function(){if(e.visible)return e.position()}),this.overlay.on("click",function(){if(!e.options.closeButton)return e.hide()}),this.titleElement.find(".mercury-lightview-close").on("click",function(){return e.hide()}),this.options.ujsHandling&&this.element.on("ajax:beforeSend",function(t,n,r){return r.success=function(t){return e.loadContent(t)}}),jQuery(document).on("keydown",function(t){if(t.keyCode===27&&e.visible)return e.hide()})},t.prototype.appear=function(){var e=this;return this.showing=!0,this.position(),this.overlay.show().css({opacity:0}),this.overlay.animate({opacity:1},200,"easeInOutSine",function(){return e.setTitle(),e.element.show().css({opacity:0}),e.element.stop().animate({opacity:1},200,"easeInOutSine",function(){return e.visible=!0,e.showing=!1,e.load()})})},t.prototype.resize=function(e){var t,n,r,i,s,o,u=this;s=e?"visible":"hidden",i=Mercury.displayRect.width,r=Mercury.displayRect.fullHeight,n=this.titleElement.outerHeight(),o=this.contentElement.outerWidth();if(o>i-40||this.options.fullSize)o=i-40;this.contentPane&&this.contentPane.css({height:"auto"}),this.contentElement.css({height:"auto",visibility:s,display:"block"}),t=this.contentElement.outerHeight()+n;if(t>r-20||this.options.fullSize)t=r-20;return o<300&&(o=300),t<150&&(t=150),this.element.stop().animate({top:(r-t)/2+10,left:(Mercury.displayRect.width-o)/2,width:o,height:t},200,"easeInOutSine",function(){var e;return u.contentElement.css({visibility:"visible",display:"block"}),u.contentPane.length?(u.contentElement.css({height:t-n,overflow:"visible"}),e=u.contentControl.length?u.contentControl.outerHeight():0,u.contentPane.css({height:t-n-e-40}),u.contentPane.find(".mercury-display-pane").css({width:o-40})):u.contentElement.css({height:t-n-30,overflow:"auto"})})},t.prototype.position=function(){var e,t,n,r,i,s;i=Mercury.displayRect.width,r=Mercury.displayRect.fullHeight,this.contentPane&&this.contentPane.css({height:"auto"}),this.contentElement.css({height:"auto"}),this.element.css({width:"auto",height:"auto",display:"block",visibility:"hidden"}),s=this.contentElement.width()+40,t=this.contentElement.height()+this.titleElement.outerHeight()+30;if(s>i-40||this.options.fullSize)s=i-40;if(t>r-20||this.options.fullSize)t=r-20;return s<300&&(s=300),t<150&&(t=150),n=this.titleElement.outerHeight(),this.contentPane&&this.contentPane.length?(this.contentElement.css({height:t-n,overflow:"visible"}),e=this.contentControl.length?this.contentControl.outerHeight():0,this.contentPane.css({height:t-n-e-40}),this.contentPane.find(".mercury-display-pane").css({width:s-40})):this.contentElement.css({height:t-n-30,overflow:"auto"}),this.element.css({top:(r-t)/2+10,left:(i-s)/2,width:s,height:t,display:this.visible?"block":"none",visibility:"visible"})},t.prototype.update=function(){return this.reset(),this.resize(),this.load()},t.prototype.load=function(){var e=this;this.setTitle();if(!this.url)return;return this.element.addClass("loading"),Mercury.preloadedViews[this.url]?setTimeout(function(){return e.loadContent(Mercury.preloadedViews[e.url])},10):jQuery.ajax(this.url,{headers:Mercury.ajaxHeaders(),type:this.options.loadType||"GET",data:this.options.loadData,success:function(t){return e.loadContent(t)},error:function(){return e.hide(),Mercury.notify("Mercury was unable to load %s for the lightview.",e.url)}})},t.prototype.loadContent=function(e,t){return t==null&&(t=null),this.initializeLightview(),this.options=t||this.options,this.setTitle(),this.loaded=!0,this.element.removeClass("loading"),this.contentElement.html(e),this.contentElement.css({display:"none",visibility:"hidden"}),this.contentPane=this.element.find(".mercury-display-pane-container"),this.contentControl=this.element.find(".mercury-display-controls"),this.options.afterLoad&&this.options.afterLoad.call(this),this.options.handler&&(Mercury.modalHandlers[this.options.handler]?typeof Mercury.modalHandlers[this.options.handler]=="function"?Mercury.modalHandlers[this.options.handler].call(this):(jQuery.extend(this,Mercury.modalHandlers[this.options.handler]),this.initialize()):Mercury.lightviewHandlers[this.options.handler]&&(typeof Mercury.lightviewHandlers[this.options.handler]=="function"?Mercury.lightviewHandlers[this.options.handler].call(this):(jQuery.extend(this,Mercury.lightviewHandlers[this.options.handler]),this.initialize()))),Mercury.config.localization.enabled&&this.element.localize(Mercury.locale()),this.element.find(".lightview-close").on("click",this.hide),this.resize()},t.prototype.setTitle=function(){return this.titleElement.find("span").html(Mercury.I18n(this.options.title))},t.prototype.serializeForm=function(){return this.element.find("form").serializeObject()||{}},t.prototype.reset=function(){return this.titleElement.find("span").html(""),this.contentElement.html("")},t.prototype.hide=function(){if(this.showing)return;return this.options={},Mercury.trigger("focus:frame"),this.element.hide(),this.overlay.hide(),this.reset(),this.visible=!1},t}()}.call(this),function(){this.Mercury.Statusbar=function(){function e(e){this.options=e!=null?e:{},this.visible=this.options.visible,this.build(),this.bindEvents()}return e.prototype.build=function(){var e;return this.element=jQuery("<div>",{"class":"mercury-statusbar"}),this.aboutElement=jQuery("<a>",{"class":"mercury-statusbar-about"}).appendTo(this.element).html("Mercury Editor v"+Mercury.version),this.pathElement=jQuery("<div>",{"class":"mercury-statusbar-path"}).appendTo(this.element),this.visible||this.element.css({visibility:"hidden"}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body")},e.prototype.bindEvents=function(){var e=this;return Mercury.on("region:update",function(t,n){if(n.region&&jQuery.type(n.region.path)==="function")return e.setPath(n.region.path())}),this.aboutElement.on("click",function(){return Mercury.lightview("/mercury/lightviews/about.html",{title:"Mercury Editor v"+Mercury.version})})},e.prototype.height=function(){return this.element.outerHeight()},e.prototype.top=function(){var e,t;return t=this.element.offset().top,e=parseInt(this.element.css("bottom"))<0?t-this.element.outerHeight():t,this.visible?e:t+this.element.outerHeight()},e.prototype.setPath=function(e){var t,n,r,i;n=[];for(r=0,i=e.length;r<i;r++)t=e[r],n.push("<a>"+t.tagName.toLowerCase()+"</a>");return this.pathElement.html("<span><strong>"+Mercury.I18n("Path:")+" </strong>"+n.reverse().join(" &raquo; ")+"</span>")},e.prototype.show=function(){return this.visible=!0,this.element.css({opacity:0,visibility:"visible"}),this.element.animate({opacity:1},200,"easeInOutSine")},e.prototype.hide=function(){return this.visible=!1,this.element.css({visibility:"hidden"})},e}()}.call(this),function(){var e={}.hasOwnProperty;this.Mercury.Toolbar=function(){function t(e){this.options=e!=null?e:{},this.visible=this.options.visible,this.build(),this.bindEvents()}return t.prototype.build=function(){var t,n,r,i,s,o,u,a,f,l;this.element=jQuery("<div>",{"class":"mercury-toolbar-container",style:"width:10000px"}),this.element.css({width:"100%"}),this.element.appendTo((f=jQuery(this.options.appendTo).get(0))!=null?f:"body"),l=Mercury.config.toolbars;for(a in l){if(!e.call(l,a))continue;r=l[a];if(r._custom)continue;u=jQuery("<div>",{"class":"mercury-toolbar mercury-"+a+"-toolbar"}).appendTo(this.element),r._regions&&u.attr("data-regions",r._regions),i=jQuery("<div>",{"class":"mercury-toolbar-button-container"}).appendTo(u);for(n in r){if(!e.call(r,n))continue;o=r[n];if(n==="_regions")continue;t=this.buildButton(n,o),t&&t.appendTo(i)}i.css("white-space")==="nowrap"&&(s=new Mercury.Toolbar.Expander(a,{appendTo:u,"for":i}),s.appendTo(this.element)),Mercury.config.toolbars.primary&&a!=="primary"&&u.addClass("disabled")}if(!this.visible)return this.element.css({display:"none"})},t.prototype.buildButton=function(t,n){var r,i,s,o,u,a,f;if(t[0]==="_")return!1;switch(jQuery.type(n)){case"array":return f=n[0],a=n[1],o=n[2],new Mercury.Toolbar.Button(t,f,a,o,{appendDialogsTo:this.element});case"object":s=new Mercury.Toolbar.ButtonGroup(t,n);for(r in n){if(!e.call(n,r))continue;u=n[r],i=this.buildButton(r,u),i&&i.appendTo(s)}return s;case"string":return jQuery("<hr>",{"class":"mercury-"+(n==="-"?"line-separator":"separator")});default:throw Mercury.I18n('Unknown button structure -- please provide an array, object, or string for "%s".',t)}},t.prototype.bindEvents=function(){var e=this;return Mercury.on("region:focused",function(t,n){var r,i,s,o,u,a;u=e.element.find(".mercury-toolbar"),a=[];for(s=0,o=u.length;s<o;s++)i=u[s],i=jQuery(i),(r=i.data("regions"))?r.split(",").indexOf(n.region.type())>-1?a.push(i.removeClass("disabled")):a.push(void 0):a.push(void 0);return a}),Mercury.on("region:blurred",function(t,n){var r,i,s,o,u,a;u=e.element.find(".mercury-toolbar"),a=[];for(s=0,o=u.length;s<o;s++)i=u[s],i=jQuery(i),(r=i.data("regions"))?r.split(",").indexOf(n.region.type())>-1?a.push(i.addClass("disabled")):a.push(void 0):a.push(void 0);return a}),this.element.on("click",function(){return Mercury.trigger("hide:dialogs")}),this.element.on("mousedown",function(e){return e.preventDefault()})},t.prototype.height=function(e){return e==null&&(e=!1),this.visible||e?this.element.outerHeight():0},t.prototype.top=function(){return this.visible?this.element.offset().top:0},t.prototype.show=function(){return this.visible=!0,this.element.css({top:-this.element.outerHeight(),display:"block"}),this.element.animate({top:0},200,"easeInOutSine")},t.prototype.hide=function(){return this.visible=!1,this.element.hide()},t}()}.call(this),function(){var e={}.hasOwnProperty;this.Mercury.Toolbar.Button=function(){function t(e,t,n,r,i){return this.name=e,this.title=t,this.summary=n!=null?n:null,this.types=r!=null?r:{},this.options=i!=null?i:{},this.title&&(this.title=Mercury.I18n(this.title)),this.summary&&(this.summary=Mercury.I18n(this.summary)),this.build(),this.bindEvents(),this.element}return t.prototype.build=function(){var t,n,r,i,s,o;this.element=jQuery("<div>",{title:(i=this.summary)!=null?i:this.title,"class":"mercury-button mercury-"+this.name+"-button"}).html("<em>"+this.title+"</em>"),this.element.data("expander",'<div class="mercury-expander-button" data-button="'+this.name+'"><em></em><span>'+this.title+"</span></div>"),this.handled={},s=this.types,o=[];for(r in s){if(!e.call(s,r))continue;t=s[r];switch(r){case"preload":o.push(!0);break;case"regions":this.element.addClass("disabled"),o.push(this.handled[r]=jQuery.isFunction(t)?t.call(this,this.name):t);break;case"toggle":o.push(this.handled[r]=!0);break;case"mode":o.push(this.handled[r]=t===!0?this.name:t);break;case"context":o.push(this.handled[r]=jQuery.isFunction(t)?t:Mercury.Toolbar.Button.contexts[this.name]);break;case"palette":this.element.addClass("mercury-button-palette"),n=jQuery.isFunction(t)?t.call(this,this.name):t,o.push(this.handled[r]=typeof n=="string"?new Mercury.Palette(n,this.name,this.defaultDialogOptions()):n);break;case"select":this.element.addClass("mercury-button-select").find("em").html(this.title),n=jQuery.isFunction(t)?t.call(this,this.name):t,o.push(this.handled[r]=typeof n=="string"?new Mercury.Select(n,this.name,this.defaultDialogOptions()):n);break;case"panel":this.element.addClass("mercury-button-panel"),this.handled.toggle=!0,n=jQuery.isFunction(t)?t.call(this,this.name):t,o.push(this.handled[r]=typeof n=="string"?new Mercury.Panel(n,this.name,this.defaultDialogOptions()):n);break;case"modal":o.push(this.handled[r]=jQuery.isFunction(t)?t.call(this,this.name):t);break;case"lightview":o.push(this.handled[r]=jQuery.isFunction(t)?t.call(this,this.name):t);break;default:throw Mercury.I18n('Unknown button type "%s" used for the "%s" button.',r,this.name)}}return o},t.prototype.bindEvents=function(){var e=this;return Mercury.on("button",function(t,n){if(n.action===e.name)return e.element.click()}),Mercury.on("mode",function(t,n){if(e.handled.mode===n.mode&&e.handled.toggle)return e.togglePressed()}),Mercury.on("region:update",function(t,n){var r;return e.handled.context&&n.region&&jQuery.type(n.region.currentElement)==="function"?(r=n.region.currentElement(),r.length&&e.handled.context.call(e,r,n.region.element)?e.element.addClass("active"):e.element.removeClass("active")):e.element.removeClass("active")}),Mercury.on("region:focused",function(t,n){if(e.handled.regions&&n.region&&n.region.type())return e.handled.regions.indexOf(n.region.type())>-1?e.element.removeClass("disabled"):e.element.addClass("disabled")}),Mercury.on("region:blurred",function(){if(e.handled.regions)return e.element.addClass("disabled")}),this.element.on("mousedown",function(){return e.element.addClass("active")}),this.element.on("mouseup",function(){return e.element.removeClass("active")}),this.element.on("click",function(t){if(e.element.closest(".disabled").length)return;return e.handleClick(t)})},t.prototype.handleClick=function(t){var n,r,i,s;n=!1,s=this.handled;for(i in s){if(!e.call(s,i))continue;r=s[i];switch(i){case"toggle":this.handled.mode||this.togglePressed();break;case"mode":n=!0,Mercury.trigger("mode",{mode:r});break;case"modal":n=this.handleModal(t);break;case"lightview":n=this.handleLightview(t);break;case"palette":case"select":case"panel":n=this.handleDialog(t,i)}}n||Mercury.trigger("action",{action:this.name});if(this.options.regions&&this.options.regions.length)return Mercury.trigger("focus:frame")},t.prototype.handleModal=function(e){return Mercury.modal(this.handled.modal,{title:this.summary||this.title,handler:this.name}),!0},t.prototype.handleLightview=function(e){return Mercury.lightview(this.handled.lightview,{title:this.summary||this.title,handler:this.name,closeButton:!0}),!0},t.prototype.handleDialog=function(e,t){return e.stopPropagation(),this.handled[t].toggle(),!0},t.prototype.defaultDialogOptions=function(){return{title:this.summary||this.title,preload:this.types.preload,appendTo:this.options.appendDialogsTo||"body",closeButton:!0,"for":this.element}},t.prototype.togglePressed=function(){return this.element.toggleClass("pressed")},t}(),this.Mercury.Toolbar.Button.contexts={backColor:function(e){return this.element.css("background-color",e.css("background-color"))},foreColor:function(e){return this.element.css("background-color",e.css("color"))},bold:function(e){var t;return t=e.css("font-weight"),t==="bold"||t>400},italic:function(e){return e.css("font-style")==="italic"},overline:function(e){var t,n,r,i;if(e.css("text-decoration")==="overline")return!0;i=e.parentsUntil(this.element);for(n=0,r=i.length;n<r;n++){t=i[n];if(jQuery(t).css("text-decoration")==="overline")return!0}return!1},strikethrough:function(e,t){return e.css("text-decoration")==="line-through"||!!e.closest("strike",t).length},underline:function(e,t){return e.css("text-decoration")==="underline"||!!e.closest("u",t).length},subscript:function(e,t){return!!e.closest("sub",t).length},superscript:function(e,t){return!!e.closest("sup",t).length},justifyLeft:function(e){return e.css("text-align").indexOf("left")>-1},justifyCenter:function(e){return e.css("text-align").indexOf("center")>-1},justifyRight:function(e){return e.css("text-align").indexOf("right")>-1},justifyFull:function(e){return e.css("text-align").indexOf("justify")>-1},insertOrderedList:function(e,t){return!!e.closest("ol",t.element).length},insertUnorderedList:function(e,t){return!!e.closest("ul",t.element).length}}}.call(this),function(){this.Mercury.Toolbar.ButtonGroup=function(){function e(e,t){return this.name=e,this.options=t!=null?t:{},this.build(),this.bindEvents(),this.regions=this.options._regions,this.element}return e.prototype.build=function(){this.element=jQuery("<div>",{"class":"mercury-button-group mercury-"+this.name+"-group"});if(this.options._context||this.options._regions)return this.element.addClass("disabled")},e.prototype.bindEvents=function(){var e=this;return Mercury.on("region:update",function(t,n){var r,i;r=Mercury.Toolbar.ButtonGroup.contexts[e.name];if(r&&n.region&&jQuery.type(n.region.currentElement)==="function")return i=n.region.currentElement(),i.length&&r.call(e,i,n.region.element)?e.element.removeClass("disabled"):e.element.addClass("disabled")}),Mercury.on("region:focused",function(t,n){if(e.regions&&n.region&&n.region.type()){if(!(e.regions.indexOf(n.region.type())>-1))return e.element.addClass("disabled");if(!e.options._context)return e.element.removeClass("disabled")}}),Mercury.on("region:blurred",function(t,n){if(e.options.regions)return e.element.addClass("disabled")})},e}(),this.Mercury.Toolbar.ButtonGroup.contexts={table:function(e,t){return!!e.closest("table",t).length}}}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Toolbar.Expander=function(e){function n(e,t){return this.name=e,this.options=t,this.container=this.options["for"],n.__super__.constructor.call(this,null,this.name,this.options),this.element}return t(n,e),n.prototype.build=function(){var e;return this.container.css({whiteSpace:"normal",visibility:"hidden",display:"block"}),this.containerWidth=this.container.outerWidth(),this.container.css({visibility:"visible"}),this.trigger=jQuery("<div>",{"class":"mercury-toolbar-expander"}).appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body"),this.element=jQuery("<div>",{"class":"mercury-palette mercury-expander mercury-"+this.name+"-expander",style:"display:none"}),this.windowResize()},n.prototype.bindEvents=function(){var e=this;return Mercury.on("hide:dialogs",function(t,n){if(n!==e)return e.hide()}),Mercury.on("resize",function(){return e.windowResize()}),n.__super__.bindEvents.apply(this,arguments),this.trigger.click(function(t){var n,r,i,s,o;t.stopPropagation(),r=[],o=e.container.find(".mercury-button");for(i=0,s=o.length;i<s;i++)n=o[i],n=jQuery(n),n.position().top>5&&r.push(n.data("expander"));return e.loadContent(r.join("")),e.toggle()}),this.element.click(function(t){var n,r;return r=jQuery(t.target).closest("[data-button]").data("button"),n=e.container.find(".mercury-"+r+"-button"),n.click()})},n.prototype.windowResize=function(){return this.containerWidth>jQuery(window).width()?this.trigger.show():this.trigger.hide(),this.hide()},n.prototype.position=function(e){var t,n;return this.element.css({top:0,left:0,display:"block",visibility:"hidden"}),t=this.trigger.position(),n=this.element.width(),t.left+n>jQuery(window).width()&&(t.left=t.left-n+this.trigger.width()),this.element.css({top:t.top+this.trigger.height(),left:t.left,display:e?"block":"none",visibility:"visible"})},n}(Mercury.Palette)}.call(this),function(){this.Mercury.tooltip=function(e,t,n){return n==null&&(n={}),Mercury.tooltip.show(e,t,n),Mercury.tooltip},jQuery.extend(Mercury.tooltip,{show:function(e,t,n){return this.forElement=e,this.content=t,this.options=n!=null?n:{},this.document=this.forElement.get(0).ownerDocument,this.initialize(),this.visible?this.update():this.appear()},initialize:function(){if(this.initialized)return;return this.build(),this.bindEvents(),this.initialized=!0},build:function(){var e;return this.element=jQuery("<div>",{"class":"mercury-tooltip"}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body")},bindEvents:function(){var e,t,n,r,i=this;Mercury.on("resize",function(){if(i.visible)return i.position()}),this.element.on("mousedown",function(e){return e.preventDefault(),e.stopPropagation()}),r=this.forElement.parentsUntil(jQuery("body",this.document));for(t=0,n=r.length;t<n;t++){e=r[t];if(!(e.scrollHeight>e.clientHeight))continue;jQuery(e).on("scroll",function(){if(i.visible)return i.position()})}return jQuery(this.document).on("scroll",function(){if(i.visible)return i.position()})},appear:function(){var e=this;return this.update(),this.element.show(),this.element.animate({opacity:1},200,"easeInOutSine",function(){return e.visible=!0})},update:function(){return this.element.html(this.content),this.position()},position:function(){var e,t,n,r;return t=this.forElement.offset(),r=this.element.width(),n=t.top+(Mercury.displayRect.top-jQuery(this.document).scrollTop())+this.forElement.outerHeight(),e=t.left-jQuery(this.document).scrollLeft(),e+r+25>Mercury.displayRect.width&&(e=e-(e+r+25)-Mercury.displayRect.width),e=e<=0?0:e,this.element.css({top:n,left:e})},hide:function(){if(!this.initialized)return;return this.element.hide(),this.visible=!1}})}.call(this),function(){var e={}.hasOwnProperty;this.Mercury.Snippet=function(){function t(e,t,n){this.name=e,this.identity=t,n==null&&(n={}),this.version=0,this.data="",this.wrapperTag="div",this.history=new Mercury.HistoryBuffer,this.setOptions(n)}return t.all=[],t.displayOptionsFor=function(e,t,n){var r;return t==null&&(t={}),n==null&&(n=!0),n?Mercury.modal(this.optionsUrl(e),jQuery.extend({title:"Snippet Options",handler:"insertSnippet",snippetName:e,loadType:Mercury.config.snippets.method},t)):(r=Mercury.Snippet.create(e),Mercury.trigger("action",{action:"insertSnippet",value:r})),Mercury.snippet=null},t.optionsUrl=function(e){var t;return t=Mercury.config.snippets.optionsUrl,typeof t=="function"&&(t=t()),t.replace(":name",e)},t.previewUrl=function(e){var t;return t=Mercury.config.snippets.previewUrl,typeof t=="function"&&(t=t()),t.replace(":name",e)},t.create=function(e,t){var n;return n=new Mercury.Snippet(e,this.uniqueId(),t),this.all.push(n),n},t.uniqueId=function(){var e,t,n,r,i;i=[0,"snippet_0"],e=i[0],n=i[1],t=function(){var e,t,n,i;n=this.all,i=[];for(e=0,t=n.length;e<t;e++)r=n[e],i.push(r.identity
);return i}.call(this);while(t.indexOf(n)!==-1)e+=1,n="snippet_"+e;return n},t.find=function(e){var t,n,r,i;i=this.all;for(n=0,r=i.length;n<r;n++){t=i[n];if(t.identity===e)return t}return null},t.load=function(t){var n,r,i,s;s=[];for(r in t){if(!e.call(t,r))continue;n=t[r],i=new Mercury.Snippet(n.name,r,n),s.push(this.all.push(i))}return s},t.clearAll=function(){return delete this.all,this.all=[]},t.prototype.getHTML=function(e,t){var n;return t==null&&(t=null),n=jQuery("<"+this.wrapperTag+">",{"class":""+this.name+"-snippet",contenteditable:"false","data-snippet":this.identity,"data-version":this.version},e),n.html("["+this.identity+"]"),this.loadPreview(n,t),n},t.prototype.getText=function(e){return"[--"+this.identity+"--]"},t.prototype.loadPreview=function(e,n){var r=this;return n==null&&(n=null),jQuery.ajax(t.previewUrl(this.name),{headers:Mercury.ajaxHeaders(),type:Mercury.config.snippets.method,data:this.options,success:function(t){r.data=t,e.html(t);if(n)return n()},error:function(){return Mercury.notify('Error loading the preview for the "%s" snippet.',r.name)}})},t.prototype.displayOptions=function(){return Mercury.snippet=this,Mercury.modal(t.optionsUrl(this.name),{title:"Snippet Options",handler:"insertSnippet",loadType:Mercury.config.snippets.method,loadData:this.options})},t.prototype.setOptions=function(e){return this.options=e,delete this.options.authenticity_token,delete this.options.utf8,this.options.wrapperTag&&(this.wrapperTag=this.options.wrapperTag),this.version+=1,this.history.push(this.options)},t.prototype.setVersion=function(e){return e==null&&(e=null),e=parseInt(e),e&&this.history.stack[e-1]?(this.version=e,this.options=this.history.stack[e-1],!0):!1},t.prototype.serialize=function(){return $.extend({name:this.name},this.options)},t}()}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.SnippetToolbar=function(n){function r(e,t){this.document=e,this.options=t!=null?t:{},this._boundEvents=[],r.__super__.constructor.call(this,this.options)}return t(r,n),r.prototype.build=function(){var t,n,r,i,s,o;this.element=jQuery("<div>",{"class":"mercury-toolbar mercury-snippet-toolbar",style:"display:none"}),this.element.appendTo((i=jQuery(this.options.appendTo).get(0))!=null?i:"body"),s=Mercury.config.toolbars.snippets,o=[];for(n in s){if(!e.call(s,n))continue;r=s[n],t=this.buildButton(n,r),t?o.push(t.appendTo(this.element)):o.push(void 0)}return o},r.prototype.bindEvents=function(){var e=this;return this.bindReleasableEvent(Mercury,"show:toolbar",function(t,n){if(!n.snippet)return;return n.snippet.mouseout(function(){return e.hide()}),e.show(n.snippet)}),this.bindReleasableEvent(Mercury,"hide:toolbar",function(t,n){if(!n.type||n.type!=="snippet")return;return e.hide(n.immediately)}),this.bindReleasableEvent(jQuery(this.document),"scroll",function(){if(e.visible)return e.position()}),this.element.mousemove(function(){return clearTimeout(e.hideTimeout)}),this.element.mouseout(function(){return e.hide()})},r.prototype.bindReleasableEvent=function(e,t,n){return e.on(t,n),this._boundEvents.push([e,t,n])},r.prototype.show=function(e){return this.snippet=e,Mercury.tooltip.hide(),this.position(),this.appear()},r.prototype.position=function(){var e,t,n;return t=this.snippet.offset(),n=t.top+Mercury.displayRect.top-jQuery(this.document).scrollTop()-this.height()+10,e=t.left-jQuery(this.document).scrollLeft(),this.element.css({top:n,left:e})},r.prototype.appear=function(){clearTimeout(this.hideTimeout);if(this.visible)return;return this.visible=!0,this.element.css({display:"block",opacity:0}),this.element.stop().animate({opacity:1},200,"easeInOutSine")},r.prototype.hide=function(e){var t=this;return e==null&&(e=!1),clearTimeout(this.hideTimeout),e?(this.element.hide(),this.visible=!1):this.hideTimeout=setTimeout(function(){return t.element.stop().animate({opacity:0},300,"easeInOutSine",function(){return t.element.hide()}),t.visible=!1},500)},r.prototype.release=function(){var e,t,n,r,i,s,o;this.element.off(),this.element.remove(),s=this._boundEvents;for(r=0,i=s.length;r<i;r++)o=s[r],n=o[0],e=o[1],t=o[2],n.off(e,t);return this._boundEvents=[]},r}(Mercury.Toolbar)}.call(this),function(){this.Mercury.Region=function(){function e(e,t,n){this.element=e,this.window=t,this.options=n!=null?n:{},Mercury.log("building "+this.type(),this.element,this.options),this.document=this.window.document,this.name=this.element.attr(Mercury.config.regions.identifier),this.history=new Mercury.HistoryBuffer,this.build(),this.bindEvents(),this.pushHistory(),this.element.data("region",this)}return e.prototype.type=function(){return"unknown"},e.prototype.build=function(){},e.prototype.focus=function(){},e.prototype.bindEvents=function(){var e=this;return Mercury.on("mode",function(t,n){if(n.mode==="preview")return e.togglePreview()}),Mercury.on("focus:frame",function(){if(e.previewing||Mercury.region!==e)return;return e.focus()}),Mercury.on("action",function(t,n){if(e.previewing||Mercury.region!==e||t.isDefaultPrevented())return;if(n.action)return e.execCommand(n.action,n)}),this.element.on("mousemove",function(t){var n;if(e.previewing||Mercury.region!==e)return;n=jQuery(t.target).closest("[data-snippet]");if(n.length){e.snippet=n;if(e.snippet.data("snippet"))return Mercury.trigger("show:toolbar",{type:"snippet",snippet:e.snippet})}}),this.element.on("mouseout",function(){if(e.previewing)return;return Mercury.trigger("hide:toolbar",{type:"snippet",immediately:!1})})},e.prototype.content=function(e,t){var n,r,i,s,o;e==null&&(e=null),t==null&&(t=!1);if(e!==null)return this.element.html(e);n=jQuery("<div>").appendTo(this.document.createDocumentFragment()),n.html(this.element.html().replace(/^\s+|\s+$/g,""));if(t){o=n.find("[data-snippet]");for(i=0,s=o.length;i<s;i++)r=o[i],r=jQuery(r),r.attr({contenteditable:null,"data-version":null}),r.html("["+r.data("snippet")+"]")}return n.html()},e.prototype.togglePreview=function(){if(!this.previewing)return this.previewing=!0,this.element.removeAttr(Mercury.config.regions.attribute),Mercury.trigger("region:blurred",{region:this});this.previewing=!1,this.element.attr(Mercury.config.regions.attribute,this.type());if(Mercury.region===this)return this.focus()},e.prototype.execCommand=function(e,t){return t==null&&(t={}),this.focus(),e!=="redo"&&this.pushHistory(),Mercury.log("execCommand",e,t.value),Mercury.changes=!0},e.prototype.pushHistory=function(){return this.history.push(this.content())},e.prototype.snippets=function(){var e,t,n,r,i,s;n={},s=this.element.find("[data-snippet]");for(r=0,i=s.length;r<i;r++){e=s[r],t=Mercury.Snippet.find(jQuery(e).data("snippet"));if(!t)continue;n[t.identity]=t.serialize()}return n},e.prototype.dataAttributes=function(){var e,t,n,r,i;t={},i=Mercury.config.regions.dataAttributes;for(n=0,r=i.length;n<r;n++)e=i[n],t[e]=(this.container||this.element).attr("data-"+e);return t},e.prototype.serialize=function(){return{type:this.type(),data:this.dataAttributes(),value:this.content(null,!0),snippets:this.snippets()}},e}()}.call(this),function(){var e={}.hasOwnProperty;this.Mercury.uploader=function(e,t){return Mercury.config.uploading.enabled&&Mercury.uploader.show(e,t),Mercury.uploader},jQuery.extend(Mercury.uploader,{show:function(e,t){this.options=t!=null?t:{},this.file=new Mercury.uploader.File(e);if(this.file.errors){alert("Error: "+this.file.errors);return}if(!this.supported())return;return Mercury.trigger("focus:window"),this.initialize(),this.appear()},initialize:function(){if(this.initialized)return;return this.build(),this.bindEvents(),this.initialized=!0},supported:function(){var e;return e=new XMLHttpRequest,window.Uint8Array&&window.ArrayBuffer&&!XMLHttpRequest.prototype.sendAsBinary&&(XMLHttpRequest.prototype.sendAsBinary=function(e){var t,n,r,i,s;r=new Uint8Array(e.length);for(n=i=0,s=e.length;i<s;n=++i)t=e[n],r[n]=e.charCodeAt(n)&255;return this.send(r.buffer)}),!(!e.upload||!e.sendAsBinary||!Mercury.uploader.fileReaderSupported()&&!Mercury.uploader.formDataSupported())},fileReaderSupported:function(){return!!window.FileReader},formDataSupported:function(){return!!window.FormData},build:function(){var e,t;return this.element=jQuery("<div>",{"class":"mercury-uploader",style:"display:none"}),this.element.append('<div class="mercury-uploader-preview"><b><img/></b></div>'),this.element.append('<div class="mercury-uploader-details"></div>'),this.element.append('<div class="mercury-uploader-progress"><span></span><div class="mercury-uploader-indicator"><div><b>0%</b></div></div></div>'),this.updateStatus("Processing..."),this.overlay=jQuery("<div>",{"class":"mercury-uploader-overlay",style:"display:none"}),this.element.appendTo((e=jQuery(this.options.appendTo).get(0))!=null?e:"body"),this.overlay.appendTo((t=jQuery(this.options.appendTo).get(0))!=null?t:"body")},bindEvents:function(){var e=this;return Mercury.on("resize",function(){return e.position()})},appear:function(){var e=this;return this.fillDisplay(),this.position(),this.overlay.show(),this.overlay.animate({opacity:1},200,"easeInOutSine",function(){return e.element.show(),e.element.animate({opacity:1},200,"easeInOutSine",function(){return e.visible=!0,e.loadImage()})})},position:function(){var e,t;return t=this.element.outerWidth(),e=this.element.outerHeight(),this.element.css({top:(Mercury.displayRect.height-e)/2,left:(Mercury.displayRect.width-t)/2})},fillDisplay:function(){var e;return e=[Mercury.I18n("Name: %s",this.file.name),Mercury.I18n("Size: %s",this.file.readableSize),Mercury.I18n("Type: %s",this.file.type)],this.element.find(".mercury-uploader-details").html(e.join("<br/>"))},loadImage:function(){var e=this;return Mercury.uploader.fileReaderSupported()?this.file.readAsDataURL(function(t){return e.element.find(".mercury-uploader-preview b").html(jQuery("<img>",{src:t})),e.upload()}):this.upload()},upload:function(){var e,t,n=this;return t=new XMLHttpRequest,jQuery.each(["onloadstart","onprogress","onload","onabort","onerror"],function(e,r){return t.upload[r]=function(e){return n.uploaderEvents[r].call(n,e)}}),t.onload=function(e){var t,r;if(e.currentTarget.status>=400)return n.updateStatus("Error: Unable to upload the file"),Mercury.notify("Unable to process response: %s",e.currentTarget.status),n.hide();try{t=Mercury.config.uploading.handler?Mercury.config.uploading.handler(e.target.responseText):jQuery.parseJSON(e.target.responseText),r=t.url||t.image.url;if(!r)throw"Malformed response from server.";return Mercury.trigger("action",{action:"insertImage",value:{src:r}}),n.hide()}catch(i){return n.updateStatus("Error: Unable to upload the file"),Mercury.notify("Unable to process response: %s",i),n.hide()}},t.open("post",Mercury.config.uploading.url,!0),t.setRequestHeader("Accept","application/json, text/javascript, text/html, application/xml, text/xml, */*"),t.setRequestHeader("X-Requested-With","XMLHttpRequest"),t.setRequestHeader(Mercury.config.csrfHeader,Mercury.csrfToken),Mercury.uploader.fileReaderSupported()?this.file.readAsBinaryString(function(e){var r;return r=new Mercury.uploader.MultiPartPost(Mercury.config.uploading.inputName,n.file,e),n.file.updateSize(r.delta),t.setRequestHeader("Content-Type","multipart/form-data; boundary="+r.boundary),t.sendAsBinary(r.body)}):(e=new FormData,e.append(Mercury.config.uploading.inputName,this.file.file,this.file.file.name),t.send(e))},updateStatus:function(e,t){var n;this.element.find(".mercury-uploader-progress span").html(Mercury.I18n(e).toString());if(t)return n=Math.floor(t*100/this.file.size)+"%",this.element.find(".mercury-uploader-indicator div").css({width:n}),this.element.find(".mercury-uploader-indicator b").html(n).show()},hide:function(e){var t=this;return e==null&&(e=0),setTimeout(function(){return t.element.animate({opacity:0},200,"easeInOutSine",function(){return t.overlay.animate({opacity:0},200,"easeInOutSine",function(){return t.overlay.hide(),t.element.hide(),t.reset(),t.visible=!1,Mercury.trigger("focus:frame")})})},e*1e3)},reset:function(){return this.element.find(".mercury-uploader-preview b").html(""),this.element.find(".mercury-uploader-indicator div").css({width:0}),this.element.find(".mercury-uploader-indicator b").html("0%").hide(),this.updateStatus("Processing...")},uploaderEvents:{onloadstart:function(){return this.updateStatus("Uploading...")},onprogress:function(e){return this.updateStatus("Uploading...",e.loaded)},onabort:function(){return this.updateStatus("Aborted"),this.hide(1)},onload:function(){return this.updateStatus("Successfully uploaded...",this.file.size)},onerror:function(){return this.updateStatus("Error: Unable to upload the file"),this.hide(3)}}}),Mercury.uploader.File=function(){function e(e){var t;this.file=e,this.fullSize=this.size=this.file.size||this.file.fileSize,this.readableSize=this.size.toBytes(),this.name=this.file.name||this.file.fileName,this.type=this.file.type||this.file.fileType,t=[],this.size>=Mercury.config.uploading.maxFileSize&&t.push(Mercury.I18n("Too large")),Mercury.config.uploading.allowedMimeTypes.indexOf(this.type)>-1||t.push(Mercury.I18n("Unsupported format")),t.length&&(this.errors=t.join(" / "))}return e.prototype.readAsDataURL=function(e){var t,n=this;return e==null&&(e=null),t=new FileReader,t.readAsDataURL(this.file),t.onload=function(){if(e)return e(t.result)}},e.prototype.readAsBinaryString=function(e){var t,n=this;return e==null&&(e=null),t=new FileReader,t.readAsBinaryString(this.file),t.onload=function(){if(e)return e(t.result)}},e.prototype.updateSize=function(e){return this.fullSize=this.size+e},e}(),Mercury.uploader.MultiPartPost=function(){function t(e,t,n,r){this.inputName=e,this.file=t,this.contents=n,this.formInputs=r!=null?r:{},this.boundary="Boundaryx20072377098235644401115438165x",this.body="",this.buildBody(),this.delta=this.body.length-this.file.size}return t.prototype.buildBody=function(){var t,n,r,i;t="--"+this.boundary,i=this.formInputs;for(n in i){if(!e.call(i,n))continue;r=i[n],this.body+=""+t+'\r\nContent-Disposition: form-data; name="'+n+'"\r\n\r\n'+unescape(encodeURIComponent(r))+"\r\n"}return this.body+=""+t+'\r\nContent-Disposition: form-data; name="'+this.inputName+'"; filename="'+this.file.name+'"\r\nContent-Type: '+this.file.type+"\r\nContent-Transfer-Encoding: binary\r\n\r\n"+this.contents+"\r\n"+t+"--"},t}()}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t},n=[].indexOf||function(e){for(var t=0,n=this.length;t<n;t++)if(t in this&&this[t]===e)return t;return-1};this.Mercury.Regions.Full=function(e){function i(e,t,n){this.element=e,this.window=t,this.options=n!=null?n:{},i.__super__.constructor.apply(this,arguments)}var r;return t(i,e),i.supported=document.designMode&&!jQuery.browser.konqueror&&(!jQuery.browser.msie||jQuery.browser.msie&&parseFloat(jQuery.browser.version,10)>=10),i.supportedText="Chrome 10+, Firefox 4+, Safari 5+, Opera 11.64+",r="full",i.prototype.type=function(){return r},i.prototype.build=function(){var e,t,n,r;jQuery.browser.mozilla&&this.content()===""&&this.content("&nbsp;"),this.element.data({originalOverflow:this.element.css("overflow")}),this.element.css({overflow:"auto"}),this.specialContainer=jQuery.browser.mozilla&&this.element.get(0).tagName!=="DIV",this.element.get(0).contentEditable=!0,r=this.element.find("[data-snippet]");for(t=0,n=r.length;t<n;t++)e=r[t],e.contentEditable=!1,jQuery(e).attr("data-version","1");if(!this.document.mercuryEditing){this.document.mercuryEditing=!0;try{return this.document.execCommand("styleWithCSS",!1,!1),this.document.execCommand("insertBROnReturn",!1,!0),this.document.execCommand("enableInlineTableEditing",!1,!1),this.document.execCommand("enableObjectResizing",!1,!1)}catch(i){}}},i.prototype.bindEvents=function(){var e=this;return i.__super__.bindEvents.apply(this,arguments),Mercury.on("region:update",function(){var t,n,r;if(e.previewing||Mercury.region!==e)return;setTimeout(function(){return e.selection().forceSelection(e.element.get(0))},1),n=e.currentElement();if(n.length)return r=n.closest("table",e.element),r.length&&Mercury.tableEditor(r,n.closest("tr, td"),"<br/>"),t=n.closest("a",e.element),t.length&&t.attr("href")?Mercury.tooltip(t,'<a href="'+t.attr("href")+'" target="_blank">'+t.attr("href")+"</a>",{position:"below"}):Mercury.tooltip.hide()}),this.element.on("dragenter",function(t){if(e.previewing)return;return Mercury.snippet||t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy"}),this.element.on("dragover",function(t){if(e.previewing)return;Mercury.snippet||t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy";if(jQuery.browser.webkit)return clearTimeout(e.dropTimeout),e.dropTimeout=setTimeout(function(){return e.element.trigger("possible:drop")},10)}),this.element.on("drop",function(t){if(e.previewing)return;clearTimeout(e.dropTimeout),e.dropTimeout=setTimeout(function(){return e.element.trigger("possible:drop")},1);if(!t.originalEvent.dataTransfer.files.length)return;return t.preventDefault(),e.focus(),Mercury.uploader(t.originalEvent.dataTransfer.files[0])}),this.element.on("possible:drop",function(){var t;if(e.previewing)return;if(t=e.element.find("img[data-snippet]").get(0))return e.focus(),Mercury.Snippet.displayOptionsFor(jQuery(t).data("snippet"),{},jQuery(t).data("options")),e.document.execCommand("undo",!1,null)}),this.element.on("paste",function(t){if(e.previewing||Mercury.region!==e)return;if(e.specialContainer){t.preventDefault();return}if(e.pasting)return;return Mercury.changes=!0,e.handlePaste(t.originalEvent)}),this.element.on("focus",function(){if(e.previewing)return;return Mercury.region=e,setTimeout(function(){return e.selection().forceSelection(e.element.get(0))},1),Mercury.trigger("region:focused",{region:e})}),this.element.on("blur",function(){if(e.previewing)return;return Mercury.trigger("region:blurred",{region:e}),Mercury.tooltip.hide()}),this.element.on("click",function(t){if(e.previewing)return jQuery(t.target).closest("a").attr("target","_parent")}),this.element.on("dblclick",function(t){var n;if(e.previewing)return;n=jQuery(t.target).closest("img",e.element);if(n.length)return e.selection().selectNode(n.get(0),!0),Mercury.trigger("button",{action:"insertMedia"})}),this.element.on("mouseup",function(){if(e.previewing)return;return e.pushHistory(),Mercury.trigger("region:update",{region:e})}),this.element.on("keydown",function(t){var n;if(e.previewing)return;switch(t.keyCode){case 90:if(!t.metaKey)return;t.preventDefault(),t.shiftKey?e.execCommand("redo"):e.execCommand("undo");return;case 13:if(jQuery.browser.webkit&&e.selection().commonAncestor().closest("li, ul, ol",e.element).length===0)t.preventDefault(),e.document.execCommand("insertParagraph",!1,null);else if(e.specialContainer||jQuery.browser.opera)t.preventDefault(),e.document.execCommand("insertHTML",!1,"<br/>");break;case 9:t.preventDefault(),n=e.selection().commonAncestor(),n.closest("li",e.element).length?t.shiftKey?n.parents("ul, ol").length>1&&e.execCommand("outdent"):e.execCommand("indent"):e.execCommand("insertHTML",{value:"&nbsp;&nbsp;"})}if(t.metaKey)switch(t.keyCode){case 66:e.execCommand("bold"),t.preventDefault();break;case 73:e.execCommand("italic"),t.preventDefault();break;case 85:e.execCommand("underline"),t.preventDefault()}return e.pushHistory(t.keyCode)}),this.element.on("keyup",function(){if(e.previewing)return;return Mercury.trigger("region:update",{region:e}),Mercury.changes=!0})},i.prototype.focus=function(){var e=this;if(Mercury.region!==this){setTimeout(function(){return e.element.focus()},10);try{this.selection().selection.collapseToStart()}catch(t){}}else setTimeout(function(){return e.element.focus()},10);return Mercury.trigger("region:focused",{region:this}),Mercury.trigger("region:update",{region:this})},i.prototype.content=function(e,t,n){var r,i,s,o,u,a,f,l,c,h,p,d,v;e==null&&(e=null),t==null&&(t=!0),n==null&&(n=!1);if(e!==null){r=jQuery("<div>").appendTo(this.document.createDocumentFragment()),r.html(e),d=r.find("[data-snippet]");for(l=0,h=d.length;l<h;l++){s=d[l],s.contentEditable=!1,s=jQuery(s);if(a=Mercury.Snippet.find(s.data("snippet")))if(s.data("version"))a.setVersion(s.data("version"));else try{f=parseInt(s.html().match(/\/(\d+)\]/)[1]),f&&(a.setVersion(f),s.attr({"data-version":f}),s.html(a.data))}catch(m){}}return this.element.html(r.html()),this.selection().selectMarker(this.element)}this.element.find("meta").remove(),n&&(u=this.selection(),u.placeMarker()),r=jQuery("<div>").appendTo(this.document.createDocumentFragment()),r.html(this.element.html().replace(/^\s+|\s+$/g,""));if(t){v=r.find("[data-snippet]");for(o=c=0,p=v.length;c<p;o=++c){s=v[o],s=jQuery(s);if(a=Mercury.Snippet.find(s.data("snippet")))a.data=s.html();s.html("["+s.data("snippet")+"/"+s.data("version")+"]"),s.attr({contenteditable:null,"data-version":null})}}return i=r.html(),n&&u.removeMarker(),i},i.prototype.togglePreview=function(){return this.previewing?(this.element.get(0).contentEditable=!0,this.element.css({overflow:"auto"})):(this.content(this.content()),this.element.get(0).contentEditable=!1,this.element.css({overflow:this.element.data("originalOverflow")}),this.element.blur()),i.__super__.togglePreview.apply(this,arguments)},i.prototype.execCommand=function(e,t){var n,r;t==null&&(t={}),i.__super__.execCommand.apply(this,arguments);if(n=Mercury.config.behaviors[e]||Mercury.Regions.Full.actions[e])n.call(this,this.selection(),t);else{e==="indent"&&(r=this.element.get(0).previousSibling),e==="insertHTML"&&t.value&&t.value.get&&(t.value=jQuery("<div>").html(t.value).html());try{this.document.execCommand(e,!1,t.value)}catch(s){e==="indent"&&this.element.prev()!==r&&this.element.prev().remove()}}return this.element.find("img").one("error",function(){return jQuery(this).attr({src:"/assets/mercury/missing-image.png",title:"Image not found"})})},i.prototype.pushHistory=function(e){var t,n,r,i=this;return t=[13,46,8],r=2.5,e&&(n=t.indexOf(e)),clearTimeout(this.historyTimeout),n>=0&&n!==this.lastKnownKeyCode?this.history.push(this.content(null,!1,!0)):e?this.historyTimeout=setTimeout(function(){return i.history.push(i.content(null,!1,!0))},r*1e3):this.history.push(this.content(null,!1,!0)),this.lastKnownKeyCode=n},i.prototype.selection=function(){return new Mercury.Regions.Full.Selection(this.window.getSelection(),this.document)},i.prototype.path=function(){var e;return e=this.selection().commonAncestor(),e?e.get(0)===this.element.get(0)?[]:e.parentsUntil(this.element):[]},i.prototype.currentElement=function(){var e,t;return e=[],t=this.selection(),t.range&&(e=t.commonAncestor(),e.get(0).nodeType===3&&(e=e.parent())),e},i.prototype.handlePaste=function(e){var t,n,r=this;if(Mercury.config.pasting.sanitize!=="text"||!e.clipboardData)return n=this.selection(),n.placeMarker(),t=jQuery("#mercury_sanitizer",this.document).focus(),setTimeout(function(){var e;return e=r.sanitize(t),n.selectMarker(r.element),n.removeMarker(),r.element.focus(),r.execCommand("insertHTML",{value:e})},1);this.execCommand("insertHTML",{value:e.clipboardData.getData("text/plain")}),e.preventDefault()},i.prototype.sanitize=function(e){var t,r,i,s,o,u,a,f,l,c,h,p,d,v;e.find("["+Mercury.config.regions.attribute+"]").remove(),e.find('[src*="webkit-fake-url://"]').remove();if(Mercury.config.pasting.sanitize)switch(Mercury.config.pasting.sanitize){case"blacklist":e.find("[style]").removeAttr("style"),e.find('[class="Apple-style-span"]').removeClass("Apple-style-span"),o=e.html();break;case"whitelist":h=e.find("*");for(a=0,l=h.length;a<l;a++){u=h[a],t=!1,p=Mercury.config.pasting.whitelist;for(i in p){r=p[i];if(u.tagName.toLowerCase()===i.toLowerCase()){t=!0,d=jQuery(u.attributes);for(f=0,c=d.length;f<c;f++)s=d[f],(v=s.name,n.call(r,v)<0)&&jQuery(u).removeAttr(s.name);break}}t||jQuery(u).replaceWith(jQuery(u).contents())}o=e.html();break;default:o=e.text()}else{o=e.html();if(o.indexOf("<!--StartFragment-->")>-1||o.indexOf('="mso-')>-1||o.indexOf("<o:")>-1||o.indexOf('="Mso')>-1)o=e.text()}return e.html(""),o},i.actions={insertRowBefore:function(){return Mercury.tableEditor.addRow("before")},insertRowAfter:function(){return Mercury.tableEditor.addRow("after")},insertColumnBefore:function(){return Mercury.tableEditor.addColumn("before")},insertColumnAfter:function(){return Mercury.tableEditor.addColumn("after")},deleteColumn:function(){return Mercury.tableEditor.removeColumn()},deleteRow:function(){return Mercury.tableEditor.removeRow()},increaseColspan:function(){return Mercury.tableEditor.increaseColspan()},decreaseColspan:function(){return Mercury.tableEditor.decreaseColspan()},increaseRowspan:function(){return Mercury.tableEditor.increaseRowspan()},decreaseRowspan:function(){return Mercury.tableEditor.decreaseRowspan()},undo:function(){return this.content(this.history.undo())},redo:function(){return this.content(this.history.redo())},horizontalRule:function(){return this.execCommand("insertHTML",{value:"<hr/>"})},removeFormatting:function(e){return e.insertTextNode(e.textContent())},backColor:function(e,t){return e.wrap('<span style="background-color:'+t.value.toHex()+'">',!0)},overline:function(e){return e.wrap('<span style="text-decoration:overline">',!0)},style:function(e,t){return e.wrap('<span class="'+t.value+'">',!0)},replaceHTML:function(e,t){return this.content(t.value)},insertImage:function(e,t){return this.execCommand("insertHTML",{value:jQuery("<img/>",t.value)})},insertTable:function(e,t){return this.execCommand("insertHTML",{value:t.value})},insertLink:function(e,t){var n;return n=jQuery("<"+t.value.tagName+">",this.document).attr(t.value.attrs).html(t.value.content),e.insertNode(n)},replaceLink:function(e,t){var n,r;return n=jQuery("<"+t.value.tagName+">",this.document).attr(t.value.attrs).html(t.value.content),e.selectNode(t.node),r=jQuery("<div>").html(e.content()).find("a").html(),e.replace(jQuery(n,e.context).html(r))},insertSnippet:function(e,t){var n,r;return r=t.value,(n=this.element.find("[data-snippet="+r.identity+"]")).length&&e.selectNode(n.get(0)),e.insertNode(r.getHTML(this.document))},editSnippet:function(){var e;if(!this.snippet)return;return e=Mercury.Snippet.find(this.snippet.data("snippet")),e.displayOptions()},removeSnippet:function(){return this.snippet&&this.snippet.remove(),Mercury.trigger("hide:toolbar",{type:"snippet",immediately:!0})}},i}(Mercury.Region),Mercury.Regions.Full.Selection=function(){function e(e,t){this.selection=e,this.context=t;if(!(this.selection.rangeCount>=1))return;this.range=this.selection.getRangeAt(0),this.fragment=this.range.cloneContents(),this.clone=this.range.cloneRange(),this.collapsed=this.selection.isCollapsed}return e.prototype.commonAncestor=function(e){var t;return e==null&&(e=!1),this.range?(t=this.range.commonAncestorContainer,t.nodeType===3&&e&&(t=t.parentNode),jQuery(t)):null},e.prototype.wrap=function(e,t){return t==null&&(t=!1),e=jQuery(e,this.context).html(this.fragment),t&&this.replace(e),e},e.prototype.textContent=function(){return this.content().textContent},e.prototype.htmlContent=function(){var e;return e=this.content(),jQuery("<div>").html(e).html()},e.prototype.content=function(){return this.range.cloneContents()},e.prototype.is=function(e){var t;return t=this.content(),jQuery(t).length===1&&jQuery(t.firstChild).is(e)?jQuery(t.firstChild):!1},e.prototype.forceSelection=function(e){var t,n;if(!jQuery.browser.webkit)return;n=this.context.createRange(),this.range?this.commonAncestor(!0).closest("[data-snippet]").length&&(t=this.context.createTextNode("\0"),e.appendChild(t)):e.lastChild&&e.lastChild.nodeType===3&&e.lastChild.textContent.replace(/^[\s+|\n+]|[\s+|\n+]$/,"")===""?(t=e.lastChild,e.lastChild.textContent="\0"):(t=this.context.createTextNode("\0"),e.appendChild(t));if(t)return n.setStartBefore(t),n.setEndBefore(t),this.selection.addRange(n)},e.prototype.selectMarker=function(e){var t,n;t=e.find("em.mercury-marker");if(!t.length)return;return n=this.context.createRange(),n.setStartBefore(t.get(0)),t.length>=2&&n.setEndBefore(t.get(1)),t.remove(),this.selection.removeAllRanges(),this.selection.addRange(n)},e.prototype.placeMarker=function(){var e,t;if(!this.range)return;return this.startMarker=jQuery('<em class="mercury-marker"/>',this.context).get(0),this.endMarker=jQuery('<em class="mercury-marker"/>',this.context).get(0),e=this.range.cloneRange(),e.collapse(!1),e.insertNode(this.endMarker),this.range.collapsed||(t=this.range.cloneRange(),t.collapse(!0),t.insertNode(this.startMarker)),this.selection.removeAllRanges(),this.selection.addRange(this.range)},e.prototype.removeMarker=function(){return jQuery(this.startMarker).remove(),jQuery(this.endMarker).remove()},e.prototype.insertTextNode=function(e){var t;return t=this.context.createTextNode(e),this.range.extractContents(),this.range.insertNode(t),this.range.selectNodeContents(t),this.selection.addRange(this.range)},e.prototype.insertNode=function(e){return e.get&&(e=e.get(0)),jQuery.type(e)==="string"&&(e=jQuery(e,this.context).get(0)),this.range.deleteContents(),this.range.insertNode(e),this.range.selectNodeContents(e),this.selection.addRange(this.range)},e.prototype.selectNode=function(e,t){return t==null&&(t=!1),this.range.selectNode(e),t&&this.selection.removeAllRanges(),this.selection.addRange(this.range)},e.prototype.replace=function(e,t){e.get&&(e=e.get(0)),jQuery.type(e)==="string"&&(e=jQuery(e,this.context).get(0)),this.range.deleteContents(),this.range.insertNode(e),this.range.selectNodeContents(e),this.selection.addRange(this.range);if(t)return this.range.collapse(!1)},e}()}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Regions.Image=function(e){function r(e,t,n){this.element=e,this.window=t,this.options=n!=null?n:{},r.__super__.constructor.apply(this,arguments)}var n;return t(r,e),r.supported=document.getElementById,r.supportedText="Chrome 10+, Firefox 4+, IE 7+, Safari 5+, Opera 8+",n="image",r.prototype.type=function(){return n},r.prototype.bindEvents=function(){var e=this;return Mercury.on("mode",function(t,n){if(n.mode==="preview")return e.togglePreview()}),Mercury.on("focus:frame",function(){if(e.previewing||Mercury.region!==e)return;return e.focus()}),Mercury.on("action",function(t,n){if(e.previewing||Mercury.region!==e)return;if(n.action)return e.execCommand(n.action,n)}),this.element.on("dragenter",function(t){if(e.previewing)return;return t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy"}),this.element.on("dragover",function(t){if(e.previewing)return;return t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy"}),this.element.on("drop",function(t){if(e.previewing)return;if(t.originalEvent.dataTransfer.files.length)return t.preventDefault(),e.focus(),Mercury.uploader(t.originalEvent.dataTransfer.files[0])}),this.element.on("focus",function(){return e.focus()})},r.prototype.togglePreview=function(){return this.previewing?(this.previewing=!1,this.element.attr(Mercury.config.regions.attribute,n),this.build()):(this.previewing=!0,this.element.removeAttr(Mercury.config.regions.attribute),Mercury.trigger("region:blurred",{region:this}))},r.prototype.focus=function(){if(this.previewing)return;return Mercury.region=this,Mercury.trigger("region:focused",{region:this}),Mercury.trigger("region:update",{region:this})},r.prototype.execCommand=function(e,t){var n;t==null&&(t={}),r.__super__.execCommand.apply(this,arguments);if(n=Mercury.Regions.Image.actions[e])return n.call(this,t)},r.prototype.pushHistory=function(){return this.history.push({src:this.element.attr("src")})},r.prototype.updateSrc=function(e){return this.element.attr("src",e)},r.prototype.serialize=function(){return{type:n,data:this.dataAttributes(),attributes:{src:this.element.attr("src")}}},r.actions={undo:function(){var e;if(e=this.history.undo())return this.updateSrc(e.src)},redo:function(){var e;if(e=this.history.redo())return this.updateSrc(e.src)},insertImage:function(e){return this.updateSrc(e.value.src)}},r}(Mercury.Region)}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Regions.Markdown=function(e){function r(e,t,n){this.element=e,this.window=t,this.options=n!=null?n:{},r.__super__.constructor.apply(this,arguments),this.converter=new Showdown.converter}var n;return t(r,e),r.supported=document
.getElementById,r.supportedText="Chrome 10+, Firefox 4+, IE 7+, Safari 5+, Opera 8+",n="markdown",r.prototype.type=function(){return n},r.prototype.build=function(){var e,t,n;return n="100%",e=this.element.height(),t=this.element.html().replace(/^\s+|\s+$/g,"").replace("&gt;",">"),this.textarea=jQuery("<textarea>",this.document).val(t).addClass("mercury-textarea"),this.textarea.css({border:0,background:"transparent",display:"block","overflow-y":"hidden",width:n,height:e,fontFamily:'"Courier New", Courier, monospace'}),this.element.empty().append(this.textarea),this.previewElement=jQuery("<div>",this.document),this.element.append(this.previewElement),this.container=this.element,this.container.data("region",this),this.element=this.textarea,this.resize()},r.prototype.dataAttributes=function(){var e,t,n,r,i;t={},i=Mercury.config.regions.dataAttributes;for(n=0,r=i.length;n<r;n++)e=i[n],t[e]=this.container.attr("data-"+e);return t},r.prototype.bindEvents=function(){var e=this;return Mercury.on("mode",function(t,n){if(n.mode==="preview")return e.togglePreview()}),Mercury.on("focus:frame",function(){if(!e.previewing&&Mercury.region===e)return e.focus()}),Mercury.on("action",function(t,n){if(e.previewing||Mercury.region!==e)return;if(n.action)return e.execCommand(n.action,n)}),Mercury.on("unfocus:regions",function(){if(e.previewing||Mercury.region!==e)return;return e.element.blur(),e.container.removeClass("focus"),Mercury.trigger("region:blurred",{region:e})}),this.element.on("dragenter",function(t){if(e.previewing)return;return t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy"}),this.element.on("dragover",function(t){if(e.previewing)return;return t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy"}),this.element.on("drop",function(t){if(e.previewing)return;Mercury.snippet&&(t.preventDefault(),e.focus(),Mercury.Snippet.displayOptionsFor(Mercury.snippet.name,{},Mercury.snippet.hasOptions));if(t.originalEvent.dataTransfer.files.length)return t.preventDefault(),e.focus(),Mercury.uploader(t.originalEvent.dataTransfer.files[0])}),this.element.on("focus",function(){if(e.previewing)return;return Mercury.region=e,e.container.addClass("focus"),Mercury.trigger("region:focused",{region:e})}),this.element.on("keydown",function(t){var n,r,i,s,o,u;if(e.previewing)return;e.resize();switch(t.keyCode){case 90:if(!t.metaKey)return;t.preventDefault(),t.shiftKey?e.execCommand("redo"):e.execCommand("undo");return;case 13:s=e.selection(),u=e.element.val(),o=u.lastIndexOf("\n",s.start),n=u.indexOf("\n",s.end),n<o&&(n=u.length),u[o]==="\n"&&(o=u.lastIndexOf("\n",s.start-1)),u[o+1]==="-"&&(s.replace("\n- ",!1,!0),t.preventDefault()),/\d/.test(u[o+1])&&(r=u.substring(o,n),/(\d+)\./.test(r)&&(i=parseInt(RegExp.$1),s.replace("\n"+(i+=1)+". ",!1,!0),t.preventDefault()));break;case 9:t.preventDefault(),e.execCommand("insertHTML",{value:"  "})}if(t.metaKey)switch(t.keyCode){case 66:e.execCommand("bold"),t.preventDefault();break;case 73:e.execCommand("italic"),t.preventDefault();break;case 85:e.execCommand("underline"),t.preventDefault()}return e.pushHistory(t.keyCode)}),this.element.on("keyup",function(){if(e.previewing)return;return Mercury.changes=!0,e.resize(),Mercury.trigger("region:update",{region:e})}),this.element.on("mouseup",function(){if(e.previewing)return;return e.focus(),Mercury.trigger("region:focused",{region:e})}),this.previewElement.on("click",function(t){if(e.previewing)return $(t.target).closest("a").attr("target","_parent")})},r.prototype.focus=function(){return this.element.focus()},r.prototype.content=function(e,t){return e==null&&(e=null),t==null&&(t=!0),e!==null?jQuery.type(e)==="string"?this.element.val(e):(this.element.val(e.html),this.selection().select(e.selection.start,e.selection.end)):this.element.val()},r.prototype.contentAndSelection=function(){return{html:this.content(null,!1),selection:this.selection().serialize()}},r.prototype.togglePreview=function(){var e;if(!this.previewing)return this.previewing=!0,this.container.removeAttr(Mercury.config.regions.attribute),e=this.converter.makeHtml(this.element.val()),this.previewElement.html(e),this.previewElement.show(),this.element.hide(),Mercury.trigger("region:blurred",{region:this});this.previewing=!1,this.container.attr(Mercury.config.regions.attribute,n),this.previewElement.hide(),this.element.show();if(Mercury.region===this)return this.focus()},r.prototype.execCommand=function(e,t){var n;return t==null&&(t={}),r.__super__.execCommand.apply(this,arguments),(n=Mercury.Regions.Markdown.actions[e])&&n.call(this,this.selection(),t),this.resize()},r.prototype.pushHistory=function(e){var t,n,r,i=this;return t=[13,46,8],r=2.5,e&&(n=t.indexOf(e)),clearTimeout(this.historyTimeout),n>=0&&n!==this.lastKnownKeyCode?this.history.push(this.contentAndSelection()):e?this.historyTimeout=setTimeout(function(){return i.history.push(i.contentAndSelection())},r*1e3):this.history.push(this.contentAndSelection()),this.lastKnownKeyCode=n},r.prototype.selection=function(){return new Mercury.Regions.Markdown.Selection(this.element)},r.prototype.resize=function(){return this.element.css({height:this.element.get(0).scrollHeight-100}),this.element.css({height:this.element.get(0).scrollHeight})},r.prototype.snippets=function(){},r.actions={undo:function(){return this.content(this.history.undo())},redo:function(){return this.content(this.history.redo())},insertHTML:function(e,t){var n;return t.value.get&&(n=t.value.get(0))&&(t.value=jQuery("<div>").html(n).html()),e.replace(t.value,!1,!0)},insertImage:function(e,t){return e.replace("![add alt text]("+encodeURI(t.value.src)+")",!0)},insertTable:function(e,t){return e.replace(t.value.replace(/<br>|<br\/>/ig,""),!1,!0)},insertLink:function(e,t){return e.replace("["+t.value.content+"]("+t.value.attrs.href+" 'optional title')",!0)},insertUnorderedList:function(e){return e.addList("unordered")},insertOrderedList:function(e){return e.addList("ordered")},style:function(e,t){return e.wrap('<span class="'+t.value+'">',"</span>")},formatblock:function(e,t){var n,r,i;i={h1:["# "," #"],h2:["## "," ##"],h3:["### "," ###"],h4:["#### "," ####"],h5:["##### "," #####"],h6:["###### "," ######"],pre:["    ",""],blockquote:["> ",""],p:["\n","\n"]};for(r in i)n=i[r],e.unWrapLine(""+n[0],""+n[1]);if(t.value==="blockquote"){Mercury.Regions.Markdown.actions.indent.call(this,e,t);return}return e.wrapLine(""+i[t.value][0],""+i[t.value][1])},bold:function(e){return e.wrap("**","**")},italic:function(e){return e.wrap("_","_")},subscript:function(e){return e.wrap("<sub>","</sub>")},superscript:function(e){return e.wrap("<sup>","</sup>")},indent:function(e){return e.wrapLine("> ","",!1,!0)},outdent:function(e){return e.unWrapLine("> ","",!1,!0)},horizontalRule:function(e){return e.replace("\n- - -\n")},insertSnippet:function(e,t){var n;return n=t.value,e.replace(n.getText())}},r}(Mercury.Region),Mercury.Regions.Markdown.Selection=function(){function e(e){this.element=e,this.el=this.element.get(0),this.getDetails()}return e.prototype.serialize=function(){return{start:this.start,end:this.end}},e.prototype.getDetails=function(){return this.length=this.el.selectionEnd-this.el.selectionStart,this.start=this.el.selectionStart,this.end=this.el.selectionEnd,this.text=this.element.val().substr(this.start,this.length)},e.prototype.replace=function(e,t,n){var r,i,s;return t==null&&(t=!1),n==null&&(n=!1),this.getDetails(),s=this.element.val(),i=this.element.val(),this.element.val(s.substr(0,this.start)+e+s.substr(this.end,s.length)),r=this.element.val()!==i,t&&this.select(this.start,this.start+e.length),n&&this.select(this.start+e.length,this.start+e.length),r},e.prototype.select=function(e,t){return this.start=e,this.end=t,this.element.focus(),this.el.selectionStart=this.start,this.el.selectionEnd=this.end,this.getDetails()},e.prototype.wrap=function(e,t){this.getDetails(),this.deselectNewLines(),this.replace(e+this.text+t,this.text!=="");if(this.text==="")return this.select(this.start+e.length,this.start+e.length)},e.prototype.wrapLine=function(e,t,n,r){var i,s,o,u;n==null&&(n=!0),r==null&&(r=!1),this.getDetails(),s=this.serialize(),u=this.element.val(),o=u.lastIndexOf("\n",this.start),i=u.indexOf("\n",this.end),i<o&&(i=u.length),u[o]==="\n"&&(o=u.lastIndexOf("\n",this.start-1)),this.select(o+1,i),this.replace(e+this.text+t,n);if(r)return this.select(s.start+e.length,s.end+e.length)},e.prototype.unWrapLine=function(e,t,n,r){var i,s,o,u,a,f,l;n==null&&(n=!0),r==null&&(r=!1),this.getDetails(),a=this.serialize(),l=this.element.val(),f=l.lastIndexOf("\n",this.start),s=l.indexOf("\n",this.end),s<f&&(s=l.length),l[f]==="\n"&&(f=l.lastIndexOf("\n",this.start-1)),this.select(f+1,s),window.something=this.text,o=new RegExp("^"+e.regExpEscape()),u=new RegExp(""+t.regExpEscape()+"$"),i=this.replace(this.text.replace(o,"").replace(u,""),n);if(r&&i)return this.select(a.start-e.length,a.end-e.length)},e.prototype.addList=function(e){var t,n,r,i,s,o;return o=this.element.val(),s=o.lastIndexOf("\n",this.start),t=o.indexOf("\n",this.end),t<s&&(t=o.length),o[s]==="\n"&&(s=o.lastIndexOf("\n",this.start-1)),this.select(s+1,t),i=this.text.split("\n"),e==="unordered"?this.replace("- "+i.join("\n- "),!0):this.replace(function(){var e,t,s;s=[];for(n=e=0,t=i.length;e<t;n=++e)r=i[n],s.push(""+(n+1)+". "+r);return s}().join("\n"),!0)},e.prototype.deselectNewLines=function(){var e,t;return t=this.text,e=t.replace(/\n+$/g,"").length,this.select(this.start,this.start+e)},e.prototype.placeMarker=function(){return this.wrap("[mercury-marker]","[mercury-marker]")},e.prototype.removeMarker=function(){var e,t,n;n=this.element.val(),t=n.indexOf("[mercury-marker]");if(t>-1)return e=n.indexOf("[mercury-marker]",t+1)-"[mercury-marker]".length,this.element.val(this.element.val().replace(/\[mercury-marker\]/g,"")),this.select(t,e);return},e.prototype.textContent=function(){return this.text},e}()}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Regions.Simple=function(e){function r(e,t,i){this.element=e,this.window=t,this.options=i!=null?i:{},Mercury.log("building "+n,this.element,this.options),r.__super__.constructor.apply(this,arguments)}var n;return t(r,e),r.supported=document.getElementById,r.supportedText="Chrome 10+, Firefox 4+, IE 7+, Safari 5+, Opera 8+",n="simple",r.prototype.type=function(){return n},r.prototype.build=function(){var e,t,n;return this.element.css("display")==="block"?(n="100%",e=this.element.height()):(n=this.element.width(),e=this.element.height()),t=this.element.text(),this.textarea=jQuery("<textarea>",this.document).val(t).addClass("mercury-textarea"),this.textarea.css({border:0,background:"transparent","overflow-y":"hidden",width:n,height:e,fontFamily:"inherit",fontSize:"inherit",fontWeight:"inherit",fontStyle:"inherit",color:"inherit","min-height":0,padding:"0",margin:0,"border-radius":0,display:"inherit",lineHeight:"inherit",textAlign:"inherit"}),this.element.empty().append(this.textarea),this.container=this.element,this.container.data("region",this),this.element=this.textarea,this.resize()},r.prototype.bindEvents=function(){var e=this;return Mercury.on("mode",function(t,n){if(n.mode==="preview")return e.togglePreview()}),Mercury.on("focus:frame",function(){if(!e.previewing&&Mercury.region===e)return e.focus()}),Mercury.on("action",function(t,n){if(e.previewing||Mercury.region!==e)return;if(n.action)return e.execCommand(n.action,n)}),Mercury.on("unfocus:regions",function(){if(e.previewing||Mercury.region!==e)return;return e.element.blur(),e.container.removeClass("focus"),Mercury.trigger("region:blurred",{region:e})}),this.bindElementEvents()},r.prototype.bindElementEvents=function(){var e=this;return this.element.on("focus",function(){if(e.previewing)return;return Mercury.region=e,e.container.addClass("focus"),Mercury.trigger("region:focused",{region:e})}),this.element.on("keydown",function(t){var n,r,i,s,o,u;if(e.previewing)return;e.resize();switch(t.keyCode){case 90:if(!t.metaKey)return;t.preventDefault(),t.shiftKey?e.execCommand("redo"):e.execCommand("undo");return;case 13:s=e.selection(),u=e.element.val(),o=u.lastIndexOf("\n",s.start),n=u.indexOf("\n",s.end),n<o&&(n=u.length),u[o]==="\n"&&(o=u.lastIndexOf("\n",s.start-1)),u[o+1]==="-"&&s.replace("\n- ",!1,!0),/\d/.test(u[o+1])&&(r=u.substring(o,n),/(\d+)\./.test(r)&&(i=parseInt(RegExp.$1),s.replace("\n"+(i+=1)+". ",!1,!0))),t.preventDefault();break;case 9:t.preventDefault(),e.execCommand("insertHTML",{value:"  "})}return e.pushHistory(t.keyCode)}),this.element.on("keyup",function(){if(e.previewing)return;return Mercury.changes=!0,e.resize(),Mercury.trigger("region:update",{region:e})}),this.element.on("mouseup",function(){if(e.previewing)return;return e.focus(),Mercury.trigger("region:focused",{region:e})}),this.element.on("paste",function(t){if(e.previewing||Mercury.region!==e)return;if(e.specialContainer){t.preventDefault();return}if(e.pasting)return;return Mercury.changes=!0,e.handlePaste(t.originalEvent)})},r.prototype.handlePaste=function(e){this.execCommand("insertHTML",{value:e.clipboardData.getData("text/plain").replace(/\n/g," ")}),e.preventDefault()},r.prototype.path=function(){return[this.container.get(0)]},r.prototype.focus=function(){return this.element.focus()},r.prototype.content=function(e,t){return e==null&&(e=null),t==null&&(t=!0),e!==null?jQuery.type(e)==="string"?this.element.val(e):(this.element.val(e.html),this.selection().select(e.selection.start,e.selection.end)):this.element.val()},r.prototype.contentAndSelection=function(){return{html:this.content(null,!1),selection:this.selection().serialize()}},r.prototype.togglePreview=function(){var e;if(!this.previewing)return this.previewing=!0,e=jQuery("<div></div>").text(this.element.val()).html(),this.container.removeAttr(Mercury.config.regions.attribute),this.container.html(e),Mercury.trigger("region:blurred",{region:this});this.previewing=!1,this.element=this.container,this.container.attr(Mercury.config.regions.attribute,n),this.build(),this.bindElementEvents();if(Mercury.region===this)return this.focus()},r.prototype.execCommand=function(e,t){var n;return t==null&&(t={}),r.__super__.execCommand.apply(this,arguments),(n=Mercury.Regions.Simple.actions[e])&&n.call(this,this.selection(),t),this.resize()},r.prototype.pushHistory=function(e){var t,n,r,i=this;return t=[13,46,8],r=2.5,e&&(n=t.indexOf(e)),clearTimeout(this.historyTimeout),n>=0&&n!==this.lastKnownKeyCode?this.history.push(this.contentAndSelection()):e?this.historyTimeout=setTimeout(function(){return i.history.push(i.contentAndSelection())},r*1e3):this.history.push(this.contentAndSelection()),this.lastKnownKeyCode=n},r.prototype.selection=function(){return new Mercury.Regions.Simple.Selection(this.element)},r.prototype.resize=function(){return this.element.css({height:this.element.get(0).scrollHeight-100}),this.element.css({height:this.element.get(0).scrollHeight})},r.prototype.snippets=function(){},r.actions={undo:function(){return this.content(this.history.undo())},redo:function(){return this.content(this.history.redo())},insertHTML:function(e,t){var n;return t.value.get&&(n=t.value.get(0))&&(t.value=jQuery("<div>").html(n).html()),e.replace(t.value,!1,!0)}},r}(Mercury.Region),Mercury.Regions.Simple.Selection=function(){function e(e){this.element=e,this.el=this.element.get(0),this.getDetails()}return e.prototype.serialize=function(){return{start:this.start,end:this.end}},e.prototype.getDetails=function(){return this.length=this.el.selectionEnd-this.el.selectionStart,this.start=this.el.selectionStart,this.end=this.el.selectionEnd,this.text=this.element.val().substr(this.start,this.length)},e.prototype.replace=function(e,t,n){var r,i,s;return t==null&&(t=!1),n==null&&(n=!1),this.getDetails(),s=this.element.val(),i=this.element.val(),this.element.val(s.substr(0,this.start)+e+s.substr(this.end,s.length)),r=this.element.val()!==i,t&&this.select(this.start,this.start+e.length),n&&this.select(this.start+e.length,this.start+e.length),r},e.prototype.select=function(e,t){return this.start=e,this.end=t,this.element.focus(),this.el.selectionStart=this.start,this.el.selectionEnd=this.end,this.getDetails()},e.prototype.wrap=function(e,t){this.getDetails(),this.deselectNewLines(),this.replace(e+this.text+t,this.text!=="");if(this.text==="")return this.select(this.start+e.length,this.start+e.length)},e.prototype.deselectNewLines=function(){var e,t;return t=this.text,e=t.replace(/\n+$/g,"").length,this.select(this.start,this.start+e)},e.prototype.placeMarker=function(){return this.wrap("[mercury-marker]","[mercury-marker]")},e.prototype.removeMarker=function(){var e,t,n;n=this.element.val(),t=n.indexOf("[mercury-marker]");if(t>-1)return e=n.indexOf("[mercury-marker]",t+1)-"[mercury-marker]".length,this.element.val(this.element.val().replace(/\[mercury-marker\]/g,"")),this.select(t,e);return},e.prototype.textContent=function(){return this.text},e}()}.call(this),function(){var e={}.hasOwnProperty,t=function(t,n){function i(){this.constructor=t}for(var r in n)e.call(n,r)&&(t[r]=n[r]);return i.prototype=n.prototype,t.prototype=new i,t.__super__=n.prototype,t};this.Mercury.Regions.Snippets=function(e){function r(e,t,i){this.element=e,this.window=t,this.options=i!=null?i:{},Mercury.log("building "+n,this.element,this.options),r.__super__.constructor.apply(this,arguments),this.makeSortable()}var n;return t(r,e),r.supported=document.getElementById,r.supportedText="Chrome 10+, Firefox 4+, IE 7+, Safari 5+, Opera 8+",n="snippets",r.prototype.type=function(){return n},r.prototype.build=function(){var e,t,n,r;r=this.element.find("[data-snippet]");for(t=0,n=r.length;t<n;t++)e=r[t],jQuery(e).attr("data-version",0);if(this.element.css("minHeight")==="0px")return this.element.css({minHeight:20})},r.prototype.bindEvents=function(){var e=this;return r.__super__.bindEvents.apply(this,arguments),Mercury.on("unfocus:regions",function(t){if(e.previewing)return;if(Mercury.region===e)return e.element.removeClass("focus"),e.element.sortable("destroy"),Mercury.trigger("region:blurred",{region:e})}),Mercury.on("focus:window",function(t){if(e.previewing)return;if(Mercury.region===e)return e.element.removeClass("focus"),e.element.sortable("destroy"),Mercury.trigger("region:blurred",{region:e})}),this.element.on("mouseup",function(){if(e.previewing)return;return e.focus(),Mercury.trigger("region:focused",{region:e})}),this.element.on("dragover",function(t){if(e.previewing)return;return t.preventDefault(),t.originalEvent.dataTransfer.dropEffect="copy"}),this.element.on("drop",function(t){if(e.previewing||!Mercury.snippet)return;return e.focus(),t.preventDefault(),Mercury.Snippet.displayOptionsFor(Mercury.snippet.name,{},Mercury.snippet.hasOptions)}),jQuery(this.document).on("keydown",function(t){if(e.previewing||Mercury.region!==e)return;switch(t.keyCode){case 90:if(!t.metaKey)return;return t.preventDefault(),t.shiftKey?e.execCommand("redo"):e.execCommand("undo")}}),jQuery(this.document).on("keyup",function(){if(e.previewing||Mercury.region!==e)return;return Mercury.changes=!0})},r.prototype.focus=function(){return Mercury.region=this,this.makeSortable(),this.element.addClass("focus")},r.prototype.togglePreview=function(){return this.previewing?this.makeSortable():(this.element.sortable("destroy"),this.element.removeClass("focus")),r.__super__.togglePreview.apply(this,arguments)},r.prototype.execCommand=function(e,t){var n;t==null&&(t={}),r.__super__.execCommand.apply(this,arguments);if(n=Mercury.Regions.Snippets.actions[e])return n.call(this,t)},r.prototype.makeSortable=function(){var e=this;return this.element.sortable("destroy").sortable({document:this.document,scroll:!1,containment:"parent",items:"[data-snippet]",opacity:.4,revert:100,tolerance:"pointer",beforeStop:function(){return Mercury.trigger("hide:toolbar",{type:"snippet",immediately:!0}),!0},stop:function(){return setTimeout(function(){return e.pushHistory()},100),!0}})},r.actions={undo:function(){return this.content(this.history.undo())},redo:function(){return this.content(this.history.redo())},insertSnippet:function(e){var t,n,r=this;return n=e.value,(t=this.element.find("[data-snippet="+n.identity+"]")).length?t.replaceWith(n.getHTML(this.document,function(){return r.pushHistory()})):this.element.append(n.getHTML(this.document,function(){return r.pushHistory()}))},editSnippet:function(){var e;if(!this.snippet)return;return e=Mercury.Snippet.find(this.snippet.data("snippet")),e.displayOptions()},removeSnippet:function(){return this.snippet&&this.snippet.remove(),Mercury.trigger("hide:toolbar",{type:"snippet",immediately:!0})}},r}(Mercury.Region)}.call(this),function(){this.Mercury.dialogHandlers.backColor=function(){var e=this;return this.element.find(".picker, .last-picked").on("click",function(t){var n;return n=jQuery(t.target).css("background-color"),e.element.find(".last-picked").css({background:n}),e.button.css({backgroundColor:n}),Mercury.trigger("action",{action:"backColor",value:n})})}}.call(this),function(){this.Mercury.dialogHandlers.foreColor=function(){var e=this;return this.element.find(".picker, .last-picked").on("click",function(t){var n;return n=jQuery(t.target).css("background-color").toHex(),e.element.find(".last-picked").css({background:n}),e.button.css({backgroundColor:n}),Mercury.trigger("action",{action:"foreColor",value:n})})}}.call(this),function(){this.Mercury.dialogHandlers.formatblock=function(){var e=this;return this.element.find("[data-tag]").on("click",function(e){var t;return t=jQuery(e.target).data("tag"),Mercury.trigger("action",{action:"formatblock",value:t})})}}.call(this),function(){this.Mercury.dialogHandlers.snippetPanel=function(){var e=this;return this.element.find("input.filter").on("keyup",function(){var t,n,r,i,s,o;n=e.element.find("input.filter").val(),s=e.element.find("li[data-filter]"),o=[];for(r=0,i=s.length;r<i;r++)t=s[r],LiquidMetal.score(jQuery(t).data("filter"),n)===0?o.push(jQuery(t).hide()):o.push(jQuery(t).show());return o}),this.element.find("img[data-snippet]").on("dragstart",function(){return Mercury.snippet={name:jQuery(this).data("snippet"),hasOptions:jQuery(this).data("options")!==!1}})}}.call(this),function(){this.Mercury.dialogHandlers.style=function(){var e=this;return this.element.find("[data-class]").on("click",function(e){var t;return t=jQuery(e.target).data("class"),Mercury.trigger("action",{action:"style",value:t})})}}.call(this),function(){this.Mercury.modalHandlers.htmlEditor=function(){var e,t=this;return e=Mercury.region.content(null,!0,!1),this.element.find("textarea").val(e),this.element.find("form").on("submit",function(e){var n;return e.preventDefault(),n=t.element.find("textarea").val(),Mercury.trigger("action",{action:"replaceHTML",value:n}),t.hide()})}}.call(this),function(){this.Mercury.modalHandlers.insertCharacter=function(){var e=this;return this.element.find(".character").on("click",function(t){return Mercury.trigger("action",{action:"insertHTML",value:"&"+jQuery(t.target).attr("data-entity")+";"}),e.hide()})}}.call(this),function(){this.Mercury.modalHandlers.insertLink={initialize:function(){var e=this;return this.editing=!1,this.content=null,this.element.find(".control-label input").on("click",this.onLabelChecked),this.element.find(".controls .optional, .controls .required").on("focus",this.onInputFocused),this.element.find("#link_target").on("change",function(){return e.onChangeTarget()}),this.initializeForm(),this.element.find("form").on("submit",function(t){t.preventDefault(),e.validateForm();if(!e.valid){e.resize();return}return e.submitForm(),e.hide()})},initializeForm:function(){var e,t,n,r,i,s;this.fillExistingBookmarks();if(!Mercury.region||!Mercury.region.selection)return;s=Mercury.region.selection(),s.textContent&&this.element.find("#link_text").val(s.textContent()),s&&s.commonAncestor&&(e=s.commonAncestor(!0).closest("a")),s.htmlContent&&(r=/<img/.test(s.htmlContent()));if(!(r||e&&e.length))return!1;this.element.find("#link_text_container").hide(),r&&(this.content=s.htmlContent());if(!e||!e.length)return!1;this.editing=e,e.attr("href")&&e.attr("href").indexOf("#")===0?(t=this.element.find("#link_existing_bookmark"),t.val(e.attr("href").replace(/[^#]*#/,"")),t.closest(".control-group").find("input[type=radio]").prop("checked",!0)):this.element.find("#link_external_url").val(e.attr("href")),e.attr("name")&&(i=this.element.find("#link_new_bookmark"),i.val(e.attr("name")),i.closest(".control-group").find("input[type=radio]").prop("checked",!0)),e.attr("target")&&this.element.find("#link_target").val(e.attr("target"));if(e.attr("href")&&e.attr("href").indexOf("javascript:void")===0)return n=e.attr("href"),this.element.find("#link_external_url").val(n.match(/window.open\('([^']+)',/)[1]),this.element.find("#link_target").val("popup"),this.element.find("#link_popup_width").val(n.match(/width=(\d+),/)[1]),this.element.find("#link_popup_height").val(n.match(/height=(\d+),/)[1]),this.element.find("#popup_options").show()},fillExistingBookmarks:function(){var e,t,n,r,i,s;e=this.element.find("#link_existing_bookmark"),i=jQuery("a[name]",window.mercuryInstance.document),s=[];for(n=0,r=i.length;n<r;n++)t=i[n],s.push(e.append(jQuery("<option>",{value:jQuery(t).attr("name")}).text(jQuery(t).text())));return s},onLabelChecked:function(){var e;return e=jQuery(this).closest(".control-label").attr("for"),jQuery(this).closest(".control-group").find("#"+e).focus()},onInputFocused:function(){return jQuery(this).closest(".control-group").find("input[type=radio]").prop("checked",!0)},onChangeTarget:function(){return this.element.find(".link-target-options").hide(),this.element.find("#"+this.element.find("#link_target").val()+"_options").show(),this.resize(!0)},addInputError:function(e,t){return e.after('<span class="help-inline error-message">'+Mercury.I18n(t)+"</span>").closest(".control-group").addClass("error"),this.valid=!1},clearInputErrors:function(){return this.element.find(".control-group.error").removeClass("error").find(".error-message").remove(),this.valid=!0},validateForm:function(){var e,t;this.clearInputErrors(),t=this.element.find("input[name=link_type]:checked").val(),e=this.element.find("#link_"+t),e.val()||this.addInputError(e,"can't be blank");if(!this.editing&&!this.content){e=this.element.find("#link_text");if(!e.val())return this.addInputError(e,"can't be blank")}},submitForm:function(){var e,t,n,r,i,s;n=this.element.find("#link_text").val(),r=this.element.find("#link_target").val(),i=this.element.find("input[name=link_type]:checked").val();switch(i){case"existing_bookmark":t={href:"#"+this.element.find("#link_existing_bookmark").val()};break;case"new_bookmark":t={name:""+this.element.find("#link_new_bookmark").val()};break;default:t={href:this.element.find("#link_"+i).val()}}switch(r){case"popup":e={width:parseInt(this.element.find("#link_popup_width").val())||500,height:parseInt(this.element.find("#link_popup_height").val())||500,menubar:"no",toolbar:"no"},t.href="javascript:void(window.open('"+t.href+"', 'popup_window', '"+jQuery.param(e).replace(/&/g,",")+"'))";break;default:r&&(t.target=r)}return s={tagName:"a",attrs:t,content:this.content||n},this.editing?Mercury.trigger("action",{action:"replaceLink",value:s,node:this.editing.get(0)}):Mercury.trigger("action",{action:"insertLink",value:s})}}}.call(this),function(){this.Mercury.modalHandlers.insertMedia={initialize:function(){var e=this;return this.element.find(".control-label input").on("click",this.onLabelChecked),this.element.find(".controls .optional, .controls .required").on("focus",function(t){return e.onInputFocused($(t.target))}),this.focus("#media_image_url"),this.initializeForm(),this.element.find("form").on("submit",function(t){t.preventDefault(),e.validateForm();if(!e.valid){e.resize();return}return e.submitForm(),e.hide()})},initializeForm:function(){var e,t,n,r;if(!Mercury.region||!Mercury.region.selection)return;n=Mercury.region.selection();if(t=typeof n.is=="function"?n.is("img"):void 0)this.element.find("#media_image_url").val(t.attr("src")),this.element.find("#media_image_alignment").val(t.attr("align")),this.element.find("#media_image_float").val(t.attr("style")!=null?t.css("float"):""),this.focus("#media_image_url");if(e=typeof n.is=="function"?n.is("iframe"):void 0){r=e.attr("src");if(/^https?:\/\/www.youtube.com\//i.test(r))return this.element.find("#media_youtube_url").val(""+r.match(/^https?/)[0]+"://youtu.be/"+r.match(/\/embed\/(\w+)/)[1]),this.element.find("#media_youtube_width").val(e.width()),this.element.find("#media_youtube_height").val(e.height()),this.focus("#media_youtube_url");if(/^https?:\/\/player.vimeo.com\//i.test(r))return this.element.find("#media_vimeo_url").val(""+r.match(/^https?/)[0]+"://vimeo.com/"+r.match(/\/video\/(\w+)/)[1]),this.element.find("#media_vimeo_width").val(e.width()),this.element.find("#media_vimeo_height").val(e.height()),this.focus("#media_vimeo_url")}},focus:function(e){var t=this;return setTimeout(function(){return t.element.find(e).focus()},300)},onLabelChecked:function(){var e;return e=jQuery(this).closest(".control-label").attr("for"),jQuery(this).closest(".control-group").find("#"+e).focus()},onInputFocused:function(e){e.closest(".control-group").find("input[type=radio]").prop("checked",!0);if(e.closest(".media-options").length)return;return this.element.find(".media-options").hide(),this.element.find("#"+e.attr("id").replace("media_","")+"_options").show(),this.resize(!0)},addInputError:function(e,t){return e.after('<span class="help-inline error-message">'+Mercury.I18n(t)+"</span>").closest(".control-group").addClass("error"),this.valid=!1},clearInputErrors:function(){return this.element.find(".control-group.error").removeClass("error").find(".error-message").remove(),this.valid=!0},validateForm:function(){var e,t,n;this.clearInputErrors(),t=this.element.find("input[name=media_type]:checked").val(),e=this.element.find("#media_"+t);switch(t){case"youtube_url":n=this.element.find("#media_youtube_url").val();if(!/^https?:\/\/youtu.be\//.test(n))return this.addInputError(e,"is invalid");break;case"vimeo_url":n=this.element.find("#media_vimeo_url").val();if(!/^https?:\/\/vimeo.com\//.test(n))return this.addInputError(e,"is invalid");break;default:if(!e.val())return this.addInputError(e,"can't be blank")}},submitForm:function(){var e,t,n,r,i,s,o,u;s=this.element.find("input[name=media_type]:checked").val();switch(s){case"image_url":t={src:this.element.find("#media_image_url").val()};if(e=this.element.find("#media_image_alignment").val())t.align=e;if(r=this.element.find("#media_image_float").val())t.style="float: "+r+";";return Mercury.trigger("action",{action:"insertImage",value:t});case"youtube_url":return o=this.element.find("#media_youtube_url").val(),n=o.replace(/https?:\/\/youtu.be\//,""),i="http",/^https:/.test(o)&&(i="https"),u=jQuery("<iframe>",{width:parseInt(this.element.find("#media_youtube_width").val(),10)||560,height:parseInt(this.element.find("#media_youtube_height").val(),10)||349,src:""+i+"://www.youtube.com/embed/"+n+"?wmode=transparent",frameborder:0,allowfullscreen:"true"}),Mercury.trigger("action",{action:"insertHTML",value:u});case"vimeo_url":return o=this.element.find("#media_vimeo_url").val(),n=o.replace(/^https?:\/\/vimeo.com\//,""),i="http",/^https:/.test(o)&&(i="https"),u=jQuery("<iframe>",{width:parseInt(this.element.find("#media_vimeo_width").val(),10)||400,height:parseInt(this.element.find("#media_vimeo_height").val(),10)||225,src:""+i+"://player.vimeo.com/video/"+n+"?title=1&byline=1&portrait=0&color=ffffff",frameborder:0}),Mercury.trigger("action",{action:"insertHTML",value:u})}}}}.call(this),function(){this.Mercury.modalHandlers.insertSnippet=function(){var e=this;return this.element.find("form").on("submit",function(t){var n,r;return t.preventDefault(),n=e.element.find("form").serializeObject(),Mercury.snippet?(r=Mercury.snippet,r.setOptions(n),Mercury.snippet=null):r=Mercury.Snippet.create(e.options.snippetName,n),Mercury.trigger("action",{action:"insertSnippet",value:r}),e.hide()})}}.call(this),function(){this.Mercury.modalHandlers.insertTable={initialize:function(){var e=this;return this.table=this.element.find("#table_display table"),this.table.on("click",function(t){return e.onCellClick($(t.target))}),this.element.find("#table_alignment").on("change",function(){return e.setTableAlignment()}),this.element.find("#table_border").on("keyup",function(){return e.setTableBorder()}),this.element.find("#table_spacing").on("keyup",function(){return e.setTableCellSpacing()}),this.element.find("[data-action]").on("click",function(t){return t.preventDefault(),e.onActionClick(jQuery(t.
target).data("action"))}),this.selectFirstCell(),this.element.find("form").on("submit",function(t){return t.preventDefault(),e.submitForm(),e.hide()})},selectFirstCell:function(){var e;return e=this.table.find("td, th").first(),e.addClass("selected"),Mercury.tableEditor(this.table,e,"&nbsp;")},onCellClick:function(e){return this.cell=e,this.table=this.cell.closest("table"),this.table.find(".selected").removeAttr("class"),this.cell.addClass("selected"),Mercury.tableEditor(this.table,this.cell,"&nbsp;")},onActionClick:function(e){if(!e)return;return Mercury.tableEditor[e]()},setTableAlignment:function(){return this.table.attr({align:this.element.find("#table_alignment").val()})},setTableBorder:function(){var e;return e=parseInt(this.element.find("#table_border").val(),10),isNaN(e)?this.table.removeAttr("border"):this.table.attr({border:e})},setTableCellSpacing:function(){var e;return e=parseInt(this.element.find("#table_spacing").val(),10),isNaN(e)?this.table.removeAttr("cellspacing"):this.table.attr({cellspacing:e})},submitForm:function(){var e,t;return this.table.find(".selected").removeAttr("class"),this.table.find("td, th").html("<br/>"),e=jQuery("<div>").html(this.table).html(),t=e.replace(/^\s+|\n/gm,"").replace(/(<\/.*?>|<table.*?>|<tbody>|<tr>)/g,"$1\n"),Mercury.trigger("action",{action:"insertTable",value:t})}}}.call(this),function(){Mercury.onload&&Mercury.onload(),jQuery(window).trigger("mercury:loaded")}.call(this);